searchData={"items":[{"type":"module","title":"Bumblebee","doc":"Pre-trained `Axon` models for easy inference and boosted training.\n\nBumblebee provides state-of-the-art, configurable `Axon` models. On\ntop of that, it streamlines the process of loading pre-trained models\nby integrating with Hugging Face Hub and [🤗 Transformers](https://github.com/huggingface/transformers).","ref":"Bumblebee.html"},{"type":"module","title":"Usage - Bumblebee","doc":"You can load one of the supported models by specifying the model repository:\n\n    {:ok, model_info} = Bumblebee.load_model({:hf, \"google-bert/bert-base-uncased\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-uncased\"})\n\nThen you are ready to make predictions:\n\n    inputs = Bumblebee.apply_tokenizer(tokenizer, \"Hello Bumblebee!\")\n    outputs = Axon.predict(model_info.model, model_info.params, inputs)","ref":"Bumblebee.html#module-usage"},{"type":"module","title":"Tasks - Bumblebee","doc":"On top of bare models, Bumblebee provides a number of **\"servings\"**\nthat act as end-to-end pipelines for specific tasks.\n\n    serving = Bumblebee.Text.fill_mask(model_info, tokenizer)\n    Nx.Serving.run(serving, \"The capital of [MASK] is Paris.\")\n    #=> %{\n    #=>   predictions: [\n    #=>     %{score: 0.9279842972755432, token: \"france\"},\n    #=>     %{score: 0.008412551134824753, token: \"brittany\"},\n    #=>     %{score: 0.007433671969920397, token: \"algeria\"},\n    #=>     %{score: 0.004957548808306456, token: \"department\"},\n    #=>     %{score: 0.004369721747934818, token: \"reunion\"}\n    #=>   ]\n    #=> }\n\nAs you can see the **serving** takes care of pre-processing the\ntext input, runs the model and also post-processes its output into\nmore structured data. In the above example we `run` serving on the\nfly, however for production usage you can start serving as a process\nand it will automatically batch requests from multiple clients.\nProcessing inputs in batches is usually much more efficient, since\nit can take advantage of parallel capabilities of the target device,\nwhich is particularly relevant in case of GPU. For more details read\nthe `Nx.Serving` docs.\n\nFor more examples see the [Examples](examples.livemd) notebook.\n\n> #### Note {: .info}\n>\n> The models are generally large, so make sure to configure an efficient\n> `Nx` backend, such as `EXLA` or `Torchx`.","ref":"Bumblebee.html#module-tasks"},{"type":"module","title":"HuggingFace Hub - Bumblebee","doc":"HuggingFace Hub is a platform hosting models, datasets and demo apps (Spaces), all using Git repositories (with Git LFS for large files). For further information check out the [Hub documentation](https://huggingface.co/docs/hub/index) and explore the [model repositories](https://huggingface.co/models).","ref":"Bumblebee.html#module-huggingface-hub"},{"type":"module","title":"Models - Bumblebee","doc":"Model repositories are regular Git repositories, therefore they can store arbitrary files. However, most repositories store models saved using the Python [Transformers](https://github.com/huggingface/transformers) library. Bumblebee is an Elixir counterpart of Transformers and allows for importing those models, as long as they are implemented in Bumblebee.\n\nA repository in the Transformers format does not store an actual model, only the trained parameters and a configuration file. The configuration file specifies the model type (e.g. BERT) and high-level properties, such as the number layers and their size. The model implementation lives in the library code (both Transformers and Bumblebee). When loading a model, the library fetches the configuration and builds a matching model, then it fetches the trained parameters to pair them with the model. The key takeaway is that in order to use any given model, it needs to have an implementation in Bumblebee.","ref":"Bumblebee.html#module-models"},{"type":"module","title":"Model repository - Bumblebee","doc":"Here is a list of files commonly found in a repository following the Transformers format.\n\n  * `config.json` - model configuration, specifies the model type and model-specific options. You can think of this as a blueprint for how the model should be constructed\n\n  * `pytorch_model.bin` - raw model parameters (tensors) serialized from a PyTorch model using [PyTorch format](https://pytorch.org/docs/stable/generated/torch.save.html) (supported by Bumblebee)\n\n  * `model.safetensors` - raw model parameters (tensors) serialized from a PyTorch model using [Safetensors](https://github.com/huggingface/safetensors) (supported by Bumblebee)\n\n  * `flax_model.msgpack`, `tf_model.h5` - raw model parameters (tensors) serialized from Flax and Tensorflow models respectively (not supported by Bumblebee)\n\n  * `tokenizer.json`, `tokenizer_config.json` - tokenizer configuration, describes how to convert text input to model inputs (tensors). See [Tokenizer support](#tokenizer-support)\n\n  * `preprocessor_config.json` - featurizer configuration, describes how to convert real-world input (image, audio) to model inputs (tensors)\n\n  * `generation_config.json` - a set of configuration options specific to text generation, such as token sampling strategy and various constraints","ref":"Bumblebee.html#module-model-repository"},{"type":"module","title":"Model support - Bumblebee","doc":"As pointed out above, in order to load a model, the given model type must be implemented in Bumblebee. To find out whether the model is supported you can call `Bumblebee.load_model({:hf, \"model-repo\"})` or use [this tool](https://jonatanklosko-bumblebee-tools.hf.space/apps/repository-inspector) to run a number of checks against the repository.\n\nIf you prefer to poke around the code, open the `config.json` file in the model repository and copy the class name under `\"architectures\"`. Next, [search Bumblebee codebase](https://github.com/search?q=repo%3Aelixir-nx%2Fbumblebee+BertForMaskedLM&type=code) for that keyword. If you find a match, this indicates the model is supported.\n\nAlso note that certain repositories include multiple models in separate repositories, for example [`stabilityai/stable-diffusion-2`](https://huggingface.co/stabilityai/stable-diffusion-2). In such case use `Bumblebee.load_model({:hf, \"model-repo\", subdir: \"...\"})`.","ref":"Bumblebee.html#module-model-support"},{"type":"module","title":"Tokenizer support - Bumblebee","doc":"The Transformers library distinguishes two types of tokenizer implementations:\n\n  * \"slow tokenizer\" - a tokenizer implemented in Python and stored as `tokenizer_config.json` and a couple extra files\n\n  * \"fast tokenizer\" - a tokenizer implemented in Rust and stored in a single file - `tokenizer.json`\n\nBumblebee relies on the Rust implementations (through bindings to [Tokenizers](https://github.com/huggingface/tokenizers)) and therefore always requires the `tokenizer.json` file. Many repositories only include files for a \"slow tokenizer\". When you stumble upon such repository, there are two options you can try.\n\nFirst, if the repository is clearly a fine-tuned version of another model, you can look for `tokenizer.json` in the original model repository. For example, [`textattack/bert-base-uncased-yelp-polarity`](https://huggingface.co/textattack/bert-base-uncased-yelp-polarity) only includes `tokenizer_config.json`, but it is a fine-tuned version of [`bert-base-uncased`](https://huggingface.co/bert-base-uncased), which does include `tokenizer.json`. Consequently, you can safely load the model from `textattack/bert-base-uncased-yelp-polarity` and tokenizer from `bert-base-uncased`.\n\nOtherwise, the Transformers library includes conversion rules to load a \"slow tokenizer\" and convert it to a corresponding \"fast tokenizer\", which is possible in most cases. You can generate the `tokenizer.json` file using [this tool](https://jonatanklosko-bumblebee-tools.hf.space/apps/tokenizer-generator). Once successful, you can follow the steps to submit a PR adding `tokenizer.json` to the model repository. Note that you do not have to wait for the PR to be merged, instead you can copy commit SHA from the PR and load the tokenizer with `Bumblebee.load_tokenizer({:hf, \"model-repo\", revision: \"...\"})`.","ref":"Bumblebee.html#module-tokenizer-support"},{"type":"function","title":"Bumblebee.apply_featurizer/3","doc":"Featurizes `input` with the given featurizer.","ref":"Bumblebee.html#apply_featurizer/3"},{"type":"function","title":"Options - Bumblebee.apply_featurizer/3","doc":"* `:defn_options` - the options for JIT compilation. Note that\n    this is only relevant for featurizers implemented with Nx.\n    Defaults to `[]`","ref":"Bumblebee.html#apply_featurizer/3-options"},{"type":"function","title":"Examples - Bumblebee.apply_featurizer/3","doc":"featurizer = Bumblebee.configure(Bumblebee.Vision.ConvNextFeaturizer)\n    {:ok, img} = StbImage.read_file(path)\n    inputs = Bumblebee.apply_featurizer(featurizer, [img])","ref":"Bumblebee.html#apply_featurizer/3-examples"},{"type":"function","title":"Bumblebee.apply_tokenizer/3","doc":"Tokenizes and encodes `input` with the given tokenizer.","ref":"Bumblebee.html#apply_tokenizer/3"},{"type":"function","title":"Examples - Bumblebee.apply_tokenizer/3","doc":"tokenizer = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-uncased\"})\n    inputs = Bumblebee.apply_tokenizer(tokenizer, [\"The capital of France is [MASK].\"])","ref":"Bumblebee.html#apply_tokenizer/3-examples"},{"type":"function","title":"Bumblebee.build_model/2","doc":"Builds an `Axon` model according to the given specification.","ref":"Bumblebee.html#build_model/2"},{"type":"function","title":"Options - Bumblebee.build_model/2","doc":"* `:type` - either a type or `Axon.MixedPrecision` policy to apply\n    to the model","ref":"Bumblebee.html#build_model/2-options"},{"type":"function","title":"Example - Bumblebee.build_model/2","doc":"spec = Bumblebee.configure(Bumblebee.Vision.ResNet, architecture: :base, embedding_size: 128)\n    model = Bumblebee.build_model(spec)","ref":"Bumblebee.html#build_model/2-example"},{"type":"function","title":"Bumblebee.cache_dir/0","doc":"Returns the directory where downloaded files are stored.\n\nDefaults to the standard cache location for the given operating system.\nCan be configured with the `BUMBLEBEE_CACHE_DIR` environment variable.","ref":"Bumblebee.html#cache_dir/0"},{"type":"function","title":"Bumblebee.configure/2","doc":"Builds or updates a configuration object with the given options.\n\nExpects a configuration struct or a module supporting configuration.\nThese are usually configurable:\n\n  * model specification (`Bumblebee.ModelSpec`)\n\n  * featurizer (`Bumblebee.Featurizer`)\n\n  * scheduler (`Bumblebee.Scheduler`)","ref":"Bumblebee.html#configure/2"},{"type":"function","title":"Examples - Bumblebee.configure/2","doc":"To build a new configuration, pass a module:\n\n    featurizer = Bumblebee.configure(Bumblebee.Vision.ConvNextFeaturizer)\n    spec = Bumblebee.configure(Bumblebee.Vision.ResNet, architecture: :for_image_classification)\n\nSimilarly, you can update an existing configuration:\n\n    featurizer = Bumblebee.configure(featurizer, resize_method: :bilinear)\n    spec = Bumblebee.configure(spec, embedding_size: 128)","ref":"Bumblebee.html#configure/2-examples"},{"type":"function","title":"Bumblebee.load_featurizer/2","doc":"Loads featurizer from a model repository.","ref":"Bumblebee.html#load_featurizer/2"},{"type":"function","title":"Options - Bumblebee.load_featurizer/2","doc":"* `:module` - the featurizer module. By default it is inferred\n    from the preprocessor configuration file, if that is not possible,\n    it must be specified explicitly","ref":"Bumblebee.html#load_featurizer/2-options"},{"type":"function","title":"Examples - Bumblebee.load_featurizer/2","doc":"{:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"microsoft/resnet-50\"})","ref":"Bumblebee.html#load_featurizer/2-examples"},{"type":"function","title":"Bumblebee.load_generation_config/2","doc":"Loads generation config from a model repository.\n\nGeneration config includes a number of model-specific properties,\nso it is usually best to load the config and further configure,\nrather than building from scratch.\n\nSee `Bumblebee.Text.GenerationConfig` for all the available options.","ref":"Bumblebee.html#load_generation_config/2"},{"type":"function","title":"Options - Bumblebee.load_generation_config/2","doc":"* `:spec_module` - the model specification module. By default it\n    is inferred from the configuration file, if that is not possible,\n    it must be specified explicitly. Some models have extra options\n    related to generations and those are loaded into a separate\n    struct, stored under the `:extra_config` attribute","ref":"Bumblebee.html#load_generation_config/2-options"},{"type":"function","title":"Examples - Bumblebee.load_generation_config/2","doc":"{:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai-community/gpt2\"})\n\n    generation_config = Bumblebee.configure(generation_config, max_new_tokens: 10)","ref":"Bumblebee.html#load_generation_config/2-examples"},{"type":"function","title":"Bumblebee.load_model/2","doc":"Loads a pre-trained model from a model repository.\n\nThe model is downloaded and cached on your disk, use `cache_dir/0` to\nfind the location.","ref":"Bumblebee.html#load_model/2"},{"type":"function","title":"Parameters precision - Bumblebee.load_model/2","doc":"On GPUs computations that use numeric type of lower precision can\nbe faster and use less memory, while still providing valid results.\nYou can configure the model to use particular type by passing the\n`:type` option, such as `:bf16`.\n\nSome repositories have multiple variants of the parameter files\nwith different numeric types. The variant is usually indicated in\nthe file extension and you can load a particular file by specifying\n`:params_variant`, or `:params_filename`. Note however that this\ndoes not determine the numeric type used for inference. The file\ntype is relevant in context of download bandwidth and disk space.\nIf you want to use a lower precision for inference, make sure to\nalso specify `:type`.","ref":"Bumblebee.html#load_model/2-parameters-precision"},{"type":"function","title":"Options - Bumblebee.load_model/2","doc":"* `:spec` - the model specification to use when building the model.\n    By default the specification is loaded using `load_spec/2`\n\n  * `:spec_overrides` - additional options to configure the model\n    specification with. This is a shorthand for using `load_spec/2`,\n    `configure/2` and passing as `:spec`\n\n  * `:module` - the model specification module. By default it is\n    inferred from the configuration file, if that is not possible,\n    it must be specified explicitly\n\n  * `:architecture` - the model architecture, must be supported by\n    `:module`. By default it is inferred from the configuration file\n\n  * `:params_variant` - when specified, instead of loading parameters\n    from \" . \", loads from \" . . \"\n\n  * `:params_filename` - the file with the model parameters to be loaded\n\n  * `:log_params_diff` - whether to log missing, mismatched and unused\n    parameters. By default diff is logged only if some parameters\n    cannot be loaded\n\n  * `:backend` - the backend to allocate the tensors on. It is either\n    an atom or a tuple in the shape `{backend, options}`\n\n  * `:type` - either a type or `Axon.MixedPrecision` policy to apply\n    to the model. Passing this option automatically casts parameters\n    to the desired type","ref":"Bumblebee.html#load_model/2-options"},{"type":"function","title":"Examples - Bumblebee.load_model/2","doc":"By default the model type is inferred from configuration, so loading\nis as simple as:\n\n    {:ok, resnet} = Bumblebee.load_model({:hf, \"microsoft/resnet-50\"})\n    %{model: model, params: params, spec: spec} = resnet\n\nYou can explicitly specify a different architecture, in which case\nmatching parameters are still loaded:\n\n    {:ok, resnet} = Bumblebee.load_model({:hf, \"microsoft/resnet-50\"}, architecture: :base)\n\nTo further customize the model, you can also pass the specification:\n\n    {:ok, spec} = Bumblebee.load_spec({:hf, \"microsoft/resnet-50\"})\n    spec = Bumblebee.configure(spec, num_labels: 10)\n    {:ok, resnet} = Bumblebee.load_model({:hf, \"microsoft/resnet-50\"}, spec: spec)\n\nOr as a shorthand, you can pass just the options to override:\n\n    {:ok, resnet} =\n      Bumblebee.load_model({:hf, \"microsoft/resnet-50\"}, spec_overrides: [num_labels: 10])","ref":"Bumblebee.html#load_model/2-examples"},{"type":"function","title":"Bumblebee.load_scheduler/2","doc":"Loads scheduler from a model repository.","ref":"Bumblebee.html#load_scheduler/2"},{"type":"function","title":"Options - Bumblebee.load_scheduler/2","doc":"* `:module` - the scheduler module. By default it is inferred\n    from the scheduler configuration file, if that is not possible,\n    it must be specified explicitly","ref":"Bumblebee.html#load_scheduler/2-options"},{"type":"function","title":"Examples - Bumblebee.load_scheduler/2","doc":"{:ok, scheduler} =\n      Bumblebee.load_scheduler({:hf, \"CompVis/stable-diffusion-v1-4\", subdir: \"scheduler\"})","ref":"Bumblebee.html#load_scheduler/2-examples"},{"type":"function","title":"Bumblebee.load_spec/2","doc":"Loads model specification from a model repository.","ref":"Bumblebee.html#load_spec/2"},{"type":"function","title":"Options - Bumblebee.load_spec/2","doc":"* `:module` - the model specification module. By default it is\n    inferred from the configuration file, if that is not possible,\n    it must be specified explicitly\n\n  * `:architecture` - the model architecture, must be supported by\n    `:module`. By default it is inferred from the configuration file","ref":"Bumblebee.html#load_spec/2-options"},{"type":"function","title":"Examples - Bumblebee.load_spec/2","doc":"{:ok, spec} = Bumblebee.load_spec({:hf, \"microsoft/resnet-50\"})\n\nYou can explicitly specify a different architecture:\n\n    {:ok, spec} = Bumblebee.load_spec({:hf, \"microsoft/resnet-50\"}, architecture: :base)","ref":"Bumblebee.html#load_spec/2-examples"},{"type":"function","title":"Bumblebee.load_tokenizer/2","doc":"Loads tokenizer from a model repository.","ref":"Bumblebee.html#load_tokenizer/2"},{"type":"function","title":"Options - Bumblebee.load_tokenizer/2","doc":"* `:type` - the tokenizer type. By default it is inferred from\n    the configuration files, if that is not possible, it must be\n    specified explicitly","ref":"Bumblebee.html#load_tokenizer/2-options"},{"type":"function","title":"Examples - Bumblebee.load_tokenizer/2","doc":"{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-uncased\"})","ref":"Bumblebee.html#load_tokenizer/2-examples"},{"type":"function","title":"Bumblebee.scheduler_init/4","doc":"Initializes state for a new scheduler loop.\n\nReturns a pair of `{state, timesteps}`, where `state` is an opaque\ncontainer expected by `scheduler_step/4` and `timesteps` is a sequence\nof subsequent timesteps for model forward pass.\n\nNote that the number of `timesteps` may not match `num_steps` exactly.\n`num_steps` parameterizes sampling points, however depending on the\nmethod, sampling certain points may require multiple forward passes\nof the model and each element in `timesteps` corresponds to a single\nforward pass.","ref":"Bumblebee.html#scheduler_init/4"},{"type":"function","title":"Bumblebee.scheduler_step/4","doc":"Predicts sample at the previous timestep using the given scheduler.\n\nTakes the current `sample` and `prediction` (usually noise) returned\nby the model at the current timestep. Returns `{state, prev_sample}`,\nwhere `state` is the updated scheduler loop state and `prev_sample`\nis the predicted sample at the previous timestep.\n\nNote that some schedulers require several forward passes of the model\n(and a couple calls to this function) to make an actual prediction for\nthe previous sample.","ref":"Bumblebee.html#scheduler_step/4"},{"type":"type","title":"Bumblebee.model_info/0","doc":"A model together with its state and metadata.","ref":"Bumblebee.html#t:model_info/0"},{"type":"type","title":"Bumblebee.repository/0","doc":"A location to fetch model files from.\n\nCan be either:\n\n  * `{:hf, repository_id}` - the repository on Hugging Face. Options\n    may be passed as the third element:\n\n      * `:revision` - the specific model version to use, it can be\n        any valid git identifier, such as branch name, tag name, or\n        a commit hash\n\n      * `:cache_dir` - the directory to store the downloaded files\n        in. Defaults to the standard cache location for the given\n        operating system. You can also configure it globally by\n        setting the `BUMBLEBEE_CACHE_DIR` environment variable\n\n      * `:offline` - if `true`, only cached files are accessed and\n        missing files result in an error. You can also configure it\n        globally by setting the `BUMBLEBEE_OFFLINE` environment\n        variable to `true`\n\n      * `:auth_token` - the token to use as HTTP bearer authorization\n        for remote files\n\n      * `:subdir` - the directory within the repository where the\n        files are located\n\n  * `{:local, directory}` - the directory containing model files","ref":"Bumblebee.html#t:repository/0"},{"type":"module","title":"Bumblebee.Audio","doc":"High-level tasks related to audio processing.","ref":"Bumblebee.Audio.html"},{"type":"function","title":"Bumblebee.Audio.speech_to_text_whisper/5","doc":"Builds serving for speech-to-text generation with Whisper models.\n\nThe serving accepts `t:speech_to_text_whisper_input/0` and returns\n`t:speech_to_text_whisper_output/0`.\n\nThis serving always accepts a single input. A list of tensors is\ninterpreted as continuous chunks. To transcribe multiple inputs\nconcurrently use `Nx.Serving.batched_run/2`.","ref":"Bumblebee.Audio.html#speech_to_text_whisper/5"},{"type":"function","title":"Options - Bumblebee.Audio.speech_to_text_whisper/5","doc":"* `:chunk_num_seconds` - enables long-form transcription by splitting\n    the input into chunks of the given length. Models generally have\n    a limit on the input length, so by chunking we can feed smaller\n    bits into the model, then merge the individual outputs into a\n    single result at the end. By default chunking is disabled\n\n  * `:context_num_seconds` - specifies the amount of overlap between\n    chunks on both sides of split points. The context is effectively\n    discarded when merging the chunks at the end, but it improves\n    the results at the chunk edges. Note that the context is included\n    in the total `:chunk_num_seconds`. Defaults to 1/6 of\n    `:chunk_num_seconds`\n\n  * `:client_batch_size` - the number of input chunks the client\n    should gather and run at once. When streaming an input that is\n    already available (such as a file), you can use any value without\n    introducing a delay. A good default is to use the same value as\n    `opts[:compile][:batch_size]`. When streaming an input that is\n    being produced live, you may want to process chunks as soon as\n    they are available, in which case set this option to `1`. Defaults\n    to `opts[:compile][:batch_size]` if present, otherwise to `1`\n\n  * `:language` - the language of the speech, when known upfront.\n    Should be given as ISO alpha-2 code as string. By default no\n    language is assumed and it is inferred from the input\n\n  * `:task` - either of:\n\n      * `:transcribe` (default) - generate audio transcription in\n        the same language as the speech\n\n      * `:translate` - generate translation of the given speech in\n        English\n\n  * `:timestamps` - when set, the model predicts timestamps and each\n    annotated segment becomes an output chunk. Currently the only\n    supported value is `:segments`, the length of each segment is up\n    to the model\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`\n\n  * `:stream` - when `true`, the serving immediately returns a\n    stream that emits chunks as they are generated. Note that\n    when using streaming, only a single input can be given to the\n    serving. To process a batch, call the serving with each input\n    separately. Defaults to `false`","ref":"Bumblebee.Audio.html#speech_to_text_whisper/5-options"},{"type":"function","title":"Examples - Bumblebee.Audio.speech_to_text_whisper/5","doc":"{:ok, whisper} = Bumblebee.load_model({:hf, \"openai/whisper-tiny\"})\n    {:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"openai/whisper-tiny\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/whisper-tiny\"})\n    {:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai/whisper-tiny\"})\n\n    serving =\n      Bumblebee.Audio.speech_to_text_whisper(whisper, featurizer, tokenizer, generation_config,\n        defn_options: [compiler: EXLA]\n      )\n\n    output = Nx.Serving.run(serving, {:file, \"/path/to/audio.wav\"})\n    #=> %{\n    #=>   chunks: [\n    #=>     %{\n    #=>       text: \" There is a cat outside the window.\",\n    #=>       start_timestamp_seconds: nil,\n    #=>       end_timestamp_seconds: nil\n    #=>     }\n    #=>   ]\n    #=> }\n\n    text = output.chunks |> Enum.map_join(& &1.text) |> String.trim()\n    #=> \"There is a cat outside the window.\"\n\nAnd with timestamps:\n\n    serving =\n      Bumblebee.Audio.speech_to_text_whisper(whisper, featurizer, tokenizer, generation_config,\n        defn_options: [compiler: EXLA],\n        chunk_num_seconds: 30,\n        timestamps: :segments\n      )\n\n    Nx.Serving.run(serving, {:file, \"/path/to/colouredstars_08_mathers_128kb.mp3\"})\n    #=> %{\n    #=>   chunks: [\n    #=>     %{\n    #=>       text: \" Such an eight of colored stars, versions of fifty isiatic love poems by Edward Powis-Mathers.\",\n    #=>       start_timestamp_seconds: 0.0,\n    #=>       end_timestamp_seconds: 7.0\n    #=>     },\n    #=>     %{\n    #=>       text: \" This the revocs recording is in the public domain. Doubt. From the Japanese of Hori-Kawa,\",\n    #=>       start_timestamp_seconds: 7.0,\n    #=>       end_timestamp_seconds: 14.0\n    #=>     },\n    #=>     %{\n    #=>       text: \" will he be true to me that I do not know. But since the dawn, I have had as much disorder in my thoughts as in my black hair, and of doubt.\",\n    #=>       start_timestamp_seconds: 14.0,\n    #=>       end_timestamp_seconds: 27.0\n    #=>     }\n    #=>   ]\n    #=> }","ref":"Bumblebee.Audio.html#speech_to_text_whisper/5-examples"},{"type":"type","title":"Bumblebee.Audio.audio/0","doc":"A term representing audio.\n\nCan be either of:\n\n  * a 1-dimensional `Nx.Tensor` with audio samples\n\n  * an enumerable of 1-dimensional `Nx.Tensor`s, represending a\n    continuous stream of input. The tensors are not required to\n    have the same number of samples, the serving takes care of\n    accumulating and chunking the input as needed. This input is\n    only supported when chunking is enabled with the `:chunk_num_seconds`\n    option\n\n  * `{:file, path}` with path to an audio file (note that this\n    requires `ffmpeg` installed)","ref":"Bumblebee.Audio.html#t:audio/0"},{"type":"type","title":"Bumblebee.Audio.speech_to_text_whisper_chunk/0","doc":"","ref":"Bumblebee.Audio.html#t:speech_to_text_whisper_chunk/0"},{"type":"type","title":"Bumblebee.Audio.speech_to_text_whisper_input/0","doc":"","ref":"Bumblebee.Audio.html#t:speech_to_text_whisper_input/0"},{"type":"type","title":"Bumblebee.Audio.speech_to_text_whisper_output/0","doc":"","ref":"Bumblebee.Audio.html#t:speech_to_text_whisper_output/0"},{"type":"module","title":"Bumblebee.Diffusion.StableDiffusion","doc":"High-level tasks based on Stable Diffusion.","ref":"Bumblebee.Diffusion.StableDiffusion.html"},{"type":"function","title":"Bumblebee.Diffusion.StableDiffusion.text_to_image/6","doc":"Build serving for prompt-driven image generation.\n\nThe serving accepts `t:text_to_image_input/0` and returns `t:text_to_image_output/0`.\nA list of inputs is also supported.\n\nYou can specify `:safety_checker` model to automatically detect\nwhen a generated image is offensive or harmful and filter it out.","ref":"Bumblebee.Diffusion.StableDiffusion.html#text_to_image/6"},{"type":"function","title":"Options - Bumblebee.Diffusion.StableDiffusion.text_to_image/6","doc":"* `:safety_checker` - the safety checker model info map. When a\n    safety checker is used, each output entry has an additional\n    `:is_safe` property and unsafe images are automatically zeroed.\n    Make sure to also set `:safety_checker_featurizer`\n\n  * `:safety_checker_featurizer` - the featurizer to use to preprocess\n    the safety checker input images\n\n  * `:num_steps` - the number of denoising steps. More denoising\n    steps usually lead to higher image quality at the expense of\n    slower inference. Defaults to `50`\n\n  * `:num_images_per_prompt` - the number of images to generate for\n    each prompt. Defaults to `1`\n\n  * `:guidance_scale` - the scale used for classifier-free diffusion\n    guidance. Higher guidance scale makes the generated images more\n    closely reflect the text prompt. This parameter corresponds to\n    $\\omega$ in Equation (2) of the [Imagen paper](https://arxiv.org/pdf/2205.11487.pdf).\n    Defaults to `7.5`\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Diffusion.StableDiffusion.html#text_to_image/6-options"},{"type":"function","title":"Examples - Bumblebee.Diffusion.StableDiffusion.text_to_image/6","doc":"repository_id = \"CompVis/stable-diffusion-v1-4\"\n\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/clip-vit-large-patch14\"})\n    {:ok, clip} = Bumblebee.load_model({:hf, repository_id, subdir: \"text_encoder\"})\n    {:ok, unet} = Bumblebee.load_model({:hf, repository_id, subdir: \"unet\"})\n    {:ok, vae} = Bumblebee.load_model({:hf, repository_id, subdir: \"vae\"}, architecture: :decoder)\n    {:ok, scheduler} = Bumblebee.load_scheduler({:hf, repository_id, subdir: \"scheduler\"})\n    {:ok, featurizer} = Bumblebee.load_featurizer({:hf, repository_id, subdir: \"feature_extractor\"})\n    {:ok, safety_checker} = Bumblebee.load_model({:hf, repository_id, subdir: \"safety_checker\"})\n\n    serving =\n      Bumblebee.Diffusion.StableDiffusion.text_to_image(clip, unet, vae, tokenizer, scheduler,\n        num_steps: 20,\n        num_images_per_prompt: 2,\n        safety_checker: safety_checker,\n        safety_checker_featurizer: featurizer,\n        compile: [batch_size: 1, sequence_length: 60],\n        defn_options: [compiler: EXLA]\n      )\n\n    prompt = \"numbat in forest, detailed, digital art\"\n    Nx.Serving.run(serving, prompt)\n    #=> %{\n    #=>   results: [\n    #=>     %{\n    #=>       image: #Nx.Tensor<\n    #=>         u8[512][512][3]\n    #=>         ...\n    #=>       >,\n    #=>       is_safe: true\n    #=>     },\n    #=>     %{\n    #=>       image: #Nx.Tensor<\n    #=>         u8[512][512][3]\n    #=>         ...\n    #=>       >,\n    #=>       is_safe: true\n    #=>     }\n    #=>   ]\n    #=> }","ref":"Bumblebee.Diffusion.StableDiffusion.html#text_to_image/6-examples"},{"type":"type","title":"Bumblebee.Diffusion.StableDiffusion.text_to_image_input/0","doc":"","ref":"Bumblebee.Diffusion.StableDiffusion.html#t:text_to_image_input/0"},{"type":"type","title":"Bumblebee.Diffusion.StableDiffusion.text_to_image_output/0","doc":"","ref":"Bumblebee.Diffusion.StableDiffusion.html#t:text_to_image_output/0"},{"type":"type","title":"Bumblebee.Diffusion.StableDiffusion.text_to_image_result/0","doc":"","ref":"Bumblebee.Diffusion.StableDiffusion.html#t:text_to_image_result/0"},{"type":"module","title":"Bumblebee.Diffusion.StableDiffusionControlNet","doc":"High-level tasks based on Stable Diffusion with ControlNet.","ref":"Bumblebee.Diffusion.StableDiffusionControlNet.html"},{"type":"function","title":"Bumblebee.Diffusion.StableDiffusionControlNet.text_to_image/7","doc":"Build serving for prompt-driven image generation.\n\nThe serving accepts `t:text_to_image_input/0` and returns `t:text_to_image_output/0`.\nA list of inputs is also supported.\n\nYou can specify `:safety_checker` model to automatically detect\nwhen a generated image is offensive or harmful and filter it out.","ref":"Bumblebee.Diffusion.StableDiffusionControlNet.html#text_to_image/7"},{"type":"function","title":"Options - Bumblebee.Diffusion.StableDiffusionControlNet.text_to_image/7","doc":"* `:safety_checker` - the safety checker model info map. When a\n    safety checker is used, each output entry has an additional\n    `:is_safe` property and unsafe images are automatically zeroed.\n    Make sure to also set `:safety_checker_featurizer`\n\n  * `:safety_checker_featurizer` - the featurizer to use to preprocess\n    the safety checker input images\n\n  * `:num_steps` - the number of denoising steps. More denoising\n    steps usually lead to higher image quality at the expense of\n    slower inference. Defaults to `50`\n\n  * `:num_images_per_prompt` - the number of images to generate for\n    each prompt. Defaults to `1`\n\n  * `:guidance_scale` - the scale used for classifier-free diffusion\n    guidance. Higher guidance scale makes the generated images more\n    closely reflect the text prompt. This parameter corresponds to\n    $\\omega$ in Equation (2) of the [Imagen paper](https://arxiv.org/pdf/2205.11487.pdf).\n    Defaults to `7.5`\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Diffusion.StableDiffusionControlNet.html#text_to_image/7-options"},{"type":"function","title":"Examples - Bumblebee.Diffusion.StableDiffusionControlNet.text_to_image/7","doc":"repository_id = \"CompVis/stable-diffusion-v1-4\"\n\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/clip-vit-large-patch14\"})\n    {:ok, clip} = Bumblebee.load_model({:hf, repository_id, subdir: \"text_encoder\"})\n    {:ok, unet} = Bumblebee.load_model({:hf, repository_id, subdir: \"unet\"})\n    {:ok, controlnet} = Bumblebee.load_model({:hf, \"lllyasviel/sd-controlnet-scribble\"})\n    {:ok, vae} = Bumblebee.load_model({:hf, repository_id, subdir: \"vae\"}, architecture: :decoder)\n    {:ok, scheduler} = Bumblebee.load_scheduler({:hf, repository_id, subdir: \"scheduler\"})\n    {:ok, featurizer} = Bumblebee.load_featurizer({:hf, repository_id, subdir: \"feature_extractor\"})\n    {:ok, safety_checker} = Bumblebee.load_model({:hf, repository_id, subdir: \"safety_checker\"})\n\n    serving =\n      Bumblebee.Diffusion.StableDiffusionControlNet.text_to_image(\n        clip,\n        unet,\n        vae,\n        controlnet,\n        tokenizer,\n        scheduler,\n        num_steps: 20,\n        num_images_per_prompt: 2,\n        safety_checker: safety_checker,\n        safety_checker_featurizer: featurizer,\n        compile: [batch_size: 1, sequence_length: 60],\n        defn_options: [compiler: EXLA]\n      )\n\n    prompt = \"numbat in forest, detailed, digital art\"\n\n    # The conditioning image matching the given ControlNet condition,\n    # such as edges, pose or depth. Here we use a simple handcrafted\n    # tensor\n    conditioning =\n      Nx.tensor(\n        [List.duplicate(255, 8) ++ List.duplicate(0, 24)],\n        type: :u8\n      )\n      |> Nx.tile([256, 8, 3])\n      |> Nx.pad(0, [{192, 64, 0}, {192, 64, 0}, {0, 0, 0}])\n      |> Nx.transpose(axes: [1, 0, 2])\n\n    Nx.Serving.run(serving, %{prompt: prompt, conditioning: conditioning})\n    #=> %{\n    #=>   results: [\n    #=>     %{\n    #=>       image: #Nx.Tensor<\n    #=>         u8[512][512][3]\n    #=>         ...\n    #=>       >,\n    #=>       is_safe: true\n    #=>     },\n    #=>     %{\n    #=>       image: #Nx.Tensor<\n    #=>         u8[512][512][3]\n    #=>         ...\n    #=>       >,\n    #=>       is_safe: true\n    #=>     }\n    #=>   ]\n    #=> }","ref":"Bumblebee.Diffusion.StableDiffusionControlNet.html#text_to_image/7-examples"},{"type":"type","title":"Bumblebee.Diffusion.StableDiffusionControlNet.text_to_image_input/0","doc":"","ref":"Bumblebee.Diffusion.StableDiffusionControlNet.html#t:text_to_image_input/0"},{"type":"type","title":"Bumblebee.Diffusion.StableDiffusionControlNet.text_to_image_output/0","doc":"","ref":"Bumblebee.Diffusion.StableDiffusionControlNet.html#t:text_to_image_output/0"},{"type":"type","title":"Bumblebee.Diffusion.StableDiffusionControlNet.text_to_image_result/0","doc":"","ref":"Bumblebee.Diffusion.StableDiffusionControlNet.html#t:text_to_image_result/0"},{"type":"module","title":"Bumblebee.Text","doc":"High-level tasks related to text processing.","ref":"Bumblebee.Text.html"},{"type":"function","title":"Bumblebee.Text.fill_mask/3","doc":"Builds serving for the fill-mask task.\n\nThe serving accepts `t:fill_mask_input/0` and returns `t:fill_mask_output/0`.\nA list of inputs is also supported.\n\nIn the fill-mask task, the objective is to predict a masked word in\nthe text. The serving expects the input to have exactly one such word,\ndenoted as `[MASK]`.","ref":"Bumblebee.Text.html#fill_mask/3"},{"type":"function","title":"Options - Bumblebee.Text.fill_mask/3","doc":"* `:top_k` - the number of top predictions to include in the output.\n    If the configured value is higher than the number of labels, all\n    labels are returned. Defaults to `5`\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length.\n        A list can be given, in which case the serving compiles\n        a separate computation for each length and then inputs are\n        matched to the smallest bounding length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Text.html#fill_mask/3-options"},{"type":"function","title":"Examples - Bumblebee.Text.fill_mask/3","doc":"{:ok, bert} = Bumblebee.load_model({:hf, \"google-bert/bert-base-uncased\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-uncased\"})\n\n    serving = Bumblebee.Text.fill_mask(bert, tokenizer)\n\n    text = \"The capital of [MASK] is Paris.\"\n    Nx.Serving.run(serving, text)\n    #=> %{\n    #=>   predictions: [\n    #=>     %{score: 0.9279842972755432, token: \"france\"},\n    #=>     %{score: 0.008412551134824753, token: \"brittany\"},\n    #=>     %{score: 0.007433671969920397, token: \"algeria\"},\n    #=>     %{score: 0.004957548808306456, token: \"department\"},\n    #=>     %{score: 0.004369721747934818, token: \"reunion\"}\n    #=>   ]\n    #=> }","ref":"Bumblebee.Text.html#fill_mask/3-examples"},{"type":"function","title":"Bumblebee.Text.generation/4","doc":"Builds serving for prompt-driven text generation.\n\nThe serving accepts `t:generation_input/0` and returns `t:generation_output/0`.\nA list of inputs is also supported.","ref":"Bumblebee.Text.html#generation/4"},{"type":"function","title":"Options - Bumblebee.Text.generation/4","doc":"* `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length.\n        A list can be given, in which case the serving compiles\n        a separate computation for each length and then inputs are\n        matched to the smallest bounding length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`\n\n  * `:stream` - when `true`, the serving immediately returns a\n    stream that emits text chunks as they are generated. Note that\n    when using streaming, only a single input can be given to the\n    serving. To process a batch, call the serving with each input\n    separately. Defaults to `false`\n\n  * `:stream_done` - when `:stream` is enabled, this enables a final\n    event, after all chunks have been emitted. The event has the\n    shape `{:done, result}`, where `result` includes the same fields\n    as `t:generation_result/0`, except for `:text`, which has been\n    already streamed. Defaults to `false`","ref":"Bumblebee.Text.html#generation/4-options"},{"type":"function","title":"Examples - Bumblebee.Text.generation/4","doc":"{:ok, model_info} = Bumblebee.load_model({:hf, \"openai-community/gpt2\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai-community/gpt2\"})\n    {:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai-community/gpt2\"})\n    generation_config = Bumblebee.configure(generation_config, max_new_tokens: 15)\n\n    serving = Bumblebee.Text.generation(model_info, tokenizer, generation_config)\n\n    Nx.Serving.run(serving, \"Elixir is a functional\")\n    #=> %{\n    #=>   results: [\n    #=>     %{\n    #=>       text: \" programming language that is designed to be used in a variety of applications. It\",\n    #=>       token_summary: %{input: 5, output: 15, padding: 0}\n    #=>     }\n    #=>   ]\n    #=> }\n\nWe can stream the result by creating the serving with `stream: true`:\n\n    {:ok, model_info} = Bumblebee.load_model({:hf, \"openai-community/gpt2\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai-community/gpt2\"})\n    {:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai-community/gpt2\"})\n    generation_config = Bumblebee.configure(generation_config, max_new_tokens: 15)\n\n    serving = Bumblebee.Text.generation(model_info, tokenizer, generation_config, stream: true)\n\n    Nx.Serving.run(serving, \"Elixir is a functional\") |> Enum.to_list()\n    #=> [\" programming\", \" language\", \" that\", \" is\", \" designed\", \" to\", \" be\", \" used\", \" in\", \" a\",\n    #=>  \" variety\", \" of\", \" applications.\", \" It\"]","ref":"Bumblebee.Text.html#generation/4-examples"},{"type":"function","title":"Bumblebee.Text.question_answering/3","doc":"Builds serving for the question answering task.\n\nThe serving accepts `t:question_answering_input/0` and returns\n`t:question_answering_output/0`. A list of inputs is also supported.\n\nThe question answering task finds the most probable answer to a\nquestion within the given context text.","ref":"Bumblebee.Text.html#question_answering/3"},{"type":"function","title":"Options - Bumblebee.Text.question_answering/3","doc":"* `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size. Note\n        that the batch size refers to the number of prompts to classify,\n        while the model prediction is made for every combination of\n        prompt and label\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length.\n        A list can be given, in which case the serving compiles\n        a separate computation for each length and then inputs are\n        matched to the smallest bounding length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Text.html#question_answering/3-options"},{"type":"function","title":"Examples - Bumblebee.Text.question_answering/3","doc":"{:ok, roberta} = Bumblebee.load_model({:hf, \"deepset/roberta-base-squad2\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"FacebookAI/roberta-base\"})\n\n    serving = Bumblebee.Text.question_answering(roberta, tokenizer)\n\n    input = %{question: \"What's my name?\", context: \"My name is Sarah and I live in London.\"}\n    Nx.Serving.run(serving, input)\n    #=> %{results: [%{end: 16, score: 0.81039959192276, start: 11, text: \"Sarah\"}]}","ref":"Bumblebee.Text.html#question_answering/3-examples"},{"type":"function","title":"Bumblebee.Text.text_classification/3","doc":"Builds serving for text classification.\n\nThe serving accepts `t:text_classification_input/0` and returns\n`t:text_classification_output/0`. A list of inputs is also supported.","ref":"Bumblebee.Text.html#text_classification/3"},{"type":"function","title":"Options - Bumblebee.Text.text_classification/3","doc":"* `:top_k` - the number of top predictions to include in the output. If\n    the configured value is higher than the number of labels, all\n    labels are returned. Defaults to `5`\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length.\n        A list can be given, in which case the serving compiles\n        a separate computation for each length and then inputs are\n        matched to the smallest bounding length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:scores_function` - the function to use for converting logits to\n    scores. Should be one of `:softmax`, `:sigmoid`, or `:none`.\n    Defaults to `:softmax`\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Text.html#text_classification/3-options"},{"type":"function","title":"Examples - Bumblebee.Text.text_classification/3","doc":"{:ok, bertweet} = Bumblebee.load_model({:hf, \"finiteautomata/bertweet-base-sentiment-analysis\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"vinai/bertweet-base\"})\n\n    serving = Bumblebee.Text.text_classification(bertweet, tokenizer)\n\n    text = \"Cats are cute.\"\n    Nx.Serving.run(serving, text)\n    #=> %{\n    #=>   predictions: [\n    #=>     %{label: \"POS\", score: 0.9876555800437927},\n    #=>     %{label: \"NEU\", score: 0.010068908333778381},\n    #=>     %{label: \"NEG\", score: 0.002275536535307765}\n    #=>   ]\n    #=> }","ref":"Bumblebee.Text.html#text_classification/3-examples"},{"type":"function","title":"Bumblebee.Text.text_embedding/3","doc":"Builds serving for text embeddings.\n\nThe serving accepts `t:text_embedding_input/0` and returns\n`t:text_embedding_output/0`. A list of inputs is also supported.","ref":"Bumblebee.Text.html#text_embedding/3"},{"type":"function","title":"Options - Bumblebee.Text.text_embedding/3","doc":"* `:output_attribute` - the attribute of the model output map to\n    retrieve. When the output is a single tensor (rather than a map),\n    this option is ignored. Defaults to `:pooled_state`\n\n  * `:output_pool` - pooling to apply on top of the model output, in case\n    it is not already a pooled embedding. Supported values:\n\n      * `:mean_pooling` - performs a mean across all tokens\n\n      * `cls_token_pooling` - takes the embedding for the special CLS token.\n        Note that we currently assume that the CLS token is the first token\n        in the sequence\n\n    By default no pooling is applied\n\n  * `:embedding_processor` - a post-processing step to apply to the\n    embedding. Supported values: `:l2_norm`. By default the output is\n    returned as is\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length.\n        A list can be given, in which case the serving compiles\n        a separate computation for each length and then inputs are\n        matched to the smallest bounding length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Text.html#text_embedding/3-options"},{"type":"function","title":"Examples - Bumblebee.Text.text_embedding/3","doc":"{:ok, model_info} = Bumblebee.load_model({:hf, \"intfloat/e5-large\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"intfloat/e5-large\"})\n\n    serving = Bumblebee.Text.text_embedding(model_info, tokenizer)\n\n    text = \"query: Cats are cute.\"\n    Nx.Serving.run(serving, text)\n\n    #=> %{\n    #=>   embedding: #Nx.Tensor<\n    #=>     f32[1024]\n    #=>     EXLA.Backend \n    #=>     [-0.9789889454841614, -0.9814645051956177, -0.5015208125114441, 0.9867952466011047, 0.9917466640472412, -0.5557178258895874, -0.18618212640285492, 0.797040581703186, 0.8922086954116821, 0.7599573135375977, -0.16524426639080048, -0.8740050792694092, 0.9433475732803345, 0.7217797636985779, 0.9437620639801025, 0.4694959223270416, 0.40594056248664856, -0.20143413543701172, 0.7144518494606018, -0.8689796924591064, 0.94001305103302, 0.17163503170013428, -0.9896315932273865, 0.4455447494983673, 0.41139301657676697, 0.01911175064742565, -0.11275406181812286, -0.734498143196106, -0.6410953402519226, -0.628239095211029, -0.2570168673992157, 0.475137323141098, -0.7534396052360535, -0.9492156505584717, -0.17271563410758972, 0.9081271886825562, -0.4851466119289398, -0.9440935254096985, -0.20976334810256958, -0.684502899646759, -0.11581139266490936, 0.17509342730045319, 0.05547652021050453, 0.31042391061782837, 0.955132007598877, -0.35595986247062683, 0.016105204820632935, -0.3154579997062683, 0.9630348682403564, ...]\n    #=>   >\n    #=> }","ref":"Bumblebee.Text.html#text_embedding/3-examples"},{"type":"function","title":"Bumblebee.Text.token_classification/3","doc":"Builds serving for token classification.\n\nThe serving accepts `t:token_classification_input/0` and returns\n`t:token_classification_output/0`. A list of inputs is also supported.\n\nThis function can be used for tasks such as named entity recognition\n(NER) or part of speech tagging (POS).\n\nThe recognized entities can optionally be aggregated into groups\nbased on the given strategy.","ref":"Bumblebee.Text.html#token_classification/3"},{"type":"function","title":"Options - Bumblebee.Text.token_classification/3","doc":"* `:aggregation` - an optional strategy for aggregating adjacent\n    tokens. Token classification models output probabilities for\n    each possible token class. The aggregation strategy takes scores\n    for each token (which possibly represents subwords) and groups\n    tokens into phrases which are readily interpretable as entities\n    of a certain class. Supported aggregation strategies:\n\n      * `nil` (default) - corresponds to no aggregation and returns\n        the most likely label for each input token\n\n      * `:same` - groups adjacent tokens with the same label. If\n        the labels use beginning-inside-outside (BIO) tagging, the\n        boundaries are respected and the prefix is omitted in the\n        output labels\n\n      * `:word_first` - uses `:same` strategy except that word tokens\n        cannot end up with different labels. With this strategy word\n        gets the label of the first token of that word when there\n        is ambiguity. Note that this works only on word based models\n\n      * `:word_average` - uses `:same` strategy except that word tokens\n        cannot end up with different labels. With this strategy scores\n        are averaged across word tokens and then the maximum label\n        is taken. Note that this works only on word based models\n\n      * `:word_max` - uses `:same` strategy except that word tokens\n        cannot end up with different labels. With this strategy word\n        gets the label of the token with the maximum score. Note that\n        this works only on word based models\n\n  * `:ignored_labels` - the labels to ignore in the final output.\n    The labels should be specified without BIO prefix. Defaults to\n    `[\"O\"]`\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length.\n        A list can be given, in which case the serving compiles\n        a separate computation for each length and then inputs are\n        matched to the smallest bounding length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:scores_function` - the function to use for converting logits to\n    scores. Should be one of `:softmax`, `:sigmoid`, or `:none`.\n    Defaults to `:softmax`\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Text.html#token_classification/3-options"},{"type":"function","title":"Examples - Bumblebee.Text.token_classification/3","doc":"{:ok, bert} = Bumblebee.load_model({:hf, \"dslim/bert-base-NER\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-cased\"})\n\n    serving = Bumblebee.Text.token_classification(bert, tokenizer, aggregation: :same)\n\n    text = \"Rachel Green works at Ralph Lauren in New York City in the sitcom Friends\"\n    Nx.Serving.run(serving, text)\n    #=> %{\n    #=>  entities: [\n    #=>    %{end: 12, label: \"PER\", phrase: \"Rachel Green\", score: 0.9997024834156036, start: 0},\n    #=>    %{end: 34, label: \"ORG\", phrase: \"Ralph Lauren\", score: 0.9968731701374054, start: 22},\n    #=>    %{end: 51, label: \"LOC\", phrase: \"New York City\", score: 0.9995547334353129, start: 38},\n    #=>    %{end: 73, label: \"MISC\", phrase: \"Friends\", score: 0.6997143030166626, start: 66}\n    #=>  ]\n    #=>}","ref":"Bumblebee.Text.html#token_classification/3-examples"},{"type":"function","title":"Bumblebee.Text.translation/4","doc":"Builds serving for text translation.\n\nThe serving accepts `t:translation_input/0` and returns `t:translation_output/0`.\nA list of inputs is also supported.\n\nThis serving is an extension of `generation/4` that handles per-input\nlanguage configuration.\n\nNote that this serving is designed for multilingual models that\nrequire source/target language to be specified. Some text models are\ntrained for specific language pairs, others expect a command such as\n\"translate English to Spanish\", in such cases you most likely want\nto use `generation/4`.","ref":"Bumblebee.Text.html#translation/4"},{"type":"function","title":"Options - Bumblebee.Text.translation/4","doc":"See `generation/4` for available options.","ref":"Bumblebee.Text.html#translation/4-options"},{"type":"function","title":"Examples - Bumblebee.Text.translation/4","doc":"repository_id = \"facebook/nllb-200-distilled-600M\"\n\n    {:ok, model_info} = Bumblebee.load_model({:hf, repository_id})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, repository_id})\n    {:ok, generation_config} = Bumblebee.load_generation_config({:hf, repository_id})\n\n    serving = Bumblebee.Text.translation(model_info, tokenizer, generation_config)\n\n    text = \"The bank of the river is beautiful in spring\"\n\n    Nx.Serving.run(serving, %{\n      text: text,\n      source_language_token: \"eng_Latn\",\n      target_language_token: \"pol_Latn\"\n    })\n    #=> %{\n    #=>   results: [\n    #=>     %{\n    #=>       text: \"W wiosnę brzeg rzeki jest piękny\",\n    #=>       token_summary: %{input: 11, output: 13, padding: 0}\n    #=>     }\n    #=>   ]\n    #=> }","ref":"Bumblebee.Text.html#translation/4-examples"},{"type":"function","title":"Bumblebee.Text.zero_shot_classification/4","doc":"Builds serving for the zero-shot classification task.\n\nThe serving accepts `t:zero_shot_classification_input/0` and returns\n`t:zero_shot_classification_output/0`. A list of inputs is also\nsupported.\n\nThe zero-shot task predicts zero-shot labels for a given sequence by\nproposing each label as a premise-hypothesis pairing.","ref":"Bumblebee.Text.html#zero_shot_classification/4"},{"type":"function","title":"Options - Bumblebee.Text.zero_shot_classification/4","doc":"* `:top_k` - the number of top predictions to include in the output. If\n    the configured value is higher than the number of labels, all\n    labels are returned. Defaults to `5`\n\n  * `:hypothesis_template` - an arity-1 function which accepts a label\n    and returns a hypothesis. The default hypothesis format is: \"This example\n    is #{label}\".\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size. Note\n        that the batch size refers to the number of prompts to classify,\n        while the model prediction is made for every combination of\n        prompt and label\n\n      * `:sequence_length` - the maximum input sequence length. Input\n        sequences are always padded/truncated to match that length.\n        A list can be given, in which case the serving compiles\n        a separate computation for each length and then inputs are\n        matched to the smallest bounding length\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Text.html#zero_shot_classification/4-options"},{"type":"function","title":"Examples - Bumblebee.Text.zero_shot_classification/4","doc":"{:ok, model} = Bumblebee.load_model({:hf, \"facebook/bart-large-mnli\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"facebook/bart-large-mnli\"})\n\n    labels = [\"cooking\", \"traveling\", \"dancing\"]\n    zero_shot_serving = Bumblebee.Text.zero_shot_classification(model, tokenizer, labels)\n\n    output = Nx.Serving.run(zero_shot_serving, \"One day I will see the world\")\n    #=> %{\n    #=>   predictions: [\n    #=>     %{label: \"cooking\", score: 0.0070497458800673485},\n    #=>     %{label: \"traveling\", score: 0.985000491142273},\n    #=>     %{label: \"dancing\", score: 0.007949736900627613}\n    #=>   ]\n    #=> }","ref":"Bumblebee.Text.html#zero_shot_classification/4-examples"},{"type":"type","title":"Bumblebee.Text.fill_mask_input/0","doc":"","ref":"Bumblebee.Text.html#t:fill_mask_input/0"},{"type":"type","title":"Bumblebee.Text.fill_mask_output/0","doc":"","ref":"Bumblebee.Text.html#t:fill_mask_output/0"},{"type":"type","title":"Bumblebee.Text.fill_mask_prediction/0","doc":"","ref":"Bumblebee.Text.html#t:fill_mask_prediction/0"},{"type":"type","title":"Bumblebee.Text.generation_input/0","doc":"","ref":"Bumblebee.Text.html#t:generation_input/0"},{"type":"type","title":"Bumblebee.Text.generation_output/0","doc":"","ref":"Bumblebee.Text.html#t:generation_output/0"},{"type":"type","title":"Bumblebee.Text.generation_result/0","doc":"","ref":"Bumblebee.Text.html#t:generation_result/0"},{"type":"type","title":"Bumblebee.Text.question_answering_input/0","doc":"","ref":"Bumblebee.Text.html#t:question_answering_input/0"},{"type":"type","title":"Bumblebee.Text.question_answering_output/0","doc":"","ref":"Bumblebee.Text.html#t:question_answering_output/0"},{"type":"type","title":"Bumblebee.Text.question_answering_result/0","doc":"","ref":"Bumblebee.Text.html#t:question_answering_result/0"},{"type":"type","title":"Bumblebee.Text.text_classification_input/0","doc":"","ref":"Bumblebee.Text.html#t:text_classification_input/0"},{"type":"type","title":"Bumblebee.Text.text_classification_output/0","doc":"","ref":"Bumblebee.Text.html#t:text_classification_output/0"},{"type":"type","title":"Bumblebee.Text.text_classification_prediction/0","doc":"","ref":"Bumblebee.Text.html#t:text_classification_prediction/0"},{"type":"type","title":"Bumblebee.Text.text_embedding_input/0","doc":"","ref":"Bumblebee.Text.html#t:text_embedding_input/0"},{"type":"type","title":"Bumblebee.Text.text_embedding_output/0","doc":"","ref":"Bumblebee.Text.html#t:text_embedding_output/0"},{"type":"type","title":"Bumblebee.Text.token_classification_entity/0","doc":"A single entity label.\n\nNote that `start` and `end` indices are expressed in terms of UTF-8\nbytes.","ref":"Bumblebee.Text.html#t:token_classification_entity/0"},{"type":"type","title":"Bumblebee.Text.token_classification_input/0","doc":"","ref":"Bumblebee.Text.html#t:token_classification_input/0"},{"type":"type","title":"Bumblebee.Text.token_classification_output/0","doc":"","ref":"Bumblebee.Text.html#t:token_classification_output/0"},{"type":"type","title":"Bumblebee.Text.token_summary/0","doc":"","ref":"Bumblebee.Text.html#t:token_summary/0"},{"type":"type","title":"Bumblebee.Text.translation_input/0","doc":"","ref":"Bumblebee.Text.html#t:translation_input/0"},{"type":"type","title":"Bumblebee.Text.translation_output/0","doc":"","ref":"Bumblebee.Text.html#t:translation_output/0"},{"type":"type","title":"Bumblebee.Text.zero_shot_classification_input/0","doc":"","ref":"Bumblebee.Text.html#t:zero_shot_classification_input/0"},{"type":"type","title":"Bumblebee.Text.zero_shot_classification_output/0","doc":"","ref":"Bumblebee.Text.html#t:zero_shot_classification_output/0"},{"type":"type","title":"Bumblebee.Text.zero_shot_classification_prediction/0","doc":"","ref":"Bumblebee.Text.html#t:zero_shot_classification_prediction/0"},{"type":"module","title":"Bumblebee.Vision","doc":"High-level tasks related to vision.","ref":"Bumblebee.Vision.html"},{"type":"function","title":"Bumblebee.Vision.image_classification/3","doc":"Builds serving for image classification.\n\nThe serving accepts `t:image_classification_input/0` and returns\n`t:image_classification_output/0`. A list of inputs is also supported.","ref":"Bumblebee.Vision.html#image_classification/3"},{"type":"function","title":"Options - Bumblebee.Vision.image_classification/3","doc":"* `:top_k` - the number of top predictions to include in the output. If\n    the configured value is higher than the number of labels, all\n    labels are returned. Defaults to `5`\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:scores_function` - the function to use for converting logits to\n    scores. Should be one of `:softmax`, `:sigmoid`, or `:none`.\n    Defaults to `:softmax`\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Vision.html#image_classification/3-options"},{"type":"function","title":"Examples - Bumblebee.Vision.image_classification/3","doc":"{:ok, resnet} = Bumblebee.load_model({:hf, \"microsoft/resnet-50\"})\n    {:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"microsoft/resnet-50\"})\n\n    serving = Bumblebee.Vision.image_classification(resnet, featurizer)\n\n    image = StbImage.read_file!(path)\n    Nx.Serving.run(serving, image)\n    #=> %{\n    #=>   predictions: [\n    #=>     %{label: \"Egyptian cat\", score: 0.979233980178833},\n    #=>     %{label: \"tabby, tabby cat\", score: 0.00679466687142849},\n    #=>     %{label: \"tiger cat\", score: 0.005290505941957235},\n    #=>     %{label: \"lynx, catamount\", score: 0.004550771787762642},\n    #=>     %{label: \"Siamese cat, Siamese\", score: 1.1611092486418784e-4}\n    #=>   ]\n    #=> }","ref":"Bumblebee.Vision.html#image_classification/3-examples"},{"type":"function","title":"Bumblebee.Vision.image_embedding/3","doc":"Builds serving for image embeddings.\n\nThe serving accepts `t:image_embedding_input/0` and returns\n`t:image_embedding_output/0`. A list of inputs is also supported.","ref":"Bumblebee.Vision.html#image_embedding/3"},{"type":"function","title":"Options - Bumblebee.Vision.image_embedding/3","doc":"* `:output_attribute` - the attribute of the model output map to\n    retrieve. When the output is a single tensor (rather than a map),\n    this option is ignored. Defaults to `:pooled_state`\n\n  * `:embedding_processor` - a post-processing step to apply to the\n    embedding. Supported values: `:l2_norm`. By default the output is\n    returned as is\n\n  * `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Vision.html#image_embedding/3-options"},{"type":"function","title":"Examples - Bumblebee.Vision.image_embedding/3","doc":"{:ok, clip} =\n      Bumblebee.load_model({:hf, \"openai/clip-vit-base-patch32\"},\n        module: Bumblebee.Vision.ClipVision\n      )\n    {:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"openai/clip-vit-base-patch32\"})\n    serving = Bumblebee.Vision.image_embedding(clip, featurizer)\n    image = StbImage.read_file!(path)\n    Nx.Serving.run(serving, image)\n    #=> %{\n    #=>   embedding: #Nx.Tensor<\n    #=>     f32[768]\n    #=>     [-0.43403682112693787, 0.09786412119865417, -0.7233262062072754, -0.7707743644714355, 0.5550824403762817, -0.8923342227935791, 0.2687447965145111, 0.9633643627166748, 0.3520320951938629, 0.43195801973342896, 2.1438512802124023, -0.6542983651161194, -1.9736307859420776, 0.1611439287662506, 0.24555791914463043, 0.16985465586185455, 0.9012499451637268, 1.0657984018325806, 1.087411642074585, -0.5864712595939636, 0.3314521908760071, 0.8396108150482178, 0.3906593322753906, 0.13463366031646729, 0.2605385184288025, -0.07457947731018066, 0.4735124707221985, -0.41367805004119873, 0.18244807422161102, 1.4741417169570923, -5.807061195373535, 0.38920706510543823, 0.057687126100063324, 0.060301072895526886, 0.9680367708206177, 0.9670255184173584, 1.3876476287841797, -0.15498873591423035, -0.969764232635498, -0.38127464056015015, 0.05450016260147095, 2.2317700386047363, -0.07926210761070251, -0.11876475065946579, -1.5408644676208496, 0.7505669593811035, 0.9280041456222534, -0.3571934103965759, -1.1390857696533203, ...]\n    #=>   >\n    #=> }","ref":"Bumblebee.Vision.html#image_embedding/3-examples"},{"type":"function","title":"Bumblebee.Vision.image_to_text/5","doc":"Builds serving for image-to-text generation.\n\nThe serving accepts `t:image_to_text_input/0` and returns\n`t:image_to_text_output/0`. A list of inputs is also supported.","ref":"Bumblebee.Vision.html#image_to_text/5"},{"type":"function","title":"Options - Bumblebee.Vision.image_to_text/5","doc":"* `:compile` - compiles all computations for predefined input shapes\n    during serving initialization. Should be a keyword list with the\n    following keys:\n\n      * `:batch_size` - the maximum batch size of the input. Inputs\n        are optionally padded to always match this batch size\n\n    It is advised to set this option in production and also configure\n    a defn compiler using `:defn_options` to maximally reduce inference\n    time.\n\n  * `:defn_options` - the options for JIT compilation. Defaults to `[]`\n\n  * `:preallocate_params` - when `true`, explicitly allocates params\n    on the device configured by `:defn_options`. You may want to set\n    this option when using partitioned serving, to allocate params\n    on each of the devices. When using this option, you should first\n    load the parameters into the host. This can be done by passing\n    `backend: {EXLA.Backend, client: :host}` to `load_model/1` and friends.\n    Defaults to `false`","ref":"Bumblebee.Vision.html#image_to_text/5-options"},{"type":"function","title":"Examples - Bumblebee.Vision.image_to_text/5","doc":"{:ok, blip} = Bumblebee.load_model({:hf, \"Salesforce/blip-image-captioning-base\"})\n    {:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"Salesforce/blip-image-captioning-base\"})\n    {:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"Salesforce/blip-image-captioning-base\"})\n\n    {:ok, generation_config} =\n      Bumblebee.load_generation_config({:hf, \"Salesforce/blip-image-captioning-base\"})\n\n    serving =\n      Bumblebee.Vision.image_to_text(blip, featurizer, tokenizer, generation_config,\n        defn_options: [compiler: EXLA]\n      )\n\n    image = StbImage.read_file!(path)\n    Nx.Serving.run(serving, image)\n    #=> %{results: [%{text: \"a cat sitting on a chair\"}]}","ref":"Bumblebee.Vision.html#image_to_text/5-examples"},{"type":"type","title":"Bumblebee.Vision.image/0","doc":"A term representing an image.\n\nEither `Nx.Tensor` or a struct implementing `Nx.Container` and\nresolving to a tensor, with the following properties:\n\n  * HWC order\n  * RGB color channels\n  * alpha channel may be present, but it's usually stripped out\n  * integer type (`:s` or `:u`)","ref":"Bumblebee.Vision.html#t:image/0"},{"type":"type","title":"Bumblebee.Vision.image_classification_input/0","doc":"","ref":"Bumblebee.Vision.html#t:image_classification_input/0"},{"type":"type","title":"Bumblebee.Vision.image_classification_output/0","doc":"","ref":"Bumblebee.Vision.html#t:image_classification_output/0"},{"type":"type","title":"Bumblebee.Vision.image_classification_prediction/0","doc":"","ref":"Bumblebee.Vision.html#t:image_classification_prediction/0"},{"type":"type","title":"Bumblebee.Vision.image_embedding_input/0","doc":"","ref":"Bumblebee.Vision.html#t:image_embedding_input/0"},{"type":"type","title":"Bumblebee.Vision.image_embedding_output/0","doc":"","ref":"Bumblebee.Vision.html#t:image_embedding_output/0"},{"type":"type","title":"Bumblebee.Vision.image_to_text_input/0","doc":"","ref":"Bumblebee.Vision.html#t:image_to_text_input/0"},{"type":"type","title":"Bumblebee.Vision.image_to_text_output/0","doc":"","ref":"Bumblebee.Vision.html#t:image_to_text_output/0"},{"type":"type","title":"Bumblebee.Vision.image_to_text_result/0","doc":"","ref":"Bumblebee.Vision.html#t:image_to_text_result/0"},{"type":"module","title":"Bumblebee.Audio.Whisper","doc":"Whisper model family.","ref":"Bumblebee.Audio.Whisper.html"},{"type":"module","title":"Architectures - Bumblebee.Audio.Whisper","doc":"* `:base` - plain Whisper without any head on top\n\n  * `:for_conditional_generation` - Whisper with a language modeling\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Audio.Whisper.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Audio.Whisper","doc":"* `\"input_features\"` - `{batch_size, input_length, feature_size}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_features\"`, which can be specified\n    for more control over how `\"input_features\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_features\"` will be ignored.\n\n  * `\"decoder_input_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of decoder input sequence tokens in the vocabulary.\n\n  * `\"decoder_attention_mask\"` - `{batch_size, target_sequence_length}`\n\n    Mask indicating which decoder tokens to attend to. This is used\n    to ignore padding tokens, which are added when processing a batch\n    of sequences with different length.\n\n  * `\"decoder_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the decoder.\n\n  * `\"decoder_input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"decoder_input_ids\"`, which can be\n    specified for more control over how `\"decoder_input_ids\"` are\n    embedded than the model's internal embedding lookup. If\n    `\"decoder_input_embeddings\"` are present, then `\"decoder_input_ids\"`\n    will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"cross_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Audio.Whisper.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Audio.Whisper","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Audio.Whisper.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Audio.Whisper","doc":"* `:vocab_size` - the vocabulary size of the model. This corresponds to the number of distinct\n    tokens that can be represented by the decoder\n    . Defaults to `51865`\n\n  * `:feature_size` - the dimensionality of the input features. This corresponds to the number of Mel\n    bins in the preprocessed input\n    . Defaults to `80`\n\n  * `:encoder_max_positions` - the vocabulary size of the encoder position embedding. This corresponds to the maximum\n    sequence length of log-mel filter-bank features that the model can process\n    . Defaults to `1500`\n\n  * `:decoder_max_positions` - the vocabulary size of the decoder position embedding. This corresponds to the maximum\n    sequence length that this model can generate. Typically this is set to a large value just\n    in case, such as 512, 1024 or 2048\n    . Defaults to `448`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `1024`\n\n  * `:encoder_num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:decoder_num_blocks` - the number of Transformer blocks in the decoder. Defaults to `12`\n\n  * `:encoder_num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `16`\n\n  * `:decoder_num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `16`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:activation_dropout_rate` - the dropout rate for activations inside fully connected layers. Defaults to `0.0`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`","ref":"Bumblebee.Audio.Whisper.html#module-configuration"},{"type":"module","title":"Bumblebee.Diffusion.ControlNet","doc":"ControlNet model with two spatial dimensions and conditioning state.","ref":"Bumblebee.Diffusion.ControlNet.html"},{"type":"module","title":"Architectures - Bumblebee.Diffusion.ControlNet","doc":"* `:base` - the ControlNet model","ref":"Bumblebee.Diffusion.ControlNet.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Diffusion.ControlNet","doc":"* `\"sample\"` - `{batch_size, sample_size, sample_size, in_channels}`\n\n    Sample input with two spatial dimensions.\n\n  * `\"timestep\"` - `{}`\n\n    The timestep used to parameterize model behaviour in a multi-step\n    process, such as diffusion.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    The conditioning state (context) to use with cross-attention.\n\n  * `\"conditioning\"` - `{batch_size, conditioning_size, conditioning_size, 3}`\n\n    The conditioning spatial input.","ref":"Bumblebee.Diffusion.ControlNet.html#module-inputs"},{"type":"module","title":"Configuration - Bumblebee.Diffusion.ControlNet","doc":"* `:sample_size` - the size of the input spatial dimensions. Defaults to `64`\n\n  * `:in_channels` - the number of channels in the input. Defaults to `4`\n\n  * `:out_channels` - the number of channels in the output. Defaults to `4`\n\n  * `:embedding_flip_sin_to_cos` - whether to flip the sin to cos in the sinusoidal timestep embedding. Defaults to `true`\n\n  * `:embedding_frequency_correction_term` - controls the frequency formula in the timestep sinusoidal embedding. The frequency is computed\n    as $\\\\omega_i = \\\\frac{1}{10000^{\\\\frac{i}{n - s}}}$, for $i \\\\in \\\\{0, ..., n-1\\\\}$, where $n$\n    is half of the embedding size and $s$ is the shift. Historically, certain implementations of\n    sinusoidal embedding used $s=0$, while others used $s=1$\n    . Defaults to `0`\n\n  * `:hidden_sizes` - the dimensionality of hidden layers in each upsample/downsample block. Defaults to `[320, 640, 1280, 1280]`\n\n  * `:depth` - the number of residual blocks in each upsample/downsample block. Defaults to `2`\n\n  * `:down_block_types` - a list of downsample block types. The supported blocks are: `:down_block`, `:cross_attention_down_block`. Defaults to `[:cross_attention_down_block, :cross_attention_down_block, :cross_attention_down_block, :down_block]`\n\n  * `:up_block_types` - a list of upsample block types. The supported blocks are: `:up_block`, `:cross_attention_up_block`. Defaults to `[:up_block, :cross_attention_up_block, :cross_attention_up_block, :cross_attention_up_block]`\n\n  * `:downsample_padding` - the padding to use in the downsample convolution. Defaults to `[{1, 1}, {1, 1}]`\n\n  * `:mid_block_scale_factor` - the scale factor to use for the mid block. Defaults to `1`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer. Optionally can be a list with one number per block. Defaults to `8`\n\n  * `:cross_attention_size` - the dimensionality of the cross attention features. Defaults to `1024`\n\n  * `:use_linear_projection` - whether the input/output projection of the transformer block should be linear or convolutional. Defaults to `false`\n\n  * `:activation` - the activation function. Defaults to `:silu`\n\n  * `:group_norm_num_groups` - the number of groups used by the group normalization layers. Defaults to `32`\n\n  * `:group_norm_epsilon` - the epsilon used by the group normalization layers. Defaults to `1.0e-5`\n\n  * `:conditioning_embedding_hidden_sizes` - the dimensionality of hidden layers in the conditioning input embedding. Defaults to `[16, 32, 96, 256]`","ref":"Bumblebee.Diffusion.ControlNet.html#module-configuration"},{"type":"module","title":"Bumblebee.Diffusion.StableDiffusion.SafetyChecker","doc":"A CLIP-based model for detecting unsafe image content.\n\nThis model is designed primarily to check images generated using\nStable Diffusion.","ref":"Bumblebee.Diffusion.StableDiffusion.SafetyChecker.html"},{"type":"module","title":"Architectures - Bumblebee.Diffusion.StableDiffusion.SafetyChecker","doc":"* `:base` - the base safety detection model","ref":"Bumblebee.Diffusion.StableDiffusion.SafetyChecker.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Diffusion.StableDiffusion.SafetyChecker","doc":"* `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.","ref":"Bumblebee.Diffusion.StableDiffusion.SafetyChecker.html#module-inputs"},{"type":"module","title":"Configuration - Bumblebee.Diffusion.StableDiffusion.SafetyChecker","doc":"* `:clip_spec` - the specification of the CLIP model. See `Bumblebee.Multimodal.Clip` for details","ref":"Bumblebee.Diffusion.StableDiffusion.SafetyChecker.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Diffusion.StableDiffusion.SafetyChecker","doc":"* [CompVis/stable-diffusion-v1-4](https://huggingface.co/CompVis/stable-diffusion-v1-4#safety-module)","ref":"Bumblebee.Diffusion.StableDiffusion.SafetyChecker.html#module-references"},{"type":"module","title":"Bumblebee.Diffusion.UNet2DConditional","doc":"U-Net model with two spatial dimensions and conditioning state.","ref":"Bumblebee.Diffusion.UNet2DConditional.html"},{"type":"module","title":"Architectures - Bumblebee.Diffusion.UNet2DConditional","doc":"* `:base` - the U-Net model","ref":"Bumblebee.Diffusion.UNet2DConditional.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Diffusion.UNet2DConditional","doc":"* `\"sample\"` - `{batch_size, sample_size, sample_size, in_channels}`\n\n    Sample input with two spatial dimensions.\n\n  * `\"timestep\"` - `{}`\n\n    The timestep used to parameterize model behaviour in a multi-step\n    process, such as diffusion.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    The conditioning state (context) to use with cross-attention.\n\n  * `\"additional_down_block_states\"`\n\n    Optional outputs matching the structure of down blocks, added as\n    part of the encoder-decoder skip connections.\n\n  * `\"additional_mid_block_state\"`\n\n    Optional output added to the mid block result.","ref":"Bumblebee.Diffusion.UNet2DConditional.html#module-inputs"},{"type":"module","title":"Configuration - Bumblebee.Diffusion.UNet2DConditional","doc":"* `:sample_size` - the size of the input spatial dimensions. Defaults to `32`\n\n  * `:in_channels` - the number of channels in the input. Defaults to `4`\n\n  * `:out_channels` - the number of channels in the output. Defaults to `4`\n\n  * `:center_input_sample` - whether to center the input sample. Defaults to `false`\n\n  * `:embedding_flip_sin_to_cos` - whether to flip the sin to cos in the sinusoidal timestep embedding. Defaults to `true`\n\n  * `:embedding_frequency_correction_term` - controls the frequency formula in the timestep sinusoidal embedding. The frequency is computed\n    as $\\\\omega_i = \\\\frac{1}{10000^{\\\\frac{i}{n - s}}}$, for $i \\\\in \\\\{0, ..., n-1\\\\}$, where $n$\n    is half of the embedding size and $s$ is the shift. Historically, certain implementations of\n    sinusoidal embedding used $s=0$, while others used $s=1$\n    . Defaults to `0`\n\n  * `:hidden_sizes` - the dimensionality of hidden layers in each upsample/downsample block. Defaults to `[320, 640, 1280, 1280]`\n\n  * `:depth` - the number of residual blocks in each upsample/downsample block. Defaults to `2`\n\n  * `:down_block_types` - a list of downsample block types. The supported blocks are: `:down_block`, `:cross_attention_down_block`. Defaults to `[:cross_attention_down_block, :cross_attention_down_block, :cross_attention_down_block, :down_block]`\n\n  * `:up_block_types` - a list of upsample block types. The supported blocks are: `:up_block`, `:cross_attention_up_block`. Defaults to `[:up_block, :cross_attention_up_block, :cross_attention_up_block, :cross_attention_up_block]`\n\n  * `:downsample_padding` - the padding to use in the downsample convolution. Defaults to `[{1, 1}, {1, 1}]`\n\n  * `:mid_block_scale_factor` - the scale factor to use for the mid block. Defaults to `1`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer. Optionally can be a list with one number per block. Defaults to `8`\n\n  * `:cross_attention_size` - the dimensionality of the cross attention features. Defaults to `1280`\n\n  * `:use_linear_projection` - whether the input/output projection of the transformer block should be linear or convolutional. Defaults to `false`\n\n  * `:activation` - the activation function. Defaults to `:silu`\n\n  * `:group_norm_num_groups` - the number of groups used by the group normalization layers. Defaults to `32`\n\n  * `:group_norm_epsilon` - the epsilon used by the group normalization layers. Defaults to `1.0e-5`","ref":"Bumblebee.Diffusion.UNet2DConditional.html#module-configuration"},{"type":"module","title":"Bumblebee.Diffusion.VaeKl","doc":"Variational autoencoder (VAE) with Kullback–Leibler divergence (KL) loss.","ref":"Bumblebee.Diffusion.VaeKl.html"},{"type":"module","title":"Architectures - Bumblebee.Diffusion.VaeKl","doc":"* `:base` - the entire VAE model\n\n  * `:encoder` - just the encoder part of the base model\n\n  * `:decoder` - just the decoder part of the base model","ref":"Bumblebee.Diffusion.VaeKl.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Diffusion.VaeKl","doc":"* `\"sample\"` - `{batch_size, sample_size, sample_size, in_channels}`\n\n    Sample input with two spatial dimensions. Note that in case of\n    the `:decoder` model, the input usually has lower dimensionality.\n\n  * `\"sample_posterior\"` - `{}`\n\n    When `true`, the decoder input is sampled from the encoder output\n    distribution. Otherwise the distribution mode value is used instead.\n    This input is only relevant for the `:base` model. Defaults to `false`.","ref":"Bumblebee.Diffusion.VaeKl.html#module-inputs"},{"type":"module","title":"Configuration - Bumblebee.Diffusion.VaeKl","doc":"* `:sample_size` - the size of the input spatial dimensions. Defaults to `32`\n\n  * `:in_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:out_channels` - the number of channels in the output. Defaults to `3`\n\n  * `:latent_channels` - the number of channels in the latent space. Defaults to `4`\n\n  * `:hidden_sizes` - the dimensionality of hidden layers in each upsample/downsample block. Defaults to `~c\"@\"`\n\n  * `:depth` - the number of residual blocks in each upsample/downsample block. Defaults to `1`\n\n  * `:down_block_types` - a list of downsample block types. Currently the only supported type is `:down_block`. Defaults to `[:down_block]`\n\n  * `:up_block_types` - a list of upsample block types. Currently the only supported type is `:up_block`. Defaults to `[:up_block]`\n\n  * `:activation` - the activation function. Defaults to `:silu`","ref":"Bumblebee.Diffusion.VaeKl.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Diffusion.VaeKl","doc":"* [Auto-Encoding Variational Bayes](https://arxiv.org/abs/1312.6114)","ref":"Bumblebee.Diffusion.VaeKl.html#module-references"},{"type":"module","title":"Bumblebee.Multimodal.Blip","doc":"The BLIP model for text-image similarity.","ref":"Bumblebee.Multimodal.Blip.html"},{"type":"module","title":"Architectures - Bumblebee.Multimodal.Blip","doc":"* `:for_conditional_generation` - BLIP model with a language\n    modeling head","ref":"Bumblebee.Multimodal.Blip.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Multimodal.Blip","doc":"* `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.\n\n  * `\"decoder_input_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of decoder input sequence tokens in the vocabulary. If not\n    present and `\"input_ids\"` is, it will be generated by shifting\n    each token in `\"input_ids\"` to the right once.\n\n  * `\"decoder_attention_mask\"` - `{batch_size, target_sequence_length}`\n\n    Mask indicating which decoder tokens to attend to. This is used\n    to ignore padding tokens, which are added when processing a batch\n    of sequences with different length.\n\n  * `\"decoder_position_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of positions of each decoder input sequence tokens in\n    the position embeddings.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the image encoding process and use this value\n    directly for cross-attentions in the text decoder.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Multimodal.Blip.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Multimodal.Blip","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Multimodal.Blip.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Multimodal.Blip","doc":"* `:text_spec` - the specification of the text model. See `Bumblebee.Text.BlipText` for details\n\n  * `:vision_spec` - the specification of the vision model. See `Bumblebee.Vision.BlipVision` for details\n\n  * `:projection_size` - the dimensionality of text and vision projection layers. Defaults to `512`\n\n  * `:logit_scale_initial_value` - the initial value for the scaling layer used to scale similarity logits. Defaults to `2.6592`","ref":"Bumblebee.Multimodal.Blip.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Multimodal.Blip","doc":"* [BLIP: Bootstrapping Language-Image Pre-training for Unified Vision-Language Understanding and Generation](https://arxiv.org/abs/2201.12086)","ref":"Bumblebee.Multimodal.Blip.html#module-references"},{"type":"module","title":"Bumblebee.Multimodal.Clip","doc":"The CLIP model for text-image similarity.","ref":"Bumblebee.Multimodal.Clip.html"},{"type":"module","title":"Architectures - Bumblebee.Multimodal.Clip","doc":"* `:base` - the base CLIP model","ref":"Bumblebee.Multimodal.Clip.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Multimodal.Clip","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.","ref":"Bumblebee.Multimodal.Clip.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Multimodal.Clip","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Multimodal.Clip.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Multimodal.Clip","doc":"* `:text_spec` - the specification of the text model. See `Bumblebee.Text.ClipText` for details\n\n  * `:vision_spec` - the specification of the vision model. See `Bumblebee.Vision.ClipVision` for details\n\n  * `:projection_size` - the dimensionality of text and vision projection layers. Defaults to `512`\n\n  * `:logit_scale_initial_value` - the initial value for the scaling layer used to scale similarity logits. Defaults to `2.6592`","ref":"Bumblebee.Multimodal.Clip.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Multimodal.Clip","doc":"* [CLIP: Connecting Text and Images](https://openai.com/blog/clip)\n\n  * [Learning Transferable Visual Models From Natural Language Supervision](https://arxiv.org/abs/2103.00020)","ref":"Bumblebee.Multimodal.Clip.html#module-references"},{"type":"module","title":"Bumblebee.Multimodal.LayoutLm","doc":"LayoutLM Model family.","ref":"Bumblebee.Multimodal.LayoutLm.html"},{"type":"module","title":"Architectures - Bumblebee.Multimodal.LayoutLm","doc":"* `:base` - plain LayoutLM without any head on top\n\n  * `:for_masked_language_modeling` - LayoutLM with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - LayoutLM with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - LayoutLM with a token classification\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_question_answering` - LayoutLM with a span classification head.\n    The head returns logits for the span start and end positions","ref":"Bumblebee.Multimodal.LayoutLm.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Multimodal.LayoutLm","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"token_type_ids\"` - `{batch_size, sequence_length}`\n\n    Mask distinguishing groups in the input sequence. This is used\n    in when the input sequence is a semantically a pair of sequences.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"bounding_box\"` - `{batch_size, sequence_length, 4}`\n\n  Bounding boxes of each input sequence token. Each bounding box is\n  `{x0, y0, x1, y1}` where `{x0, y0}` is the upper left corner and\n  `{x1, y1}` is the lower right corner.","ref":"Bumblebee.Multimodal.LayoutLm.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Multimodal.LayoutLm","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Multimodal.LayoutLm.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Multimodal.LayoutLm","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `30522`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `1024`\n\n  * `:max_spatial_positions` - the maximum value of the spatial position embedding. Typically this is set to a large value\n    just in case, such as 512, 1024, or 2048\n    . Defaults to `1024`\n\n  * `:max_token_types` - the vocabulary size of the token type embedding (also referred to as segment embedding).\n    This corresponds to how many different token groups can be distinguished in the input\n    . Defaults to `2`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `12`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the decoder.\n    If not specified, defaults to 4 times `:hidden_size`\n    . Defaults to `3072`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.1`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. If not specified, the value of `:dropout_rate` is used instead\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Multimodal.LayoutLm.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Multimodal.LayoutLm","doc":"* [LayoutLM: LayoutLM: Pre-training of Text and Layout for Document Image Understanding](https://arxiv.org/abs/1912.13318)","ref":"Bumblebee.Multimodal.LayoutLm.html#module-references"},{"type":"module","title":"Bumblebee.Text.Albert","doc":"ALBERT model family.","ref":"Bumblebee.Text.Albert.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Albert","doc":"* `:base` - plain ALBERT without any head on top\n\n  * `:for_masked_language_modeling` - ALBERT with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - ALBERT with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - ALBERT with a token classification\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_question_answering` - ALBERT with a span classification head.\n    The head returns logits for the span start and end positions\n\n  * `:for_multiple_choice` - ALBERT with a multiple choice prediction\n    head. Each input in the batch consists of several sequences to\n    choose from and the model returns logits corresponding to those\n    choices\n\n  * `:for_pre_training` - ALBERT with both MLM and NSP heads as done\n    during the pre-training","ref":"Bumblebee.Text.Albert.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Albert","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"token_type_ids\"` - `{batch_size, sequence_length}`\n\n    Mask distinguishing groups in the input sequence. This is used\n    in when the input sequence is a semantically a pair of sequences.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.","ref":"Bumblebee.Text.Albert.html#module-inputs"},{"type":"module","title":"Exceptions - Bumblebee.Text.Albert","doc":"The `:for_multiple_choice` model accepts groups of sequences, so the\nexpected sequence shape is `{batch_size, num_choices, sequence_length}`.","ref":"Bumblebee.Text.Albert.html#module-exceptions"},{"type":"module","title":"Global layer options - Bumblebee.Text.Albert","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Albert.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Albert","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `30000`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `512`\n\n  * `:max_token_types` - the vocabulary size of the token type embedding (also referred to as segment embedding).\n    This corresponds to how many different token groups can be distinguished in the input\n    . Defaults to `2`\n\n  * `:embedding_size` - the dimensionality of all input embeddings. Defaults to `128`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of blocks in the encoder. Note that each block contains `:block_depth`\n    Transformer blocks\n    . Defaults to `12`\n\n  * `:num_groups` - the number of groups of encoder blocks. Parameters in the same group are shared. Defaults to `1`\n\n  * `:block_depth` - the number of Transformer blocks in each encoder block. Defaults to `1`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `16384`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.0`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. If not specified, the value of `:dropout_rate` is used instead\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Albert.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Text.Albert","doc":"* [ALBERT: A Lite BERT for Self-supervised Learning of Language Representations](https://arxiv.org/abs/1909.11942)","ref":"Bumblebee.Text.Albert.html#module-references"},{"type":"module","title":"Bumblebee.Text.Bart","doc":"BART model family.","ref":"Bumblebee.Text.Bart.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Bart","doc":"* `:base` - plain BART without any head on top\n\n  * `:for_causal_language_modeling` - BART with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_conditional_generation` - BART with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - BART with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_question_answering` - BART with a span classification head.\n    The head returns logits for the span start and end positions","ref":"Bumblebee.Text.Bart.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Bart","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"decoder_input_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of decoder input sequence tokens in the vocabulary. If not\n    present and `\"input_ids\"` is, it will be generated by shifting\n    each token in `\"input_ids\"` to the right once.\n\n  * `\"decoder_attention_mask\"` - `{batch_size, target_sequence_length}`\n\n    Mask indicating which decoder tokens to attend to. This is used\n    to ignore padding tokens, which are added when processing a batch\n    of sequences with different length.\n\n  * `\"decoder_position_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of positions of each decoder input sequence tokens in\n    the position embeddings.\n\n  * `\"decoder_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the decoder.\n\n  * `\"decoder_input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"decoder_input_ids\"`, which can be\n    specified for more control over how `\"decoder_input_ids\"` are\n    embedded than the model's internal embedding lookup. If\n    `\"decoder_input_embeddings\"` are present, then `\"decoder_input_ids\"`\n    will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"cross_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Bart.html#module-inputs"},{"type":"module","title":"Exceptions - Bumblebee.Text.Bart","doc":"The `:for_causal_language_modeling` model is just the decoder part and\naccepts the following inputs instead: `\"input_ids\"`, `\"attention_mask\"`,\n`\"position_ids\"`, `\"attention_head_mask\"`, `\"input_embeddings\"`, `\"encoder_hidden_state\"`,\n`\"encoder_attention_mask\"`, `\"cross_attention_head_mask\"`, `\"cache\"`.","ref":"Bumblebee.Text.Bart.html#module-exceptions"},{"type":"module","title":"Global layer options - Bumblebee.Text.Bart","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Bart.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Bart","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `50265`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `1024`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `1024`\n\n  * `:encoder_num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:decoder_num_blocks` - the number of Transformer blocks in the decoder. Defaults to `12`\n\n  * `:encoder_num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `16`\n\n  * `:decoder_num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `16`\n\n  * `:scale_embedding` - whether to scale embeddings by dividing by the square root of `:hidden_size`. Defaults to `false`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:activation_dropout_rate` - the dropout rate for activations inside fully connected layers. Defaults to `0.0`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. Defaults to `0.0`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Bart.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Bert","doc":"BERT model family.","ref":"Bumblebee.Text.Bert.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Bert","doc":"* `:base` - plain BERT without any head on top\n\n  * `:for_masked_language_modeling` - BERT with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - BERT with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - BERT with a token classification\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_question_answering` - BERT with a span classification head.\n    The head returns logits for the span start and end positions\n\n  * `:for_multiple_choice` - BERT with a multiple choice prediction\n    head. Each input in the batch consists of several sequences to\n    choose from and the model returns logits corresponding to those\n    choices\n\n  * `:for_next_sentence_prediction` - BERT with a next sentence\n    prediction head. The head returns logits predicting whether the\n    second sentence is random or in context\n\n  * `:for_pre_training` - BERT with both MLM and NSP heads as done\n    during the pre-training\n\n  * `:for_causal_language_modeling` - BERT working as a decoder with\n    a language modeling head. The head returns logits for each token\n    in the original sequence","ref":"Bumblebee.Text.Bert.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Bert","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"token_type_ids\"` - `{batch_size, sequence_length}`\n\n    Mask distinguishing groups in the input sequence. This is used\n    in when the input sequence is a semantically a pair of sequences.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.","ref":"Bumblebee.Text.Bert.html#module-inputs"},{"type":"module","title":"Exceptions - Bumblebee.Text.Bert","doc":"The `:for_multiple_choice` model accepts groups of sequences, so the\nexpected sequence shape is `{batch_size, num_choices, sequence_length}`.\n\nThe `:for_causal_language_modeling` model is a decoder and accepts\nthe following additional inputs: `\"encoder_hidden_state\"`,\n`\"encoder_attention_mask\"`, `\"cross_attention_head_mask\"`, `\"cache\"`.","ref":"Bumblebee.Text.Bert.html#module-exceptions"},{"type":"module","title":"Global layer options - Bumblebee.Text.Bert","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Bert.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Bert","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `30522`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `512`\n\n  * `:max_token_types` - the vocabulary size of the token type embedding (also referred to as segment embedding).\n    This corresponds to how many different token groups can be distinguished in the input\n    . Defaults to `2`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `3072`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.1`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. If not specified, the value of `:dropout_rate` is used instead\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`\n\n  * `:use_cross_attention` - whether cross-attention layers should be added to the model. This is only relevant for decoder models. Defaults to `false`","ref":"Bumblebee.Text.Bert.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Text.Bert","doc":"* [BERT: Pre-training of Deep Bidirectional Transformers for Language Understanding](https://arxiv.org/abs/1810.04805)","ref":"Bumblebee.Text.Bert.html#module-references"},{"type":"module","title":"Bumblebee.Text.Blenderbot","doc":"Blenderbot model family.","ref":"Bumblebee.Text.Blenderbot.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Blenderbot","doc":"* `:base` - plain Blenderbot without any head on top\n\n  * `:for_conditional_generation` - Blenderbot with a language modeling\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Text.Blenderbot.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Blenderbot","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"decoder_input_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of decoder input sequence tokens in the vocabulary. If not\n    present and `\"input_ids\"` is, it will be generated by shifting\n    each token in `\"input_ids\"` to the right once.\n\n  * `\"decoder_attention_mask\"` - `{batch_size, target_sequence_length}`\n\n    Mask indicating which decoder tokens to attend to. This is used\n    to ignore padding tokens, which are added when processing a batch\n    of sequences with different length.\n\n  * `\"decoder_position_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of positions of each decoder input sequence tokens in\n    the position embeddings.\n\n  * `\"decoder_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the decoder.\n\n  * `\"decoder_input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"decoder_input_ids\"`, which can be\n    specified for more control over how `\"decoder_input_ids\"` are\n    embedded than the model's internal embedding lookup. If\n    `\"decoder_input_embeddings\"` are present, then `\"decoder_input_ids\"`\n    will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"cross_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Blenderbot.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.Blenderbot","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Blenderbot.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Blenderbot","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `8008`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `128`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `2560`\n\n  * `:encoder_num_blocks` - the number of Transformer blocks in the encoder. Defaults to `2`\n\n  * `:decoder_num_blocks` - the number of Transformer blocks in the decoder. Defaults to `24`\n\n  * `:encoder_num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `32`\n\n  * `:decoder_num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `32`\n\n  * `:scale_embedding` - whether to scale embeddings by dividing by the square root of `:hidden_size`. Defaults to `false`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:activation_dropout_rate` - the dropout rate for activations inside fully connected layers. Defaults to `0.0`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`","ref":"Bumblebee.Text.Blenderbot.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.BlipText","doc":"The BLIP model for text encoding.","ref":"Bumblebee.Text.BlipText.html"},{"type":"module","title":"Architectures - Bumblebee.Text.BlipText","doc":"* `:base` - the base text model","ref":"Bumblebee.Text.BlipText.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.BlipText","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, encoder_sequence_length, encoder_hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"encoder_attention_mask\"` - `{batch_size, encoder_sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"cross_attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.BlipText.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.BlipText","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.BlipText.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.BlipText","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `30524`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `512`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:encoder_hidden_size` - the dimensionality of hidden layers in the vision encoder. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `8`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `3072`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.0`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`","ref":"Bumblebee.Text.BlipText.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.ClipText","doc":"The CLIP model for text encoding.","ref":"Bumblebee.Text.ClipText.html"},{"type":"module","title":"Architectures - Bumblebee.Text.ClipText","doc":"* `:base` - the base text model\n\n  * `:for_embedding` - the base model with a single projection layer\n    on top. The head returns a vector embedded in the joint text-image\n    CLIP space","ref":"Bumblebee.Text.ClipText.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.ClipText","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.","ref":"Bumblebee.Text.ClipText.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.ClipText","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.ClipText.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.ClipText","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `49408`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `77`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `512`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `8`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `2048`\n\n  * `:projection_size` - the dimensionality of the projection layer. Defaults to `512`\n\n  * `:activation` - the activation function. Defaults to `:gelu_approx_sigmoid`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-5`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.ClipText.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Distilbert","doc":"DistilBERT model family.","ref":"Bumblebee.Text.Distilbert.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Distilbert","doc":"* `:base` - plain DistilBERT without any head on top\n\n  * `:for_masked_language_modeling` - DistilBERT with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - DistilBERT with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - DistilBERT with a token classification\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_question_answering` - DistilBERT with a span classification head.\n    The head returns logits for the span start and end positions\n\n  * `:for_multiple_choice` - DistilBERT with a multiple choice prediction\n    head. Each input in the batch consists of several sequences to\n    choose from and the model returns logits corresponding to those\n    choices","ref":"Bumblebee.Text.Distilbert.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Distilbert","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n    Mask distinguishing groups in the input sequence. This is used\n    in when the input sequence is a semantically a pair of sequences.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.","ref":"Bumblebee.Text.Distilbert.html#module-inputs"},{"type":"module","title":"Exceptions - Bumblebee.Text.Distilbert","doc":"The `:for_multiple_choice` model accepts groups of sequences, so the\nexpected sequence shape is `{batch_size, num_choices, sequence_length}`.","ref":"Bumblebee.Text.Distilbert.html#module-exceptions"},{"type":"module","title":"Global layer options - Bumblebee.Text.Distilbert","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Distilbert.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Distilbert","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `30522`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `512`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `6`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `3072`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.1`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. If not specified, the value of `:dropout_rate` is used instead\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`\n\n  * `:use_cross_attention` - whether cross-attention layers should be added to the model. This is only relevant for decoder models. Defaults to `false`","ref":"Bumblebee.Text.Distilbert.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Text.Distilbert","doc":"* [DistilBERT, a distilled version of BERT: smaller, faster, cheaper and lighter](https://arxiv.org/abs/1910.01108)","ref":"Bumblebee.Text.Distilbert.html#module-references"},{"type":"module","title":"Bumblebee.Text.Gemma","doc":"Gemma model family.","ref":"Bumblebee.Text.Gemma.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Gemma","doc":"* `:base` - plain Gemma without any head on top\n\n  * `:for_causal_language_modeling` - Gemma with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - Gemma with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes","ref":"Bumblebee.Text.Gemma.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Gemma","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Gemma.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.Gemma","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Gemma.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Gemma","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `256000`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `8192`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `3072`\n\n  * `:intermediate_size` - the dimensionality of intermediate layers. Defaults to `24576`\n\n  * `:attention_head_size` - the size of the key, value, and query projection per attention head. Defaults to `256`\n\n  * `:num_blocks` - the number of Transformer blocks in the model. Defaults to `28`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the model. Defaults to `16`\n\n  * `:num_key_value_heads` - the number of key value heads for each attention layer in the model. Defaults to `16`\n\n  * `:activation` - the activation function. Defaults to `:gelu_approx_tanh`\n\n  * `:rotary_embedding_base` - base for computing rotary embedding frequency. Defaults to `10000`\n\n  * `:rotary_embedding_scaling_strategy` - scaling configuration for rotary embedding. Currently the supported values are:\n    \n      * `%{type: :linear, factor: number()}`\n    \n      * `%{type: :dynamic, factor: number()}`\n    \n    For more details see https://www.reddit.com/r/LocalLlama/comments/14mrgpr/dynamically_scaled_rope_further_increases\n    \n\n  * `:use_attention_bias` - whether or not to use bias in the query, key, value, and output projections in attention layers. Defaults to `false`\n\n  * `:layer_norm_epsilon` - the epsilon used by RMS normalization layers. Defaults to `1.0e-6`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Gemma.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Gpt2","doc":"GPT-2 model family.","ref":"Bumblebee.Text.Gpt2.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Gpt2","doc":"* `:base` - plain GPT-2 without any head on top\n\n  * `:for_causal_language_modeling` - GPT-2 with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - GPT-2 with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - GPT-2 with a token classification\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Text.Gpt2.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Gpt2","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, encoder_sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"encoder_attention_mask\"` - `{batch_size, encoder_sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"cross_attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Gpt2.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.Gpt2","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Gpt2.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Gpt2","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `50257`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `1024`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the decoder. Defaults to `24`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `16`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the decoder.\n    If not specified, defaults to 4 times `:hidden_size`\n    \n\n  * `:activation` - the activation function. Defaults to `:gelu_approx_tanh`\n\n  * `:scale_attention_weights` - whether to scale attention weights to have variance of 1. Defaults to `true`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.1`\n\n  * `:embeddings_dropout_rate` - the dropout rate for embeddings. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.1`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. Defaults to `0.1`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-5`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`\n\n  * `:use_cross_attention` - whether cross-attention layers should be added to the model. This is only relevant for decoder models. Defaults to `false`","ref":"Bumblebee.Text.Gpt2.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.GptBigCode","doc":"GPT-BigCode model family.","ref":"Bumblebee.Text.GptBigCode.html"},{"type":"module","title":"Architectures - Bumblebee.Text.GptBigCode","doc":"* `:base` - plain GPT-BigCode without any head on top\n\n  * `:for_causal_language_modeling` - GPT-BigCode with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - GPT-BigCode with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - GPT-BigCode with a token classification\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Text.GptBigCode.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.GptBigCode","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, encoder_sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"encoder_attention_mask\"` - `{batch_size, encoder_sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"cross_attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.GptBigCode.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.GptBigCode","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.GptBigCode.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.GptBigCode","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `50257`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `1024`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the decoder. Defaults to `24`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `16`\n\n  * `:num_key_value_heads` - the number of key value heads for each attention layer in the model\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the decoder.\n    If not specified, defaults to 4 times `:hidden_size`\n    \n\n  * `:activation` - the activation function. Defaults to `:gelu_approx_tanh`\n\n  * `:scale_attention_weights` - whether to scale attention weights to have variance of 1. Defaults to `true`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.1`\n\n  * `:embeddings_dropout_rate` - the dropout rate for embeddings. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.1`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. Defaults to `0.1`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-5`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`\n\n  * `:use_cross_attention` - whether cross-attention layers should be added to the model. This is only relevant for decoder models. Defaults to `false`","ref":"Bumblebee.Text.GptBigCode.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.GptNeoX","doc":"GPT-NeoX model family.","ref":"Bumblebee.Text.GptNeoX.html"},{"type":"module","title":"Architectures - Bumblebee.Text.GptNeoX","doc":"* `:base` - plain GPT-NeoX without any head on top\n\n  * `:for_causal_language_modeling` - GPT-NeoX with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - GPT-NeoX with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - GPT-NeoX with a token classification\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Text.GptNeoX.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.GptNeoX","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.GptNeoX.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.GptNeoX","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.GptNeoX.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.GptNeoX","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `32000`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `4096`\n\n  * `:intermediate_size` - the dimensionality of intermediate layers. Defaults to `11008`\n\n  * `:num_blocks` - the number of Transformer blocks in the model. Defaults to `32`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the model. Defaults to `32`\n\n  * `:activation` - the activation function. Defaults to `:silu`\n\n  * `:rotary_embedding_percentage` - percentage of hidden dimensions to allocate to rotary embeddings. Defaults to `0.25`\n\n  * `:rotary_embedding_base` - base for computing rotary embedding frequency. Defaults to `10000`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. Defaults to `0.1`\n\n  * `:layer_norm_epsilon` - the epsilon used by RMS normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:use_parallel_transformer_block` - whether to use the parallel formulation of the Transformer block, where attention and FFN is computed independently. Defaults to `true`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.GptNeoX.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Llama","doc":"LLaMA model family.","ref":"Bumblebee.Text.Llama.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Llama","doc":"* `:base` - plain LLaMA without any head on top\n\n  * `:for_causal_language_modeling` - LLaMA with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - LLaMA with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes","ref":"Bumblebee.Text.Llama.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Llama","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Llama.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.Llama","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Llama.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Llama","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `32000`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `1024`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `4096`\n\n  * `:intermediate_size` - the dimensionality of intermediate layers. Defaults to `11008`\n\n  * `:attention_head_size` - the size of the key, value, and query projection per attention head.\n    Defaults to `div(hidden_size, num_attention_heads)`\n    \n\n  * `:num_blocks` - the number of Transformer blocks in the model. Defaults to `32`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the model. Defaults to `32`\n\n  * `:num_key_value_heads` - the number of key value heads for each attention layer in the model\n\n  * `:activation` - the activation function. Defaults to `:silu`\n\n  * `:rotary_embedding_base` - base for computing rotary embedding frequency. Defaults to `10000`\n\n  * `:rotary_embedding_scaling_strategy` - scaling configuration for rotary embedding. Currently the supported values are:\n    \n      * `%{type: :linear, factor: number()}`\n    \n      * `%{type: :dynamic, factor: number()}`\n    \n      * `%{type: :llama3, factor: number(), low_frequency_factor: number(), high_frequency_factor: number(), original_max_positions: pos_integer()}`\n    \n    For more details see https://www.reddit.com/r/LocalLLaMA/comments/14mrgpr/dynamically_scaled_rope_further_increases\n    \n\n  * `:layer_norm_epsilon` - the epsilon used by RMS normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:tie_word_embeddings` - whether to tie input and output embedding weights. Defaults to `false`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Llama.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.M2m100","doc":"M2M100 model family.","ref":"Bumblebee.Text.M2m100.html"},{"type":"module","title":"Architectures - Bumblebee.Text.M2m100","doc":"* `:base` - plain M2M100 without any head on top\n\n  * `:for_conditional_generation` - M2M100 with a language modeling\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Text.M2m100.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.M2m100","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"decoder_input_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of decoder input sequence tokens in the vocabulary. If not\n    present and `\"input_ids\"` is, it will be generated by shifting\n    each token in `\"input_ids\"` to the right once.\n\n  * `\"decoder_attention_mask\"` - `{batch_size, target_sequence_length}`\n\n    Mask indicating which decoder tokens to attend to. This is used\n    to ignore padding tokens, which are added when processing a batch\n    of sequences with different length.\n\n  * `\"decoder_position_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of positions of each decoder input sequence tokens in\n    the position embeddings.\n\n  * `\"decoder_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the decoder.\n\n  * `\"decoder_input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"decoder_input_ids\"`, which can be\n    specified for more control over how `\"decoder_input_ids\"` are\n    embedded than the model's internal embedding lookup. If\n    `\"decoder_input_embeddings\"` are present, then `\"decoder_input_ids\"`\n    will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"cross_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.M2m100.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.M2m100","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.M2m100.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.M2m100","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `128112`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `1024`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `1024`\n\n  * `:encoder_num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:decoder_num_blocks` - the number of Transformer blocks in the decoder. Defaults to `12`\n\n  * `:encoder_num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `16`\n\n  * `:decoder_num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `16`\n\n  * `:encoder_intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `4096`\n\n  * `:decoder_intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the decoder. Defaults to `4096`\n\n  * `:scale_embedding` - scale embeddings by dividing by sqrt(hidden_size). Defaults to `true`\n\n  * `:activation` - the activation function. Defaults to `:relu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.1`\n\n  * `:activation_dropout_rate` - the dropout rate for activations inside fully connected layers. Defaults to `0.0`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`","ref":"Bumblebee.Text.M2m100.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Mbart","doc":"mBART model family.","ref":"Bumblebee.Text.Mbart.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Mbart","doc":"* `:base` - plain mBART without any head on top\n\n  * `:for_causal_language_modeling` - mBART with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_conditional_generation` - mBART with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - mBART with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_question_answering` - mBART with a span classification head.\n    The head returns logits for the span start and end positions","ref":"Bumblebee.Text.Mbart.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Mbart","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"decoder_input_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of decoder input sequence tokens in the vocabulary. If not\n    present and `\"input_ids\"` is, it will be generated by shifting\n    each token in `\"input_ids\"` to the right once.\n\n  * `\"decoder_attention_mask\"` - `{batch_size, target_sequence_length}`\n\n    Mask indicating which decoder tokens to attend to. This is used\n    to ignore padding tokens, which are added when processing a batch\n    of sequences with different length.\n\n  * `\"decoder_position_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of positions of each decoder input sequence tokens in\n    the position embeddings.\n\n  * `\"decoder_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the decoder.\n\n  * `\"decoder_input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"decoder_input_ids\"`, which can be\n    specified for more control over how `\"decoder_input_ids\"` are\n    embedded than the model's internal embedding lookup. If\n    `\"decoder_input_embeddings\"` are present, then `\"decoder_input_ids\"`\n    will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"cross_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Mbart.html#module-inputs"},{"type":"module","title":"Exceptions - Bumblebee.Text.Mbart","doc":"The `:for_causal_language_modeling` model is just the decoder part and\naccepts the following inputs instead: `\"input_ids\"`, `\"attention_mask\"`,\n`\"position_ids\"`, `\"attention_head_mask\"`, `\"input_embeddings\"`, `\"encoder_hidden_state\"`,\n`\"encoder_attention_mask\"`, `\"cross_attention_head_mask\"`, `\"cache\"`.","ref":"Bumblebee.Text.Mbart.html#module-exceptions"},{"type":"module","title":"Global layer options - Bumblebee.Text.Mbart","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Mbart.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Mbart","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `50265`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `1024`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `1024`\n\n  * `:encoder_num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:decoder_num_blocks` - the number of Transformer blocks in the decoder. Defaults to `12`\n\n  * `:encoder_num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `16`\n\n  * `:decoder_num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `16`\n\n  * `:encoder_intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `4096`\n\n  * `:decoder_intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the decoder. Defaults to `4096`\n\n  * `:scale_embedding` - scale embeddings by dividing by sqrt(hidden_size). Defaults to `false`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:activation_dropout_rate` - the dropout rate for activations inside fully connected layers. Defaults to `0.0`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. Defaults to `0.0`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Mbart.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Mistral","doc":"Mistral model family.","ref":"Bumblebee.Text.Mistral.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Mistral","doc":"* `:base` - plain Mistral without any head on top\n\n  * `:for_causal_language_modeling` - Mistral with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - Mistral with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes","ref":"Bumblebee.Text.Mistral.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Mistral","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Mistral.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.Mistral","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Mistral.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Mistral","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `32000`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `131072`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `4096`\n\n  * `:intermediate_size` - the dimensionality of intermediate layers. Defaults to `14336`\n\n  * `:num_blocks` - the number of Transformer blocks in the model. Defaults to `32`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the model. Defaults to `32`\n\n  * `:num_key_value_heads` - the number of key-value heads used to implement Grouped Query Attention. If\n    this value is set to the same as the number of attention heads, it will use\n    regular MHA. If it's set to 1, it will use MQA, otherwise it uses Grouped Query\n    Attention\n    . Defaults to `8`\n\n  * `:attention_window_size` - window size for both sides of the sliding attention window. Defaults to `4096`\n\n  * `:activation` - the activation function. Defaults to `:silu`\n\n  * `:layer_norm_epsilon` - the epsilon used by RMS normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:rotary_embedding_base` - base for computing rotary embedding frequency. Defaults to `10000`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Mistral.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Phi","doc":"Phi model family.","ref":"Bumblebee.Text.Phi.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Phi","doc":"* `:base` - plain Phi without any head on top\n\n  * `:for_causal_language_modeling` - Phi with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - Phi with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - Phi with a token classification\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Text.Phi.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Phi","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Phi.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.Phi","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Phi.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Phi","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `51200`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `2048`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `2048`\n\n  * `:intermediate_size` - the dimensionality of intermediate layers. Defaults to `8192`\n\n  * `:num_blocks` - the number of Transformer blocks in the model. Defaults to `24`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the model. Defaults to `32`\n\n  * `:num_key_value_heads` - the number of key-value heads used to implement Grouped Query Attention. If\n    this value is set to the same as the number of attention heads, it will use\n    regular MHA. If it's set to 1, it will use MQA, otherwise it uses Grouped Query\n    Attention\n    \n\n  * `:activation` - the activation function. Defaults to `:gelu_approx_tanh`\n\n  * `:rotary_embedding_percentage` - percentage of the query and keys that will have rotary embedding. Defaults to `0.5`\n\n  * `:rotary_embedding_base` - base for computing rotary embedding frequency. Defaults to `10000`\n\n  * `:layer_norm_epsilon` - the epsilon used by RMS normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Phi.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Phi3","doc":"Phi-3 model family.","ref":"Bumblebee.Text.Phi3.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Phi3","doc":"* `:base` - plain Phi-3 without any head on top\n\n  * `:for_causal_language_modeling` - Phi-3 with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - Phi-3 with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - Phi-3 with a token classification\n    head. The head returns logits for each token in the original\n    sequence","ref":"Bumblebee.Text.Phi3.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Phi3","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.Phi3.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.Phi3","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Phi3.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Phi3","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `51200`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `2048`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `2048`\n\n  * `:intermediate_size` - the dimensionality of intermediate layers. Defaults to `8192`\n\n  * `:num_blocks` - the number of Transformer blocks in the model. Defaults to `24`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the model. Defaults to `32`\n\n  * `:num_key_value_heads` - the number of key-value heads used to implement Grouped Query Attention. If\n    this value is set to the same as the number of attention heads, it will use\n    regular MHA. If it's set to 1, it will use MQA, otherwise it uses Grouped Query\n    Attention\n    \n\n  * `:attention_window_size` - window size for both sides of the sliding attention window. Defaults to `262144`\n\n  * `:activation` - the activation function. Defaults to `:gelu_approx_tanh`\n\n  * `:rotary_embedding_percentage` - percentage of hidden dimensions to allocate to rotary embeddings. Defaults to `1.0`\n\n  * `:rotary_embedding_base` - base for computing rotary embedding frequency. Defaults to `10000`\n\n  * `:rotary_embedding_scaling_strategy` - scaling configuration for rotary embedding. Currently the supported values are:\n    \n      * `%{type: :longrope, short_factor: list(number()), long_factor: list(number()), original_max_positions: pos_integer()}`\n    \n    \n\n  * `:layer_norm_epsilon` - the epsilon used by RMS normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.Phi3.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.Roberta","doc":"RoBERTa model family.","ref":"Bumblebee.Text.Roberta.html"},{"type":"module","title":"Architectures - Bumblebee.Text.Roberta","doc":"* `:base` - plain RoBERTa without any head on top\n\n  * `:for_masked_language_modeling` - RoBERTa with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_sequence_classification` - RoBERTa with a sequence\n    classification head. The head returns logits corresponding to\n    possible classes\n\n  * `:for_token_classification` - RoBERTa with a token classification\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:for_question_answering` - RoBERTa with a span classification head.\n    The head returns logits for the span start and end positions\n\n  * `:for_multiple_choice` - RoBERTa with a multiple choice prediction\n    head. Each input in the batch consists of several sequences to\n    choose from and the model returns logits corresponding to those\n    choices\n\n  * `:for_causal_language_modeling` - RoBERTa working as a decoder with\n    a language modeling head. The head returns logits for each token in\n    the original sequence","ref":"Bumblebee.Text.Roberta.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.Roberta","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"token_type_ids\"` - `{batch_size, sequence_length}`\n\n    Mask distinguishing groups in the input sequence. This is used\n    in when the input sequence is a semantically a pair of sequences.\n\n  * `\"position_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of positions of each input sequence tokens in the position\n    embeddings.\n\n  * `\"attention_head_mask\"` - `{num_blocks, num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.","ref":"Bumblebee.Text.Roberta.html#module-inputs"},{"type":"module","title":"Exceptions - Bumblebee.Text.Roberta","doc":"The `:for_multiple_choice` model accepts groups of sequences, so the\nexpected sequence shape is `{batch_size, num_choices, sequence_length}`.\n\nThe `:for_causal_language_modeling` model is a decoder and accepts\nthe following additional inputs: `\"encoder_hidden_state\"`,\n`\"encoder_attention_mask\"`, `\"cross_attention_head_mask\"`, `\"cache\"`.","ref":"Bumblebee.Text.Roberta.html#module-exceptions"},{"type":"module","title":"Global layer options - Bumblebee.Text.Roberta","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.Roberta.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.Roberta","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `30522`\n\n  * `:max_positions` - the vocabulary size of the position embedding. This corresponds to the maximum sequence\n    length that this model can process. Typically this is set to a large value just in case,\n    such as 512, 1024 or 2048\n    . Defaults to `512`\n\n  * `:max_token_types` - the vocabulary size of the token type embedding (also referred to as segment embedding).\n    This corresponds to how many different token groups can be distinguished in the input\n    . Defaults to `2`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:intermediate_size` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder. Defaults to `3072`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for embedding and encoder. Defaults to `0.1`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.1`\n\n  * `:classifier_dropout_rate` - the dropout rate for the classification head. If not specified, the value of `:dropout_rate` is used instead\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_range` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`\n\n  * `:use_cross_attention` - whether cross-attention layers should be added to the model. This is only relevant for decoder models. Defaults to `false`","ref":"Bumblebee.Text.Roberta.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.T5","doc":"T5 model family.","ref":"Bumblebee.Text.T5.html"},{"type":"module","title":"Architectures - Bumblebee.Text.T5","doc":"* `:base` - plain T5 without any head on top\n\n  * `:for_conditional_generation` - T5 with a language modeling\n    head. The head returns logits for each token in the original\n    sequence\n\n  * `:encoder` - just the encoder part of the base model","ref":"Bumblebee.Text.T5.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Text.T5","doc":"* `\"input_ids\"` - `{batch_size, sequence_length}`\n\n    Indices of input sequence tokens in the vocabulary.\n\n  * `\"attention_mask\"` - `{batch_size, sequence_length}`\n\n    Mask indicating which tokens to attend to. This is used to ignore\n    padding tokens, which are added when processing a batch of sequences\n    with different length.\n\n  * `\"attention_head_mask\"` - `{encoder_num_blocks, encoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the encoder.\n\n  * `\"input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"input_ids\"`, which can be specified\n    for more control over how `\"input_ids\"` are embedded than the\n    model's internal embedding lookup. If `\"input_embeddings\"` are present,\n    then `\"input_ids\"` will be ignored.\n\n  * `\"decoder_input_ids\"` - `{batch_size, target_sequence_length}`\n\n    Indices of decoder input sequence tokens in the vocabulary. If not\n    present and `\"input_ids\"` is, it will be generated by shifting\n    each token in `\"input_ids\"` to the right once.\n\n  * `\"decoder_attention_mask\"` - `{batch_size, target_sequence_length}`\n\n    Mask indicating which decoder tokens to attend to. This is used\n    to ignore padding tokens, which are added when processing a batch\n    of sequences with different length.\n\n  * `\"decoder_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the self-attention blocks in\n    the decoder.\n\n  * `\"decoder_input_embeddings\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Embedded representation of `\"decoder_input_ids\"`, which can be\n    specified for more control over how `\"decoder_input_ids\"` are\n    embedded than the model's internal embedding lookup. If\n    `\"decoder_input_embeddings\"` are present, then `\"decoder_input_ids\"`\n    will be ignored.\n\n  * `\"encoder_hidden_state\"` - `{batch_size, sequence_length, hidden_size}`\n\n    Last hidden state output from the encoder. This hidden state is\n    used in cross-attention blocks in the decoder. If specified, the\n    model will skip the encoding process and use this value directly\n    for cross-attentions in the decoder.\n\n  * `\"cross_attention_head_mask\"` - `{decoder_num_blocks, decoder_num_attention_heads}`\n\n    Mask to nullify selected heads of the cross-attention blocks in\n    the decoder with shape.\n\n  * `\"cache\"`\n\n    A container with cached layer results used to speed up sequential\n    decoding (autoregression). With cache, certain hidden states are\n    taken from the cache, rather than recomputed on every decoding\n    pass. The cache should be treated as opaque and initialized with\n    `Bumblebee.Text.Generation.init_cache/4`.","ref":"Bumblebee.Text.T5.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Text.T5","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Text.T5.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Text.T5","doc":"* `:vocab_size` - the vocabulary size of the token embedding. This corresponds to the number of distinct\n    tokens that can be represented in model input and output\n    . Defaults to `32128`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `512`\n\n  * `:attention_head_size` - the size of the key, value, and query projection per attention head. Defaults to `64`\n\n  * `:encoder_num_blocks` - the number of Transformer blocks in the encoder. Defaults to `6`\n\n  * `:decoder_num_blocks` - the number of Transformer blocks in the decoder. Defaults to `6`\n\n  * `:encoder_num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `8`\n\n  * `:decoder_num_attention_heads` - the number of attention heads for each attention layer in the decoder. Defaults to `8`\n\n  * `:activation` - the activation function. Defaults to `:relu`\n\n  * `:ffn_gated_activation` - whether to use a gated variant of the activation function in the feed-forward network (FFN). Defaults to `false`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.1`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `1.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-6`\n\n  * `:tie_word_embeddings` - whether or not to tie encoder and decoder token embedding. Defaults to `true`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Text.T5.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.BlipVision","doc":"The BLIP model for image encoding.","ref":"Bumblebee.Vision.BlipVision.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.BlipVision","doc":"* `:base` - the base image model","ref":"Bumblebee.Vision.BlipVision.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Vision.BlipVision","doc":"* `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.","ref":"Bumblebee.Vision.BlipVision.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Vision.BlipVision","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Vision.BlipVision.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.BlipVision","doc":"* `:image_size` - the size of the input spatial dimensions. Defaults to `384`\n\n  * `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:patch_size` - the size of the patch spatial dimensions. Defaults to `16`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-5`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`","ref":"Bumblebee.Vision.BlipVision.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.ClipVision","doc":"The CLIP model for image encoding.","ref":"Bumblebee.Vision.ClipVision.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.ClipVision","doc":"* `:base` - the base image model\n\n  * `:for_embedding` - the base model with a single projection layer\n    on top. The head returns a vector embedded in the joint text-image\n    CLIP space","ref":"Bumblebee.Vision.ClipVision.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Vision.ClipVision","doc":"* `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.","ref":"Bumblebee.Vision.ClipVision.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Vision.ClipVision","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Vision.ClipVision.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.ClipVision","doc":"* `:image_size` - the size of the input spatial dimensions. Defaults to `224`\n\n  * `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:patch_size` - the size of the patch spatial dimensions. Defaults to `32`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:projection_size` - the dimensionality of the projection layer. Defaults to `512`\n\n  * `:activation` - the activation function. Defaults to `:gelu_approx_sigmoid`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-5`","ref":"Bumblebee.Vision.ClipVision.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.ConvNext","doc":"ConvNeXT model family.","ref":"Bumblebee.Vision.ConvNext.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.ConvNext","doc":"* `:base` - plain ConvNeXT without any head on top\n\n  * `:for_image_classification` - ConvNeXT with a classification head.\n    The head consists of a single dense layer on top of the pooled\n    features","ref":"Bumblebee.Vision.ConvNext.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Vision.ConvNext","doc":"* `\"pixel_values\"` - {batch_size, height, width, num_channels}\n\n    Featurized image pixel values (224x224).","ref":"Bumblebee.Vision.ConvNext.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Vision.ConvNext","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states","ref":"Bumblebee.Vision.ConvNext.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.ConvNext","doc":"* `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:patch_size` - the size of the patch spatial dimensions. Defaults to `4`\n\n  * `:hidden_sizes` - the dimensionality of hidden layers at each stage. Defaults to `[96, 192, 384, 768]`\n\n  * `:depths` - the depth (number of residual blocks) at each stage. Defaults to `[3, 3, 9, 3]`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:scale_initial_value` - the initial value for scaling layers. Defaults to `1.0e-6`\n\n  * `:drop_path_rate` - the drop path rate used to for stochastic depth. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Vision.ConvNext.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Vision.ConvNext","doc":"* [A ConvNet for the 2020s](https://arxiv.org/abs/2201.03545)","ref":"Bumblebee.Vision.ConvNext.html#module-references"},{"type":"module","title":"Bumblebee.Vision.Deit","doc":"DeiT model family.","ref":"Bumblebee.Vision.Deit.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.Deit","doc":"* `:base` - plain DeiT without any head on top\n\n  * `:for_image_classification` - DeiT with a classification head.\n    The head consists of two dense layers on top of the final\n    hidden state of the CLS token\n\n  * `:for_image_classification_with teacher` - DeiT with a\n    classification head. The head consists of two dense layers\n    on top of the final hidden state of the CLS token and the\n    final hidden state of the distillation token\n\n  * `:for_masked_image_modeling` - DEiT with a language modeling\n    head on top for predicting visual tokens","ref":"Bumblebee.Vision.Deit.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Vision.Deit","doc":"* `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.\n\n  * `\"patch_mask\"` - `{batch_size, num_patches}`\n\n    Mask to nullify selected embedded patches.","ref":"Bumblebee.Vision.Deit.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Vision.Deit","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Vision.Deit.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.Deit","doc":"* `:image_size` - the size of the input spatial dimensions. Defaults to `224`\n\n  * `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:patch_size` - the size of the patch spatial dimensions. Defaults to `16`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:use_attention_bias` - whether to use bias in query, key, and value projections. Defaults to `true`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.0`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Vision.Deit.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Vision.Deit","doc":"* [An Image is Worth 16x16 Words: Transformers for Image Recognition at Scale](https://arxiv.org/abs/2010.11929)","ref":"Bumblebee.Vision.Deit.html#module-references"},{"type":"module","title":"Bumblebee.Vision.DinoV2","doc":"DINOv2 model family.","ref":"Bumblebee.Vision.DinoV2.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.DinoV2","doc":"* `:base` - plain DINOv2 without any head on top\n\n  * `:for_image_classification` - DINOv2 with head for image classification\n\n  * `:backbone` - DINOv2 with feature maps output","ref":"Bumblebee.Vision.DinoV2.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Vision.DinoV2","doc":"* `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.\n\n  * `\"patch_mask\"` - `{batch_size, num_patches}`\n\n    Mask to nullify selected embedded patches.","ref":"Bumblebee.Vision.DinoV2.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Vision.DinoV2","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Vision.DinoV2.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.DinoV2","doc":"* `:image_size` - the size of the input spatial dimensions. The model is trained for this size, however\n    the model supports any other input size by interpolating position embeddings\n    . Defaults to `518`\n\n  * `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:patch_size` - the size of the patch spatial dimensions. Defaults to `14`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `384`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:intermediate_size_ratio` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder,\n    expressed as a multiplier of `:hidden_size`\n    . Defaults to `4`\n\n  * `:use_attention_bias` - whether to use bias in query, key, and value projections. Defaults to `true`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:ffn_swiglu_activation` - whether to use the gated SwiGLU activation function in the feed-forward network (FFN). Defaults to `false`\n\n  * `:scale_initial_value` - the initial value for scaling layers. Defaults to `1.0`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.0`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-6`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:backbone_output_indices` - list of indices indicating which feature maps to include in the output. If not specified, only\n    the last feature map is included\n    \n\n  * `:backbone_use_norm` - whether to add layer normalization layer to each of the feature maps returned by the backbone. Defaults to `true`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Vision.DinoV2.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Vision.DinoV2","doc":"* [DINOv2: Learning Robust Visual Features without Supervision](https://arxiv.org/abs/2304.07193)","ref":"Bumblebee.Vision.DinoV2.html#module-references"},{"type":"module","title":"Bumblebee.Vision.ResNet","doc":"ResNet model family.","ref":"Bumblebee.Vision.ResNet.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.ResNet","doc":"* `:base` - plain ResNet without any head on top\n\n  * `:for_image_classification` - ResNet with a classification head.\n    The head consists of a single dense layer on top of the pooled\n    features and it returns logits corresponding to possible classes","ref":"Bumblebee.Vision.ResNet.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Vision.ResNet","doc":"* `\"pixel_values\"` - {batch_size, height, width, num_channels}\n\n    Featurized image pixel values (224x224).","ref":"Bumblebee.Vision.ResNet.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Vision.ResNet","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states","ref":"Bumblebee.Vision.ResNet.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.ResNet","doc":"* `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:embedding_size` - the dimensionality of the embedding layer. Defaults to `64`\n\n  * `:hidden_sizes` - the dimensionality of hidden layers at each stage. Defaults to `[256, 512, 1024, 2048]`\n\n  * `:depths` - the depth (number of residual blocks) at each stage. Defaults to `[3, 4, 6, 3]`\n\n  * `:residual_block_type` - the residual block to use, either `:basic` (used for smaller models, like ResNet-18 or ResNet-34)\n    or `:bottleneck` (used for larger models like ResNet-50 and above)\n    . Defaults to `:bottleneck`\n\n  * `:activation` - the activation function. Defaults to `:relu`\n\n  * `:downsample_in_first_stage` - whether the first stage should downsample the inputs using a stride of 2. Defaults to `false`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Vision.ResNet.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.Swin","doc":"Swin Transformer model.","ref":"Bumblebee.Vision.Swin.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.Swin","doc":"* `:base` - plain Swin without any head on top\n\n  * `:for_image_classification` - Swin tranformer model with a\n    classification head","ref":"Bumblebee.Vision.Swin.html#module-architectures"},{"type":"module","title":"Global layer options - Bumblebee.Vision.Swin","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Vision.Swin.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.Swin","doc":"* `:image_size` - the size of the input spatial dimensions. Defaults to `224`\n\n  * `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:patch_size` - the size of the patch spatial dimensions. Defaults to `4`\n\n  * `:embedding_size` - the dimensionality of patch embedding layer. Defaults to `96`\n\n  * `:use_absolute_position_embeddings` - whether to add absolute position embeddings to the patch embeddings. Defaults to `false`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder at each stage. Defaults to `[2, 2, 6, 2]`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder at each stage. Defaults to `[3, 6, 12, 24]`\n\n  * `:window_size` - the window size, used to limit self-attention computation to non-overlapping windows. Defaults to `7`\n\n  * `:intermediate_size_ratio` - the dimensionality of the intermediate layer in the transformer feed-forward network (FFN) in the encoder,\n    expressed as a multiplier of hidden size (at the given stage)\n    . Defaults to `4`\n\n  * `:use_attention_bias` - whether to use bias in query, key, and value projections. Defaults to `true`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.0`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:drop_path_rate` - the drop path rate used to for stochastic depth. Defaults to `0.1`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-5`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Vision.Swin.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Vision.Swin","doc":"* [Swin Transformer: Hierarchical Vision Transformer using Shifted Windows](https://arxiv.org/abs/2103.14030)","ref":"Bumblebee.Vision.Swin.html#module-references"},{"type":"module","title":"Bumblebee.Vision.Vit","doc":"ViT model family.","ref":"Bumblebee.Vision.Vit.html"},{"type":"module","title":"Architectures - Bumblebee.Vision.Vit","doc":"* `:base` - plain ViT without any head on top\n\n  * `:for_image_classification` - ViT with a classification head.\n    The head consists of a single dense layer on top of the pooled\n    features\n\n  * `:for_masked_image_modeling` - ViT with a language modeling\n    head on top for predicting visual tokens","ref":"Bumblebee.Vision.Vit.html#module-architectures"},{"type":"module","title":"Inputs - Bumblebee.Vision.Vit","doc":"* `\"pixel_values\"` - `{batch_size, image_size, image_size, num_channels}`\n\n    Featurized image pixel values.\n\n  * `\"patch_mask\"` - `{batch_size, num_patches}`\n\n    Mask to nullify selected embedded patches.","ref":"Bumblebee.Vision.Vit.html#module-inputs"},{"type":"module","title":"Global layer options - Bumblebee.Vision.Vit","doc":"* `:output_hidden_states` - when `true`, the model output includes all hidden states\n\n  * `:output_attentions` - when `true`, the model output includes all attention weights","ref":"Bumblebee.Vision.Vit.html#module-global-layer-options"},{"type":"module","title":"Configuration - Bumblebee.Vision.Vit","doc":"* `:image_size` - the size of the input spatial dimensions. Defaults to `224`\n\n  * `:num_channels` - the number of channels in the input. Defaults to `3`\n\n  * `:patch_size` - the size of the patch spatial dimensions. Defaults to `16`\n\n  * `:hidden_size` - the dimensionality of hidden layers. Defaults to `768`\n\n  * `:num_blocks` - the number of Transformer blocks in the encoder. Defaults to `12`\n\n  * `:num_attention_heads` - the number of attention heads for each attention layer in the encoder. Defaults to `12`\n\n  * `:use_attention_bias` - whether to use bias in query, key, and value projections. Defaults to `true`\n\n  * `:activation` - the activation function. Defaults to `:gelu`\n\n  * `:dropout_rate` - the dropout rate for encoder and decoder. Defaults to `0.0`\n\n  * `:attention_dropout_rate` - the dropout rate for attention weights. Defaults to `0.0`\n\n  * `:layer_norm_epsilon` - the epsilon used by the layer normalization layers. Defaults to `1.0e-12`\n\n  * `:initializer_scale` - the standard deviation of the normal initializer used for initializing kernel parameters. Defaults to `0.02`\n\n  * `:num_labels` - the number of labels to use in the last layer for the classification task. Defaults to `2`\n\n  * `:id_to_label` - a map from class index to label. Defaults to `%{}`","ref":"Bumblebee.Vision.Vit.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Vision.Vit","doc":"* [An Image is Worth 16x16 Words: Transformers for Image Recognition at Scale](https://arxiv.org/abs/2010.11929)","ref":"Bumblebee.Vision.Vit.html#module-references"},{"type":"module","title":"Bumblebee.Audio.WhisperFeaturizer","doc":"Whisper featurizer for audio data.","ref":"Bumblebee.Audio.WhisperFeaturizer.html"},{"type":"module","title":"Configuration - Bumblebee.Audio.WhisperFeaturizer","doc":"* `:feature_size` - the dimension of the extracted features. This corresponds to the number of Mel bins. Defaults to `80`\n\n  * `:sampling_rate` - the sampling rate at which the audio files should be digitally expressed in Hertz. Defaults to `16000`\n\n  * `:num_seconds` - the maximum duration of the audio sequence. This implies that the maximum length of the\n    input sequence is `:num_seconds` * `:sampling_rate`\n    . Defaults to `30`\n\n  * `:hop_length` - the hop between consecutive overlapping windows for the STFT used to obtain Mel Frequency coefficients. Defaults to `160`\n\n  * `:fft_length` - the size of the fourier transform. Defaults to `400`\n\n  * `:padding_value` - the value used to pad the audio. Should correspond to silence. Defaults to `0.0`","ref":"Bumblebee.Audio.WhisperFeaturizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Text.PreTrainedTokenizer","doc":"Wraps a pre-trained tokenizer from the `Tokenizers` library.","ref":"Bumblebee.Text.PreTrainedTokenizer.html"},{"type":"module","title":"Configuration - Bumblebee.Text.PreTrainedTokenizer","doc":"* `:add_special_tokens` - whether to add special tokens during tokenization. Defaults to `true`\n\n  * `:length` - applies fixed length padding or truncation to the given input if set. Can be either\n    a specific number or a list of numbers. When a list is given, the smallest number\n    that exceeds all input lengths is used as the padding length\n    \n\n  * `:pad_direction` - the padding direction, either `:right` or `:left`. Defaults to `:right`\n\n  * `:truncate_direction` - the truncation direction, either `:right` or `:left`. Defaults to `:right`\n\n  * `:return_attention_mask` - whether to return attention mask for encoded sequence. The mask is a boolean tensor\n    indicating which tokens are padding and should effectively be ignored by the model\n    . Defaults to `true`\n\n  * `:return_token_type_ids` - whether to return token type ids for encoded sequence. Defaults to `true`\n\n  * `:return_special_tokens_mask` - whether to return special tokens mask for encoded sequence. The mask is a boolean\n    tensor indicating which tokens are special\n    . Defaults to `false`\n\n  * `:return_offsets` - whether to return token offsets for encoded sequence. This tensor includes a list of\n    position pairs that map tokens to the input text\n    . Defaults to `false`\n\n  * `:return_length` - whether to return the sequence length. The length is the effective number of tokens,\n    so it is calculated after truncation, but does not include padding\n    . Defaults to `false`\n\n  * `:template_options` - options configuring the tokenization template, specific to the given tokenizer type.\n    Recognised options are:\n    \n      * `:language_token` - for tokenizers: `:nllb`\n    \n    . Defaults to `[]`","ref":"Bumblebee.Text.PreTrainedTokenizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.BitFeaturizer","doc":"BiT featurizer for image data.","ref":"Bumblebee.Vision.BitFeaturizer.html"},{"type":"module","title":"Configuration - Bumblebee.Vision.BitFeaturizer","doc":"* `:resize` - whether to resize the input to the given `:size`. Defaults to `true`\n\n  * `:size` - the size to resize the input to, either `%{height: ..., width: ...}` or `%{shortest_edge: ...}`.\n    Only has an effect if `:resize` is `true`\n    . Defaults to `%{shortest_edge: 448}`\n\n  * `:resize_method` - the resizing method, either of `:nearest`, `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. Defaults to `:bicubic`\n\n  * `:center_crop` - whether to crop the input at the center. If the input size is smaller than `:crop_size` along\n    any edge, the image is padded with zeros and then center cropped\n    . Defaults to `true`\n\n  * `:crop_size` - the size to center crop the image to, given as `%{height: ..., width: ...}`. Only has an effect\n    if `:center_crop` is `true`\n    . Defaults to `%{width: 448, height: 448}`\n\n  * `:rescale` - whether to rescale the input by the given `:rescale_factor`. Defaults to `true`\n\n  * `:rescale_factor` - the factor by which to rescale the input. A single number\n    Only has an effect if `:rescale` is `true`\n    . Defaults to `0.00392156862745098`\n\n  * `:normalize` - whether or not to normalize the input with mean and standard deviation. Defaults to `true`\n\n  * `:image_mean` - the sequence of mean values for each channel, to be used when normalizing images. Defaults to `[0.5, 0.5, 0.5]`\n\n  * `:image_std` - the sequence of standard deviations for each channel, to be used when normalizing images. Defaults to `[0.5, 0.5, 0.5]`","ref":"Bumblebee.Vision.BitFeaturizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.BlipFeaturizer","doc":"BLIP featurizer for image data.","ref":"Bumblebee.Vision.BlipFeaturizer.html"},{"type":"module","title":"Configuration - Bumblebee.Vision.BlipFeaturizer","doc":"* `:resize` - whether to resize the input to the given `:size`. Defaults to `true`\n\n  * `:size` - the size to resize the input to, given as `%{height: ..., width: ...}`. Only has\n    an effect if `:resize` is `true`\n    . Defaults to `%{width: 384, height: 384}`\n\n  * `:resize_method` - the resizing method, either of `:nearest`, `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. Defaults to `:bicubic`\n\n  * `:normalize` - whether or not to normalize the input with mean and standard deviation. Defaults to `true`\n\n  * `:image_mean` - the sequence of mean values for each channel, to be used when normalizing images. Defaults to `[0.48145466, 0.4578275, 0.40821073]`\n\n  * `:image_std` - the sequence of standard deviations for each channel, to be used when normalizing images. Defaults to `[0.26862954, 0.26130258, 0.27577711]`","ref":"Bumblebee.Vision.BlipFeaturizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.ClipFeaturizer","doc":"CLIP featurizer for image data.","ref":"Bumblebee.Vision.ClipFeaturizer.html"},{"type":"module","title":"Configuration - Bumblebee.Vision.ClipFeaturizer","doc":"* `:resize` - whether to resize the input to the given `:size`. Defaults to `true`\n\n  * `:size` - the size to resize the input to, either `%{height: ..., width: ...}` or `%{shortest_edge: ...}`.\n    Only has an effect if `:resize` is `true`\n    . Defaults to `%{shortest_edge: 224}`\n\n  * `:resize_method` - the resizing method, either of `:nearest`, `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. Defaults to `:bicubic`\n\n  * `:center_crop` - whether to crop the input at the center. If the input size is smaller than `:crop_size` along\n    any edge, the image is padded with zeros and then center cropped\n    . Defaults to `true`\n\n  * `:crop_size` - the size to center crop the image to, given as `%{height: ..., width: ...}`. Only has an effect\n    if `:center_crop` is `true`\n    . Defaults to `%{width: 224, height: 224}`\n\n  * `:normalize` - whether or not to normalize the input with mean and standard deviation. Defaults to `true`\n\n  * `:image_mean` - the sequence of mean values for each channel, to be used when normalizing images. Defaults to `[0.485, 0.456, 0.406]`\n\n  * `:image_std` - the sequence of standard deviations for each channel, to be used when normalizing images. Defaults to `[0.229, 0.224, 0.225]`","ref":"Bumblebee.Vision.ClipFeaturizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.ConvNextFeaturizer","doc":"ConvNeXT featurizer for image data.","ref":"Bumblebee.Vision.ConvNextFeaturizer.html"},{"type":"module","title":"Configuration - Bumblebee.Vision.ConvNextFeaturizer","doc":"* `:resize` - whether to resize (and optionally center crop) the input to the given `:size`. Defaults to `true`\n\n  * `:size` - the size to resize the input to. If 384 or larger, the image is resized to (`:size`, `:size`).\n    Otherwise, the shorter edge of the image is matched to `:size` / `:crop_percentage`, then image\n    is cropped to `:size`. Only has an effect if `:resize` is `true`\n    . Defaults to `224`\n\n  * `:resize_method` - the resizing method, either of `:nearest`, `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. Defaults to `:bicubic`\n\n  * `:crop_percentage` - the percentage of the image to crop. Only has an effect if `:resize` is `true` and `:size` < 384. Defaults to `0.875`\n\n  * `:normalize` - whether or not to normalize the input with mean and standard deviation. Defaults to `true`\n\n  * `:image_mean` - the sequence of mean values for each channel, to be used when normalizing images. Defaults to `[0.485, 0.456, 0.406]`\n\n  * `:image_std` - the sequence of standard deviations for each channel, to be used when normalizing images. Defaults to `[0.229, 0.224, 0.225]`","ref":"Bumblebee.Vision.ConvNextFeaturizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.DeitFeaturizer","doc":"DeiT featurizer for image data.","ref":"Bumblebee.Vision.DeitFeaturizer.html"},{"type":"module","title":"Configuration - Bumblebee.Vision.DeitFeaturizer","doc":"* `:resize` - whether to resize the input to the given `:size`. Defaults to `true`\n\n  * `:size` - the size to resize the input to, given as `%{height: ..., width: ...}`. Only has\n    an effect if `:resize` is `true`\n    . Defaults to `%{width: 256, height: 256}`\n\n  * `:resize_method` - the resizing method, either of `:nearest`, `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. Defaults to `:bicubic`\n\n  * `:center_crop` - whether to crop the input at the center. If the input size is smaller than `:crop_size` along\n    any edge, the image is padded with zeros and then center cropped\n    . Defaults to `true`\n\n  * `:crop_size` - the size to center crop the image to, given as `%{height: ..., width: ...}`. Only has an effect\n    if `:center_crop` is `true`\n    . Defaults to `%{width: 224, height: 224}`\n\n  * `:normalize` - whether or not to normalize the input with mean and standard deviation. Defaults to `true`\n\n  * `:image_mean` - the sequence of mean values for each channel, to be used when normalizing images. Defaults to `[0.485, 0.456, 0.406]`\n\n  * `:image_std` - the sequence of standard deviations for each channel, to be used when normalizing images. Defaults to `[0.229, 0.224, 0.225]`","ref":"Bumblebee.Vision.DeitFeaturizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Vision.VitFeaturizer","doc":"ViT featurizer for image data.","ref":"Bumblebee.Vision.VitFeaturizer.html"},{"type":"module","title":"Configuration - Bumblebee.Vision.VitFeaturizer","doc":"* `:resize` - whether to resize the input to the given `:size`. Defaults to `true`\n\n  * `:size` - the size to resize the input to, given as `%{height: ..., width: ...}`. Only has\n    an effect if `:resize` is `true`\n    . Defaults to `%{width: 224, height: 224}`\n\n  * `:resize_method` - the resizing method, either of `:nearest`, `:bilinear`, `:bicubic`, `:lanczos3`, `:lanczos5`. Defaults to `:bilinear`\n\n  * `:normalize` - whether or not to normalize the input with mean and standard deviation. Defaults to `true`\n\n  * `:image_mean` - the sequence of mean values for each channel, to be used when normalizing images. Defaults to `[0.5, 0.5, 0.5]`\n\n  * `:image_std` - the sequence of standard deviations for each channel, to be used when normalizing images. Defaults to `[0.5, 0.5, 0.5]`","ref":"Bumblebee.Vision.VitFeaturizer.html#module-configuration"},{"type":"module","title":"Bumblebee.Diffusion.DdimScheduler","doc":"Denoising diffusion implicit models (DDIMs).\n\nThis sampling method was proposed as a follow up to the original\ndenoising diffusion probabilistic models (DDPMs) in order to heavily\nreduce the number of steps during inference. DDPMs model the diffusion\nprocess as a Markov chain; DDIMs generalize this considering\nnon-Markovian diffusion processes that lead to the same objective.\nThis enables a reverse process with many less samples, as compared\nto DDPMs, while using the same denoising model.\n\nDDIMs were shown to be a simple variant of pseudo numerical methods\nfor diffusion models (PNDMs), see `Bumblebee.Diffusion.PndmScheduler`\nand the corresponding paper for more details.","ref":"Bumblebee.Diffusion.DdimScheduler.html"},{"type":"module","title":"Configuration - Bumblebee.Diffusion.DdimScheduler","doc":"* `:num_train_steps` - the number of diffusion steps used to train the model. Defaults to `1000`\n\n  * `:beta_schedule` - the beta schedule type, a mapping from a beta range to a sequence of betas for stepping the model.\n    Either of `:linear`, `:quadratic`, or `:squared_cosine`\n    . Defaults to `:linear`\n\n  * `:beta_start` - the start value for the beta schedule. Defaults to `0.0001`\n\n  * `:beta_end` - the end value for the beta schedule. Defaults to `0.02`\n\n  * `:prediction_type` - prediction type of the denoising model. Either of:\n    \n      * `:noise` (default) - the model predicts the noise of the diffusion process\n    \n      * `:angular_velocity` - the model predicts velocity in angular parameterization.\n        See Section 2.4 in [Imagen Video: High Definition Video Generation with Diffusion Models](https://imagen.research.google/video/paper.pdf),\n        then Section 4 in [Progressive Distillation for Fast Sampling of Diffusion Models](https://arxiv.org/pdf/2202.00512.pdf)\n        and Appendix D\n    \n    . Defaults to `:noise`\n\n  * `:alpha_clip_strategy` - each step $t$ uses the values of $\\bar{\\alpha}\\_t$ and $\\bar{\\alpha}\\_{t-1}$,\n    however for $t = 0$ there is no previous alpha. The strategy can be either\n    `:one` ($\\bar{\\alpha}\\_{t-1} = 1$) or `:alpha_zero` ($\\bar{\\alpha}\\_{t-1} = \\bar{\\alpha}\\_0$)\n    . Defaults to `:one`\n\n  * `:timesteps_offset` - an offset added to the inference steps. You can use a combination of `timesteps_offset: 1` and\n    `alpha_clip_strategy: :alpha_zero`, so that the last step $t = 1$ uses $\\bar{\\alpha}\\_1$\n    and $\\bar{\\alpha}\\_0$, as done in stable diffusion\n    . Defaults to `0`\n\n  * `:clip_denoised_sample` - whether to clip the predicted denoised sample ($x_0$ in Equation (12)) into $[-1, 1]$\n    for numerical stability\n    . Defaults to `true`\n\n  * `:rederive_noise` - whether the noise (output of the denoising model) should be re-derived at each step based on the\n    predicted denoised sample ($x_0$) and the current sample. This technique is used in OpenAI GLIDE\n    . Defaults to `false`\n\n  * `:eta` - a weight for the noise added in a denoising diffusion step. This scales the value of $\\sigma_t$\n    in Equation (12) in the original paper, as per Equation (16)\n    . Defaults to `0.0`","ref":"Bumblebee.Diffusion.DdimScheduler.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Diffusion.DdimScheduler","doc":"* [Denoising Diffusion Implicit Models](https://arxiv.org/abs/2010.02502)","ref":"Bumblebee.Diffusion.DdimScheduler.html#module-references"},{"type":"module","title":"Bumblebee.Diffusion.LcmScheduler","doc":"Latent Consistency Model (LCM) sampling.\n\nThis sampling method should be used in combination with LCM. LCM is\na model distilled from a regular diffusion model to predict the\nfinal denoised sample in a single step. The sample quality can be\nimproved by alternating a couple denoising and noise injection\nsteps (multi-step sampling), as per Appendix B.","ref":"Bumblebee.Diffusion.LcmScheduler.html"},{"type":"module","title":"Configuration - Bumblebee.Diffusion.LcmScheduler","doc":"* `:num_train_steps` - the number of diffusion steps used to train the model. Defaults to `1000`\n\n  * `:beta_schedule` - the beta schedule type, a mapping from a beta range to a sequence of betas for stepping the model.\n    Either of `:linear`, `:quadratic`, or `:squared_cosine`\n    . Defaults to `:quadratic`\n\n  * `:beta_start` - the start value for the beta schedule. Defaults to `8.5e-4`\n\n  * `:beta_end` - the end value for the beta schedule. Defaults to `0.012`\n\n  * `:prediction_type` - prediction type of the denoising model. Either of:\n    \n      * `:noise` (default) - the model predicts the noise of the diffusion process\n    \n      * `:angular_velocity` - the model predicts velocity in angular parameterization.\n        See Section 2.4 in [Imagen Video: High Definition Video Generation with Diffusion Models](https://imagen.research.google/video/paper.pdf),\n        then Section 4 in [Progressive Distillation for Fast Sampling of Diffusion Models](https://arxiv.org/pdf/2202.00512.pdf)\n        and Appendix D\n    \n    . Defaults to `:noise`\n\n  * `:alpha_clip_strategy` - each step $t$ uses the values of $\\bar{\\alpha}\\_t$ and $\\bar{\\alpha}\\_{t-1}$,\n    however for $t = 0$ there is no previous alpha. The strategy can be either\n    `:one` ($\\bar{\\alpha}\\_{t-1} = 1$) or `:alpha_zero` ($\\bar{\\alpha}\\_{t-1} = \\bar{\\alpha}\\_0$)\n    . Defaults to `:one`\n\n  * `:clip_denoised_sample` - whether to clip the predicted denoised sample ($x_0$ in Equation (12)) into $[-1, 1]$\n    for numerical stability\n    . Defaults to `false`\n\n  * `:num_original_steps` - the number of denoising steps used during Latent Consistency Distillation (LCD).\n    The LCD procedure distills a base diffusion model, but instead of sampling all\n    `:num_train_steps` it skips steps and uses another scheduler accordingly. See\n    Section 4.3\n    . Defaults to `50`\n\n  * `:boundary_condition_timestep_scale` - the scaling factor used in the consistency function coefficients. In the original\n    LCM implementation the authors use the formulation\n    $$\n    c_{skip}(t) = \\frac{\\sigma_{data}^2}{(st)^2 + \\sigma_{data}^2}, \\quad\n    c_{out}(t) = \\frac{st}{\\sqrt{(st)^2 + \\sigma_{data}^2}}\n    $$\n    where $\\sigma_{data} = 0.5$ and $s$ is the scaling factor. Increasing the scaling\n    factor will decrease approximation error, although the approximation error at the\n    default of `10.0` is already pretty small\n    . Defaults to `10.0`","ref":"Bumblebee.Diffusion.LcmScheduler.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Diffusion.LcmScheduler","doc":"* [Latent Consistency Models: Synthesizing High-Resolution Images with Few-Step Inference](https://arxiv.org/abs/2310.04378)\n  * [Consistency Models](https://arxiv.org/pdf/2303.01469.pdf)","ref":"Bumblebee.Diffusion.LcmScheduler.html#module-references"},{"type":"module","title":"Bumblebee.Diffusion.PndmScheduler","doc":"Pseudo numerical methods for diffusion models (PNDMs).\n\nThe sampling is based on two numerical methods for solving ODE: the\nRunge-Kutta method (RK) and the linear multi-step method (LMS). The\ngradient at each step is computed according to either of these methods,\nhowever the transfer part (approximating the next sample based on\ncurrent sample and gradient) is non-linear. Because of this property,\nthe authors of the paper refer to them as pseudo numerical methods,\ndenoted as PRK and PLMS respectively.","ref":"Bumblebee.Diffusion.PndmScheduler.html"},{"type":"module","title":"Configuration - Bumblebee.Diffusion.PndmScheduler","doc":"* `:num_train_steps` - the number of diffusion steps used to train the model. Defaults to `1000`\n\n  * `:beta_schedule` - the beta schedule type, a mapping from a beta range to a sequence of betas for stepping the model.\n    Either of `:linear`, `:quadratic`, or `:squared_cosine`\n    . Defaults to `:linear`\n\n  * `:beta_start` - the start value for the beta schedule. Defaults to `0.0001`\n\n  * `:beta_end` - the end value for the beta schedule. Defaults to `0.02`\n\n  * `:prediction_type` - prediction type of the denoising model. Either of:\n    \n      * `:noise` (default) - the model predicts the noise of the diffusion process\n    \n      * `:angular_velocity` - the model predicts velocity in angular parameterization.\n        See Section 2.4 in [Imagen Video: High Definition Video Generation with Diffusion Models](https://imagen.research.google/video/paper.pdf),\n        then Section 4 in [Progressive Distillation for Fast Sampling of Diffusion Models](https://arxiv.org/pdf/2202.00512.pdf)\n        and Appendix D\n    \n    . Defaults to `:noise`\n\n  * `:alpha_clip_strategy` - each step $t$ uses the values of $\\bar{\\alpha}\\_t$ and $\\bar{\\alpha}\\_{t-1}$,\n    however for $t = 0$ there is no previous alpha. The strategy can be either\n    `:one` ($\\bar{\\alpha}\\_{t-1} = 1$) or `:alpha_zero` ($\\bar{\\alpha}\\_{t-1} = \\bar{\\alpha}\\_0$)\n    . Defaults to `:alpha_zero`\n\n  * `:timesteps_offset` - an offset added to the inference steps. You can use a combination of `timesteps_offset: 1` and\n    `alpha_clip_strategy: :alpha_zero`, so that the last step $t = 1$ uses $\\bar{\\alpha}\\_1$\n    and $\\bar{\\alpha}\\_0$, as done in stable diffusion\n    . Defaults to `0`\n\n  * `:reduce_warmup` - when `true`, the first few samples are computed using lower-order linear multi-step,\n    rather than the Runge-Kutta method. This results in less forward passes of the model\n    . Defaults to `false`","ref":"Bumblebee.Diffusion.PndmScheduler.html#module-configuration"},{"type":"module","title":"References - Bumblebee.Diffusion.PndmScheduler","doc":"* [Pseudo Numerical Methods for Diffusion Models on Manifolds](https://arxiv.org/abs/2202.09778)","ref":"Bumblebee.Diffusion.PndmScheduler.html#module-references"},{"type":"behaviour","title":"Bumblebee.Configurable","doc":"An interface for configurable entities.\n\nA module implementing this behaviour is expected to define a struct\nwith configuration.","ref":"Bumblebee.Configurable.html"},{"type":"callback","title":"Bumblebee.Configurable.config/2","doc":"Configures the struct.","ref":"Bumblebee.Configurable.html#c:config/2"},{"type":"type","title":"Bumblebee.Configurable.t/0","doc":"","ref":"Bumblebee.Configurable.html#t:t/0"},{"type":"behaviour","title":"Bumblebee.Featurizer","doc":"An interface for configuring and applying featurizers.\n\nA featurizer is used to convert raw data into model input.\n\nEvery module implementing this behaviour is expected to also define\na configuration struct.","ref":"Bumblebee.Featurizer.html"},{"type":"function","title":"Bumblebee.Featurizer.batch_template/2","doc":"Returns an input template for `process_batch/2`.\n\nIf the featurizer does not define batch processing, `nil` is returned.","ref":"Bumblebee.Featurizer.html#batch_template/2"},{"type":"callback","title":"Bumblebee.Featurizer.batch_template/2","doc":"Returns an input template for `c:process_batch/2`.\n\nThe shape is effectively the same as the result of `c:process_input/2`,\nexcept for the batch size.","ref":"Bumblebee.Featurizer.html#c:batch_template/2"},{"type":"function","title":"Bumblebee.Featurizer.process_batch/2","doc":"Optional batch processing stage.\n\nThis is a numerical function. It receives the result of `c:process_input/2`,\nexcept the batch size may differ.\n\nIf the featurizer does not define batch processing, the input is\nreturned as is.","ref":"Bumblebee.Featurizer.html#process_batch/2"},{"type":"callback","title":"Bumblebee.Featurizer.process_batch/2","doc":"Optional batch processing stage.\n\nThis is a numerical function. It receives the result of `c:process_input/2`,\nexcept the batch size may differ.\n\nWhen using featurizer as part of `Nx.Serving`, the batch stage can\nbe merged with the model computation and compiled together.","ref":"Bumblebee.Featurizer.html#c:process_batch/2"},{"type":"function","title":"Bumblebee.Featurizer.process_input/2","doc":"Converts the given input to a batched tensor (or a tensor container).","ref":"Bumblebee.Featurizer.html#process_input/2"},{"type":"callback","title":"Bumblebee.Featurizer.process_input/2","doc":"Converts the given input to a batched tensor (or a tensor container).\n\nNumerical batch processing should be moved to `c:process_batch/2`\nwhenever possible.","ref":"Bumblebee.Featurizer.html#c:process_input/2"},{"type":"type","title":"Bumblebee.Featurizer.t/0","doc":"","ref":"Bumblebee.Featurizer.html#t:t/0"},{"type":"behaviour","title":"Bumblebee.ModelSpec","doc":"An interface for configuring and building models based on the same\narchitecture.\n\nEvery module implementing this behaviour is expected to also define\na configuration struct.","ref":"Bumblebee.ModelSpec.html"},{"type":"callback","title":"Bumblebee.ModelSpec.architectures/0","doc":"Returns the list of supported model architectures.","ref":"Bumblebee.ModelSpec.html#c:architectures/0"},{"type":"callback","title":"Bumblebee.ModelSpec.input_template/1","doc":"Builds a template input for the model.\n\nThe template is used to compile the model when initializing parameters.","ref":"Bumblebee.ModelSpec.html#c:input_template/1"},{"type":"callback","title":"Bumblebee.ModelSpec.model/1","doc":"Builds an `Axon` model according to the given configuration.","ref":"Bumblebee.ModelSpec.html#c:model/1"},{"type":"type","title":"Bumblebee.ModelSpec.t/0","doc":"","ref":"Bumblebee.ModelSpec.html#t:t/0"},{"type":"behaviour","title":"Bumblebee.Scheduler","doc":"An interface for configuring and using schedulers.\n\nA scheduler defines a sampling method, usually used for multi-step\ndenoising process, as in stable diffusion.\n\nEvery module implementing this behaviour is expected to also define\na configuration struct.","ref":"Bumblebee.Scheduler.html"},{"type":"behaviour","title":"Context - Bumblebee.Scheduler","doc":"Imagine a denoising model trained in 1000 steps. During training,\nwe take some original data and add random noise 1000 times, this\nway we obtain 1000 steps with increasing level of noise. Then, the\nmodel learns to predict noise at each timestep, given data at that\nstep (sample) and the timestep.\n\nOnce such model is trained, we can obtain brand new data (such as\nimage) by generating random data and denoising it with our model in\n1000 steps.\n\nDoing 1000 forward passes of the model for a single generation can\nbe expensive, hence multiple methods have been developed to reduce\nthe number of steps during denoising, with no changes to the model.\n\nEach method specifies a subset of the original timesteps, at each\ntimestep we need to do a forward pass of the model (or possibly a\nfew), then the method extrapolates the sample to the next selected\ntimestep, possibly skipping a lot of timesteps in between.","ref":"Bumblebee.Scheduler.html#module-context"},{"type":"behaviour","title":"Note on wording - Bumblebee.Scheduler","doc":"Throughout the docs and APIs the word \"steps\" refers to diffusion\nsteps, whereas \"timesteps\" is more specific and refers to the exact\nvalues $t$ (points in time).","ref":"Bumblebee.Scheduler.html#module-note-on-wording"},{"type":"callback","title":"Bumblebee.Scheduler.init/4","doc":"Initializes state for a new scheduler loop.\n\nReturns a pair of `{state, timesteps}`, where `state` is an opaque\n`Nx.Container` and `timesteps` is a tensor with the subsequent\ntimesteps for model forward pass.","ref":"Bumblebee.Scheduler.html#c:init/4"},{"type":"callback","title":"Bumblebee.Scheduler.step/4","doc":"Predicts sample at the previous timestep.\n\nTakes the current `sample` and `prediction` (usually noise) returned\nby the model at the current timestep. Returns `{state, prev_sample}`,\nwhere `state` is the updated state and `prev_sample` is the predicted\nsample at the previous timestep.","ref":"Bumblebee.Scheduler.html#c:step/4"},{"type":"type","title":"Bumblebee.Scheduler.state/0","doc":"","ref":"Bumblebee.Scheduler.html#t:state/0"},{"type":"type","title":"Bumblebee.Scheduler.t/0","doc":"","ref":"Bumblebee.Scheduler.html#t:t/0"},{"type":"behaviour","title":"Bumblebee.Text.Generation","doc":"An interface for language models supporting sequence generation.","ref":"Bumblebee.Text.Generation.html"},{"type":"function","title":"Bumblebee.Text.Generation.build_generate/4","doc":"Builds a numerical definition that generates sequences of tokens using\nthe given language model.\n\nThe model should be either a decoder or an encoder-decoder. The tokens\nare generated by iterative inference using the decoder (autoregression),\nuntil the termination criteria are met.\n\nIn case of encoder-decoder models, the corresponding encoder is run\nonly once and the intermediate state is reused during all iterations.\n\nThe generation is controlled by a number of options given as\n`%Bumblebee.Text.GenerationConfig{}`, see the corresponding docs\nfor more details.\n\nReturns a defn JIT-compatible anonymous function, which expects the\nmodel params as the first argument and inputs map as the second\nargument. Note that the inputs map should additionally include a\n`\"seed\"` tensor, with one value per input in the batch.","ref":"Bumblebee.Text.Generation.html#build_generate/4"},{"type":"function","title":"Streaming - Bumblebee.Text.Generation.build_generate/4","doc":"This function sets up a hook that is invoked after every generated\ntoken. The hook receives a map with the following attributes:\n\n  * `:token_id` - the newly generated token\n\n  * `:finished?` - a boolean indicating if the sequence is finished\n\n  * `:length` - the current length of the generated sequence. Once\n    the sequence is finished, the length does not increase\n\nEach of the attributes is a tensor with a leading batch dimension.\n\nWhen streaming you may not care about the output result, in which\ncase you can enable `:ignore_output` to reduce the output size.","ref":"Bumblebee.Text.Generation.html#build_generate/4-streaming"},{"type":"function","title":"Options - Bumblebee.Text.Generation.build_generate/4","doc":"* `:logits_processors` - a list of numerical functions to modify\n    predicted scores at each generation step. The functions are\n    applied in order, after all default processors\n\n  * `:ignore_output` - if true, returns a dummy tensor that should\n    be ignored. This is useful when you consume the generated tokens\n    in a stream fashion via the hook, so that the full output does\n    not need to be transferred unnecessarily after the computation.\n    Defaults to `false`","ref":"Bumblebee.Text.Generation.html#build_generate/4-options"},{"type":"function","title":"Bumblebee.Text.Generation.extra_config_module/1","doc":"Returns a configuration module for extra model-specific generation\nattributes to extend the base `Bumblebee.Text.GenerationConfig`.","ref":"Bumblebee.Text.Generation.html#extra_config_module/1"},{"type":"callback","title":"Bumblebee.Text.Generation.extra_config_module/1","doc":"Returns a configuration module for extra model-specific generation\nattributes to extend the base `Bumblebee.Text.GenerationConfig`.","ref":"Bumblebee.Text.Generation.html#c:extra_config_module/1"},{"type":"function","title":"Bumblebee.Text.Generation.init_cache/4","doc":"Initializes an opaque cache input for iterative inference.","ref":"Bumblebee.Text.Generation.html#init_cache/4"},{"type":"callback","title":"Bumblebee.Text.Generation.init_cache/4","doc":"Initializes an opaque cache input for iterative inference.","ref":"Bumblebee.Text.Generation.html#c:init_cache/4"},{"type":"function","title":"Bumblebee.Text.Generation.traverse_cache/3","doc":"Calls `fun` for every batched tensor in the cache.","ref":"Bumblebee.Text.Generation.html#traverse_cache/3"},{"type":"callback","title":"Bumblebee.Text.Generation.traverse_cache/3","doc":"Traverses all batched tensors in the cache.\n\nThis function is used when the cache needs to be inflated or\ndeflated for a different batch size.","ref":"Bumblebee.Text.Generation.html#c:traverse_cache/3"},{"type":"type","title":"Bumblebee.Text.Generation.cache/0","doc":"","ref":"Bumblebee.Text.Generation.html#t:cache/0"},{"type":"behaviour","title":"Bumblebee.Tokenizer","doc":"An interface for configuring and applying tokenizers.\n\nA tokenizer is used to convert raw text data into model input.\n\nEvery module implementing this behaviour is expected to also define\na configuration struct.","ref":"Bumblebee.Tokenizer.html"},{"type":"callback","title":"Bumblebee.Tokenizer.additional_special_tokens/1","doc":"Returns a list with extra special tokens, in addition to the named\n`special_tokens/1`.","ref":"Bumblebee.Tokenizer.html#c:additional_special_tokens/1"},{"type":"function","title":"Bumblebee.Tokenizer.all_special_tokens/1","doc":"Returns all special tokens, including any extra tokens.","ref":"Bumblebee.Tokenizer.html#all_special_tokens/1"},{"type":"callback","title":"Bumblebee.Tokenizer.apply/2","doc":"Performs tokenization and encoding on the given input.","ref":"Bumblebee.Tokenizer.html#c:apply/2"},{"type":"function","title":"Bumblebee.Tokenizer.decode/2","doc":"Decodes a list of token ids into a sentence.","ref":"Bumblebee.Tokenizer.html#decode/2"},{"type":"callback","title":"Bumblebee.Tokenizer.decode/2","doc":"Decodes a list of token ids into a sentence.","ref":"Bumblebee.Tokenizer.html#c:decode/2"},{"type":"function","title":"Bumblebee.Tokenizer.id_to_token/2","doc":"Converts the given token id to the corresponding token.","ref":"Bumblebee.Tokenizer.html#id_to_token/2"},{"type":"callback","title":"Bumblebee.Tokenizer.id_to_token/2","doc":"Converts the given token id the corresponding token.","ref":"Bumblebee.Tokenizer.html#c:id_to_token/2"},{"type":"function","title":"Bumblebee.Tokenizer.special_token/2","doc":"Returns a special token by name.","ref":"Bumblebee.Tokenizer.html#special_token/2"},{"type":"function","title":"Bumblebee.Tokenizer.special_token_id/2","doc":"Returns id of a special token by name.","ref":"Bumblebee.Tokenizer.html#special_token_id/2"},{"type":"callback","title":"Bumblebee.Tokenizer.special_tokens/1","doc":"Returns a map with special tokens.","ref":"Bumblebee.Tokenizer.html#c:special_tokens/1"},{"type":"function","title":"Bumblebee.Tokenizer.token_to_id/2","doc":"Converts the given token into the corresponding numeric id.","ref":"Bumblebee.Tokenizer.html#token_to_id/2"},{"type":"callback","title":"Bumblebee.Tokenizer.token_to_id/2","doc":"Converts the given token into the corresponding numeric id.","ref":"Bumblebee.Tokenizer.html#c:token_to_id/2"},{"type":"type","title":"Bumblebee.Tokenizer.input/0","doc":"","ref":"Bumblebee.Tokenizer.html#t:input/0"},{"type":"type","title":"Bumblebee.Tokenizer.special_token_type/0","doc":"A type corresponding to a special token in the vocabulary.","ref":"Bumblebee.Tokenizer.html#t:special_token_type/0"},{"type":"type","title":"Common types - Bumblebee.Tokenizer.special_token_type/0","doc":"* `:bos` - a token representing the beginning of a sentence\n\n  * `:eos` - a token representing the end of a sentence\n\n  * `:unk` - a token representing an out-of-vocabulary token\n\n  * `:sep` - a token separating two different sentences in the same\n    input\n\n  * `:pad` - a token added when processing a batch of sequences with\n    different length\n\n  * `:cls` - a token representing the class of the input\n\n  * `:mask` - a token representing a masked token, used for masked\n    language modeling tasks","ref":"Bumblebee.Tokenizer.html#t:special_token_type/0-common-types"},{"type":"type","title":"Bumblebee.Tokenizer.t/0","doc":"","ref":"Bumblebee.Tokenizer.html#t:t/0"},{"type":"type","title":"Bumblebee.Tokenizer.token/0","doc":"","ref":"Bumblebee.Tokenizer.html#t:token/0"},{"type":"type","title":"Bumblebee.Tokenizer.token_id/0","doc":"","ref":"Bumblebee.Tokenizer.html#t:token_id/0"},{"type":"module","title":"Bumblebee.Text.GenerationConfig","doc":"A set of configuration options controlling text generation.\n\nThis struct is expected by `Bumblebee.Text.Generation.build_generate/3`.","ref":"Bumblebee.Text.GenerationConfig.html"},{"type":"module","title":"Configuration - Bumblebee.Text.GenerationConfig","doc":"","ref":"Bumblebee.Text.GenerationConfig.html#module-configuration"},{"type":"module","title":"Options controlling length - Bumblebee.Text.GenerationConfig","doc":"* `:max_new_tokens` - the maximum number of tokens to be generated, ignoring the number of tokens in the prompt. Defaults to `20`\n\n  * `:min_new_tokens` - the minimum number of tokens to be generated, ignoring the number of tokens in the prompt\n\n  * `:max_length` - the maximum length of the sequence to be generated. Note that this length includes the\n    length of the input prompt (including padding). In general, prefer `:max_new_tokens`,\n    which ignores the number of tokens in the prompt\n    \n\n  * `:min_length` - the minimum length of the sequence to be generated. Note that this length includes the\n    length of the input prompt (including padding). In general, prefer `:min_new_tokens`,\n    which ignores the number of tokens in the prompt","ref":"Bumblebee.Text.GenerationConfig.html#module-options-controlling-length"},{"type":"module","title":"Options controlling strategy - Bumblebee.Text.GenerationConfig","doc":"* `:strategy` - the method deciding how tokens are selected, it has a significant impact on the quality\n    of the generated sequence. Should be a map with `:type` and strategy-specific options.\n    \n      * `:greedy_search` - the most straightforward approach, where in\n        every iteration the most probable token (as given by the model)\n        is taken.\n    \n        Example: `%{type: :greedy_search}`.\n    \n      * `:contrastive_search` - state-of-the-art decoding method, capable\n        of producing high quality, coherent sequences. The results are\n        deterministic. See [this article](https://huggingface.co/blog/introducing-csearch)\n        for more details.\n    \n          * `:top_k` (required) - the number of highest probability vocabulary tokens considered\n            as a continuation\n    \n          * `:alpha` (required) - the weight of degeneration penalty. It balances the model\n            confidence and the penalty\n    \n        Example: `%{type: :contrastive_search, top_k: 4, alpha: 0.6}`.\n    \n      * `:multinomial_sampling` - this method samples tokens according to the probability\n        distribution given by the model. The results are nondeterministic, unless a seed\n        is specified.\n    \n          * `:top_k` (optional) - when specified, restricts sampling to top-k most probable\n            candidates\n    \n          * `:top_p` (optional) - when specified, restricts sampling to tokens which probabilities\n            add up to top-p\n    . Defaults to `%{type: :greedy_search}`","ref":"Bumblebee.Text.GenerationConfig.html#module-options-controlling-strategy"},{"type":"module","title":"Options controlling generated tokens - Bumblebee.Text.GenerationConfig","doc":"* `:decoder_start_token_id` - the id of the initial token when generating from scratch, in case of encoder-decoder models\n\n  * `:forced_bos_token_id` - the id of the token to force as the first generated token\n\n  * `:forced_eos_token_id` - the id of the token to force as the last generated token when `:max_length` is reached\n\n  * `:forced_token_ids` - a list of `{index, token_id}` pairs forcing `token_id` to appear at `index` in the generated sequence. Defaults to `[]`\n\n  * `:suppressed_token_ids` - a list of token ids to suppress during generation. Defaults to `[]`\n\n  * `:no_repeat_ngram_length` - when set, n-grams of the given length can occur only once in the generated sequence\n\n  * `:temperature` - enables exponential scaling of the output probability distribution. The temperature value effectively\n    determines the randomness of the predicted tokens. Values smaller than 1.0 decrease the randomness,\n    while bigger values increase it. Note that this is only relevant for generation `:strategy` that does\n    sampling based on the output probability distribution","ref":"Bumblebee.Text.GenerationConfig.html#module-options-controlling-generated-tokens"},{"type":"module","title":"Special tokens used during generation - Bumblebee.Text.GenerationConfig","doc":"* `:bos_token_id` - the id of the beginning-of-sequence token\n\n  * `:eos_token_id` - the id of the end-of-sequence token. This can also be a list, in case multiple tokens should be recognized as EOS\n\n  * `:pad_token_id` - the id of the padding token","ref":"Bumblebee.Text.GenerationConfig.html#module-special-tokens-used-during-generation"},{"type":"type","title":"Bumblebee.Text.GenerationConfig.t/0","doc":"","ref":"Bumblebee.Text.GenerationConfig.html#t:t/0"},{"type":"module","title":"Bumblebee.Text.WhisperGenerationConfig","doc":"A set of Whisper-specific configuration options controlling text\ngeneration.\n\nThis struct is used in the `Bumblebee.Text.GenerationConfig` struct\nunder the `:extra_config` attribute.","ref":"Bumblebee.Text.WhisperGenerationConfig.html"},{"type":"module","title":"Configuration - Bumblebee.Text.WhisperGenerationConfig","doc":"* `:no_timestamps_token_id` - the id of the no-timestamps token\n\n  * `:language_to_token_id` - a map from language code to token id corresponding to that language. Defaults to `%{}`\n\n  * `:task_to_token_id` - a map from task to token id corresponding to that task. Defaults to `%{}`","ref":"Bumblebee.Text.WhisperGenerationConfig.html#module-configuration"},{"type":"type","title":"Bumblebee.Text.WhisperGenerationConfig.t/0","doc":"","ref":"Bumblebee.Text.WhisperGenerationConfig.html#t:t/0"},{"type":"extras","title":"Examples","doc":"# Examples\n\n```elixir\nMix.install([\n  {:bumblebee, \"~> 0.6.0\"},\n  {:nx, \"~> 0.9.0\"},\n  {:exla, \"~> 0.9.0\"},\n  {:axon, \"~> 0.7.0\"},\n  {:kino, \"~> 0.14.0\"}\n])\n\nNx.global_default_backend(EXLA.Backend)\n```","ref":"examples.html"},{"type":"extras","title":"Introduction - Examples","doc":"In this notebook we go through a number of examples to get a quick overview of what Bumblebee brings to the table.\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#introduction"},{"type":"extras","title":"Image classification - Examples","doc":"Let's start with image classification. First, we load a pre-trained [ResNet-50](https://huggingface.co/microsoft/resnet-50) model from a HuggingFace repository. We also load the corresponding featurizer for preprocessing input images.\n\n```elixir\n{:ok, resnet} = Bumblebee.load_model({:hf, \"microsoft/resnet-50\"})\n{:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"microsoft/resnet-50\"})\n\n:ok\n```\n\nNext, we use the high-level API to build an end-to-end task definition on top of the model we just loaded. We will also need an image to work with, so let's show an image input.\n\n```elixir\nserving = Bumblebee.Vision.image_classification(resnet, featurizer)\n\nimage_input = Kino.Input.image(\"Image\", size: {224, 224})\n```\n\nBumblebee implements end-to-end tasks using `Nx.Serving`. With serving we can choose to either do a one-off run, or to start a supervised process that automatically batches multiple inference requests. Thanks to this abstraction we can do quick experimentation and then plug the task into a production app with minimal effort.\n\nIn this case we will do the one-off run for the selected image:\n\n```elixir\nimage = Kino.Input.read(image_input)\n\n# Build a tensor from the raw pixel data\nimage =\n  image.file_ref\n  |> Kino.Input.file_path()\n  |> File.read!()\n  |> Nx.from_binary(:u8)\n  |> Nx.reshape({image.height, image.width, 3})\n\nNx.Serving.run(serving, image)\n```","ref":"examples.html#image-classification"},{"type":"extras","title":"Manual inference - Examples","doc":"<!-- livebook:{\"break_markdown\":true} -->\n\nNote that we are dealing with regular `Axon` models and the high-level API is just a convenience. If you need full control over the inference flow, you can do it manually. In this case, we would pass the image through the featurizer to get normalized model inputs, then we would run the model and finally extract the most probable label.\n\n```elixir\ninputs = Bumblebee.apply_featurizer(featurizer, image)\noutputs = Axon.predict(resnet.model, resnet.params, inputs)\n\nid = outputs.logits |> Nx.argmax() |> Nx.to_number()\nresnet.spec.id_to_label[id]\n```\n\nYou can try a number of other models, just replace the repository id with one of these:\n\n* [**facebook/convnext-tiny-224**](https://huggingface.co/facebook/convnext-tiny-224) (ConvNeXT)\n\n* [**google/vit-base-patch16-224**](https://huggingface.co/google/vit-base-patch16-224) (ViT)\n\n* [**facebook/deit-base-distilled-patch16-224**](https://huggingface.co/facebook/deit-base-distilled-patch16-224) (DeiT)\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#manual-inference"},{"type":"extras","title":"Fill-mask - Examples","doc":"Now time for some text processing. Specifically, we want to fill in the missing word in a sentence. This time we load the [BERT](https://huggingface.co/bert-base-uncased) model together with a matching tokenizer. We will use the tokenizer to preprocess our text input.\n\n```elixir\n{:ok, bert} = Bumblebee.load_model({:hf, \"google-bert/bert-base-uncased\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-uncased\"})\n\nserving = Bumblebee.Text.fill_mask(bert, tokenizer)\n\ntext_input = Kino.Input.text(\"Sentence with mask\", default: \"The capital of [MASK] is Paris.\")\n```\n\n```elixir\ntext = Kino.Input.read(text_input)\n\nNx.Serving.run(serving, text)\n```\n\nAgain, you can try other models, such as [**albert-base-v2**](https://huggingface.co/albert-base-v2) or [**roberta-base**](https://huggingface.co/roberta-base).\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#fill-mask"},{"type":"extras","title":"Text classification - Examples","doc":"In this example we will analyze text sentiment.\n\nWe will use the [BERTweet](https://huggingface.co/finiteautomata/bertweet-base-sentiment-analysis) model, trained to classify text into one of three categories: positive (POS), negative (NEG) or neutral (NEU).\n\n```elixir\n{:ok, bertweet} = Bumblebee.load_model({:hf, \"finiteautomata/bertweet-base-sentiment-analysis\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"vinai/bertweet-base\"})\n\nserving = Bumblebee.Text.text_classification(bertweet, tokenizer)\n\ntext_input = Kino.Input.text(\"Text\", default: \"This cat is so cute.\")\n```\n\n*Note: this time we need to load a matching tokenizer from a different repository.*\n\n```elixir\ntext = Kino.Input.read(text_input)\nNx.Serving.run(serving, text)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#text-classification"},{"type":"extras","title":"Named-entity recognition - Examples","doc":"In this section we look at token classification, more specifically named-entity recognition (NER), where the objective is to identify and categorize entities in text. We will once again use a fine-tuned [BERT](https://huggingface.co/dslim/bert-base-NER) model.\n\n```elixir\n{:ok, bert} = Bumblebee.load_model({:hf, \"dslim/bert-base-NER\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-cased\"})\n\nserving = Bumblebee.Text.token_classification(bert, tokenizer, aggregation: :same)\n\ntext_input =\n  Kino.Input.text(\"Text\",\n    default: \"Rachel Green works at Ralph Lauren in New York City in the sitcom Friends\"\n  )\n```\n\n```elixir\ntext = Kino.Input.read(text_input)\nNx.Serving.run(serving, text)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#named-entity-recognition"},{"type":"extras","title":"Text generation - Examples","doc":"Generation is where things get even more exciting. In this section we will use the extremely popular [GPT-2](https://huggingface.co/gpt2) model to generate text continuation.\n\nGeneration generally is an iterative process, where the model predicts the sentence token by token, adhering to some constraints. Again, we will make use of a higher-level API based on `Nx.Serving`.\n\n```elixir\n{:ok, gpt2} = Bumblebee.load_model({:hf, \"openai-community/gpt2\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai-community/gpt2\"})\n{:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai-community/gpt2\"})\n\nserving = Bumblebee.Text.generation(gpt2, tokenizer, generation_config)\n\ntext_input = Kino.Input.text(\"Text\", default: \"Yesterday, I was reading a book and\")\n```\n\n```elixir\ntext = Kino.Input.read(text_input)\nNx.Serving.run(serving, text)\n```\n\nThere is also [gpt2-medium](https://huggingface.co/gpt2-medium) and [gpt2-large](https://huggingface.co/gpt2-large) - heavier versions of the model with much more parameters.\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"examples.html#text-generation"},{"type":"extras","title":"Question answering - Examples","doc":"Another text-related task is question answering, where the objective is to retrieve the answer to a question based on a given text. We will work with a [RoBERTa](https://huggingface.co/deepset/roberta-base-squad2) model trained to do just that.\n\n```elixir\n{:ok, roberta} = Bumblebee.load_model({:hf, \"deepset/roberta-base-squad2\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"FacebookAI/roberta-base\"})\n\nserving = Bumblebee.Text.question_answering(roberta, tokenizer)\n\nquestion_input =\n  Kino.Input.text(\"Question\",\n    default: \"Which name is also used to describe the Amazon rainforest in English?\"\n  )\n\ncontext_input =\n  Kino.Input.textarea(\"Context\",\n    default:\n      ~s/The Amazon rainforest (Portuguese: Floresta Amazônica or Amazônia; Spanish: Selva Amazónica, Amazonía or usually Amazonia; French: Forêt amazonienne; Dutch: Amazoneregenwoud), also known in English as Amazonia or the Amazon Jungle, is a moist broadleaf forest that covers most of the Amazon basin of South America. This basin encompasses 7,000,000 square kilometres (2,700,000 sq mi), of which 5,500,000 square kilometres (2,100,000 sq mi) are covered by the rainforest. This region includes territory belonging to nine nations. The majority of the forest is contained within Brazil, with 60% of the rainforest, followed by Peru with 13%, Colombia with 10%, and with minor amounts in Venezuela, Ecuador, Bolivia, Guyana, Suriname and French Guiana. States or departments in four nations contain \"Amazonas\" in their names. The Amazon represents over half of the planet's remaining rainforests, and comprises the largest and most biodiverse tract of tropical rainforest in the world, with an estimated 390 billion individual trees divided into 16,000 species./\n  )\n\nKino.Layout.grid([question_input, context_input])\n```\n\n```elixir\nquestion = Kino.Input.read(question_input)\ncontext = Kino.Input.read(context_input)\nNx.Serving.run(serving, %{question: question, context: context})\n```","ref":"examples.html#question-answering"},{"type":"extras","title":"Final notes - Examples","doc":"The examples we covered should give you a good idea of what Bumblebee is about. We are excited about enabling easy access to the pre-trained, powerful deep learning models in Elixir. We are actively working on adding more models and high-level APIs, so stay tuned 🚀","ref":"examples.html#final-notes"},{"type":"extras","title":"Stable Diffusion","doc":"# Stable Diffusion\n\n```elixir\nMix.install([\n  {:bumblebee, \"~> 0.6.0\"},\n  {:nx, \"~> 0.9.0\"},\n  {:exla, \"~> 0.9.0\"},\n  {:kino, \"~> 0.14.0\"}\n])\n\nNx.global_default_backend({EXLA.Backend, client: :host})\n```","ref":"stable_diffusion.html"},{"type":"extras","title":"Introduction - Stable Diffusion","doc":"Stable Diffusion is a latent text-to-image diffusion model, primarily used to generate images based on a text prompt. Ever since it [became open-source](https://stability.ai/blog/stable-diffusion-public-release), the research, applications and tooling around it exploded. You can find a ton of resources and examples online, meanwhile let's see how to run Stable Diffusion using Bumblebee!\n\n<!-- livebook:{\"break_markdown\":true} -->\n\n> **Note:** Stable Diffusion is a very involved model, so the generation can take a long time if you run it on a CPU. Also, running on the GPU currently requires at least 5GiB of VRAM (or 3GiB with lower speed, see below).\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"stable_diffusion.html#introduction"},{"type":"extras","title":"Text to image - Stable Diffusion","doc":"Stable Diffusion is composed of several separate models and preprocessors, so we will load all of them.\n\n```elixir\nrepo_id = \"CompVis/stable-diffusion-v1-4\"\nopts = [params_variant: \"fp16\", type: :bf16, backend: EXLA.Backend]\n\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/clip-vit-large-patch14\"})\n{:ok, clip} = Bumblebee.load_model({:hf, repo_id, subdir: \"text_encoder\"}, opts)\n{:ok, unet} = Bumblebee.load_model({:hf, repo_id, subdir: \"unet\"}, opts)\n{:ok, vae} = Bumblebee.load_model({:hf, repo_id, subdir: \"vae\"}, [architecture: :decoder] ++ opts)\n{:ok, scheduler} = Bumblebee.load_scheduler({:hf, repo_id, subdir: \"scheduler\"})\n{:ok, featurizer} = Bumblebee.load_featurizer({:hf, repo_id, subdir: \"feature_extractor\"})\n{:ok, safety_checker} = Bumblebee.load_model({:hf, repo_id, subdir: \"safety_checker\"}, opts)\n\n:ok\n```\n\n> **Note:** some checkpoints, such as [runwayml/stable-diffusion-v1-5](https://huggingface.co/runwayml/stable-diffusion-v1-5), require a license agreement. In those cases, sign up on Hugging Face, accept the license on the repository page, generate an access token in [the settings](https://huggingface.co/settings/tokens) and add it to the repository specification via `:auth_token`. You can use Livebook secrets to pass the token securely.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nWith all the models loaded, we can now configure a serving implementation of the text-to-image task.\n\n```elixir\nserving =\n  Bumblebee.Diffusion.StableDiffusion.text_to_image(clip, unet, vae, tokenizer, scheduler,\n    num_steps: 20,\n    num_images_per_prompt: 1,\n    safety_checker: safety_checker,\n    safety_checker_featurizer: featurizer,\n    compile: [batch_size: 1, sequence_length: 60],\n    # Option 1\n    defn_options: [compiler: EXLA]\n    # Option 2 (reduces GPU usage, but runs noticeably slower)\n    # Also remove `backend: EXLA.Backend` from the loading options above\n    # defn_options: [compiler: EXLA, lazy_transfers: :always]\n  )\n\nKino.start_child({Nx.Serving, name: StableDiffusion, serving: serving})\n```\n\n```elixir\nprompt_input =\n  Kino.Input.text(\"Prompt\", default: \"numbat, forest, high quality, detailed, digital art\")\n\nnegative_prompt_input = Kino.Input.text(\"Negative Prompt\", default: \"darkness, rainy, foggy\")\n\nKino.Layout.grid([prompt_input, negative_prompt_input])\n```\n\nWe are ready to generate images!\n\n```elixir\nprompt = Kino.Input.read(prompt_input)\nnegative_prompt = Kino.Input.read(negative_prompt_input)\n\noutput =\n  Nx.Serving.batched_run(StableDiffusion, %{prompt: prompt, negative_prompt: negative_prompt})\n\nfor result <- output.results do\n  Kino.Image.new(result.image)\nend\n|> Kino.Layout.grid(columns: 2)\n```\n\nTo achieve a better quality you can increase the number of steps and images.","ref":"stable_diffusion.html#text-to-image"},{"type":"extras","title":"LLMs","doc":"# LLMs\n\n```elixir\nMix.install([\n  {:bumblebee, \"~> 0.6.0\"},\n  {:nx, \"~> 0.9.0\"},\n  {:exla, \"~> 0.9.0\"},\n  {:kino, \"~> 0.14.0\"}\n])\n\nNx.global_default_backend({EXLA.Backend, client: :host})\n```","ref":"llms.html"},{"type":"extras","title":"Introduction - LLMs","doc":"In this notebook we outline the general setup for running a Large Language Model (LLM).\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"llms.html#introduction"},{"type":"extras","title":"Llama 2 - LLMs","doc":"In this section we look at running [Meta's Llama](https://ai.meta.com/llama/) model, specifically Llama 2, one of the most powerful open source Large Language Models (LLMs).\n\n<!-- livebook:{\"break_markdown\":true} -->\n\n> **Note:** this is a very involved model, so the generation can take a long time if you run it on a CPU. Also, running on the GPU currently requires at least 16GiB of VRAM.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nIn order to load Llama 2, you need to ask for access on [meta-llama/Llama-2-7b-chat-hf](https://huggingface.co/meta-llama/Llama-2-7b-chat-hf). Once you are granted access, generate a [HuggingFace auth token](https://huggingface.co/settings/tokens) and put it in a `HF_TOKEN` Livebook secret.\n\nLet's load the model and create a serving for text generation:\n\n```elixir\nhf_token = System.fetch_env!(\"LB_HF_TOKEN\")\nrepo = {:hf, \"meta-llama/Llama-2-7b-chat-hf\", auth_token: hf_token}\n\n{:ok, model_info} = Bumblebee.load_model(repo, type: :bf16, backend: EXLA.Backend)\n{:ok, tokenizer} = Bumblebee.load_tokenizer(repo)\n{:ok, generation_config} = Bumblebee.load_generation_config(repo)\n\n:ok\n```\n\n```elixir\ngeneration_config =\n  Bumblebee.configure(generation_config,\n    max_new_tokens: 256,\n    strategy: %{type: :multinomial_sampling, top_p: 0.6}\n  )\n\nserving =\n  Bumblebee.Text.generation(model_info, tokenizer, generation_config,\n    compile: [batch_size: 1, sequence_length: 1028],\n    stream: true,\n    defn_options: [compiler: EXLA]\n  )\n\n# Should be supervised\nKino.start_child({Nx.Serving, name: Llama, serving: serving})\n```\n\nNote that we load the parameters directly onto the GPU with `Bumblebee.load_model(..., backend: EXLA.Backend)` and with `defn_options: [compiler: EXLA]` we tell the serving to compile and run computations on the GPU as well.\n\nWe adjust the generation config to use a non-deterministic generation strategy, so that the model is able to produce a slightly different output every time.\n\nAs for the other options, we specify `:compile` with fixed shapes, so that the model is compiled only once and inputs are always padded to match these shapes. We also enable `:stream` to receive text chunks as the generation is progressing.\n\n```elixir\nuser_input = Kino.Input.textarea(\"User prompt\", default: \"What is love?\")\n```\n\n```elixir\nuser = Kino.Input.read(user_input)\n\nprompt = \"\"\"\n[INST] < >\nYou are a helpful, respectful and honest assistant. Always answer as helpfully as possible, while being safe.  Your answers should not include any harmful, unethical, racist, sexist, toxic, dangerous, or illegal content. Please ensure that your responses are socially unbiased and positive in nature.\nIf a question does not make any sense, or is not factually coherent, explain why instead of answering something not correct. If you don't know the answer to a question, please don't share false information.\n< >\n#{user} [/INST] \\\n\"\"\"\n\nNx.Serving.batched_run(Llama, prompt) |> Enum.each(&IO.write/1)\n```\n\n<!-- livebook:{\"branch_parent_index\":0} -->","ref":"llms.html#llama-2"},{"type":"extras","title":"Mistral - LLMs","doc":"We can easily test other LLMs, we just need to change the repository and possibly adjust the prompt template. In this example we run the [Mistral](https://huggingface.co/mistralai/Mistral-7B-Instruct-v0.2) model.\n\nJust like Llama, Mistral now also requires users to request access to their models, so make sure you are granted access to the model, then generate a [HuggingFace auth token](https://huggingface.co/settings/tokens) and put it in a `HF_TOKEN` Livebook secret.\n\n```elixir\nhf_token = System.fetch_env!('LB_HF_TOKEN')\nrepo = {:hf, \"mistralai/Mistral-7B-Instruct-v0.2\", auth_token: hf_token}\n\n{:ok, model_info} = Bumblebee.load_model(repo, type: :bf16, backend: EXLA.Backend)\n{:ok, tokenizer} = Bumblebee.load_tokenizer(repo)\n{:ok, generation_config} = Bumblebee.load_generation_config(repo)\n\n:ok\n```\n\n```elixir\ngeneration_config =\n  Bumblebee.configure(generation_config,\n    max_new_tokens: 256,\n    strategy: %{type: :multinomial_sampling, top_p: 0.6}\n  )\n\nserving =\n  Bumblebee.Text.generation(model_info, tokenizer, generation_config,\n    compile: [batch_size: 1, sequence_length: 512],\n    stream: true,\n    defn_options: [compiler: EXLA]\n  )\n\n# Should be supervised\nKino.start_child({Nx.Serving, name: Mistral, serving: serving})\n```\n\n```elixir\nprompt = \"\"\"\n [INST] What is your favourite condiment? [/INST]\nWell, I'm quite partial to a good squeeze of fresh lemon juice. It adds just the right amount of zesty flavour to whatever I'm cooking up in the kitchen! \n[INST] Do you have mayonnaise recipes? [/INST]\\\n\"\"\"\n\nNx.Serving.batched_run(Mistral, prompt) |> Enum.each(&IO.write/1)\n```","ref":"llms.html#mistral"},{"type":"extras","title":"LLMs and RAG","doc":"# LLMs and RAG\n\n```elixir\nMix.install([\n  {:bumblebee, \"~> 0.6.0\"},\n  {:nx, \"~> 0.9.0\"},\n  {:exla, \"~> 0.9.0\"},\n  {:kino, \"~> 0.14.0\"},\n  {:hnswlib, \"~> 0.1.5\"},\n  {:req, \"~> 0.5.0\"}\n])\n\nNx.global_default_backend(EXLA.Backend)\n```","ref":"llms_rag.html"},{"type":"extras","title":"Introduction - LLMs and RAG","doc":"In this notebook we go through an example of in-memory Retrieval Augmented Generation (RAG).\n\nOn a high-level, we want to use a text document as the source of knowledge. When the user asks a question, we want to find relevant snippets from the essay and pass it alongside the question to the LLM. This way the LLM can provide a more accurate answer, based on the provided information.","ref":"llms_rag.html#introduction"},{"type":"extras","title":"Knowledge - LLMs and RAG","doc":"The first step is to download the text document, in this case we use an essay written by Paul Graham.\n\n```elixir\n%{body: text} =\n  Req.get!(\n    \"https://raw.githubusercontent.com/run-llama/llama_index/main/docs/docs/examples/data/paul_graham/paul_graham_essay.txt\"\n  )\n\nIO.puts(\"Document length: #{String.length(text)}\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nDocument length: 75014\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"llms_rag.html#knowledge"},{"type":"extras","title":"Generating embeddings - LLMs and RAG","doc":"There are many ways we could partition and retrieve snippets from a large text document. In this example we will use embedding-based lookup. That is, we will split the text into smaller chunks, compute an embedding (the chunk meaning compressed into a vector) and create an in-memory index for efficient lookup. In real world problems, you may want to explore other retrieval methods, such as reranking or BM25.\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nFirst, let's split the text into chunks, 1024 characters each.\n\n```elixir\nchunks =\n  text\n  |> String.codepoints()\n  |> Enum.chunk_every(1024)\n  |> Enum.map(&Enum.join/1)\n\nlength(chunks)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n74\n```\n\nTo generate our embeddings we will use the [gte-small](https://huggingface.co/thenlper/gte-small) model. Let's download it and start a serving.\n\n```elixir\nrepo = {:hf, \"thenlper/gte-small\"}\n\n{:ok, model_info} = Bumblebee.load_model(repo)\n{:ok, tokenizer} = Bumblebee.load_tokenizer(repo)\n\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n10:45:24.653 [info] successful NUMA node read from SysFS had negative value (-1), but there must be at least one NUMA node, so returning NUMA node zero. See more at https://github.com/torvalds/linux/blob/v6.0/Documentation/ABI/testing/sysfs-bus-pci#L344-L355\n\n10:45:24.653 [info] XLA service 0x7fe2640185e0 initialized for platform CUDA (this does not guarantee that XLA will be used). Devices:\n\n10:45:24.653 [info]   StreamExecutor device (0): NVIDIA A100-PCIE-40GB, Compute Capability 8.0\n\n10:45:24.653 [info] Using BFC allocator.\n\n10:45:24.654 [info] XLA backend allocating 38068951449 bytes on device 0 for BFCAllocator.\n\n10:45:25.724 [info] Loaded cuDNN version 8900\n\n10:45:25.741 [info] Using nvlink for parallel linking\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n```elixir\nserving =\n  Bumblebee.Text.TextEmbedding.text_embedding(model_info, tokenizer,\n    compile: [batch_size: 64, sequence_length: 512],\n    defn_options: [compiler: EXLA],\n    output_attribute: :hidden_state,\n    output_pool: :mean_pooling\n  )\n\nKino.start_child({Nx.Serving, serving: serving, name: GteServing})\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n10:45:40.784 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot_5', 416 bytes spill stores, 380 bytes spill loads\n\n\n10:45:40.941 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot_4', 16 bytes spill stores, 8 bytes spill loads\n\n\n10:45:41.043 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot_5', 216 bytes spill stores, 216 bytes spill loads\n\n\n10:45:41.911 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot_4', 116 bytes spill stores, 116 bytes spill loads\n\n\n10:45:42.297 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot', 224 bytes spill stores, 224 bytes spill loads\n\n\n10:45:43.685 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot', 116 bytes spill stores, 116 bytes spill loads\n\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok, #PID<0.337.0>}\n```\n\nWe are ready to generate embeddings for the chunks. We can pass the whole list to `Nx.Serving.batched_run`, it is going to split them into batches for us automatically!\n\n```elixir\nresults = Nx.Serving.batched_run(GteServing, chunks)\nchunk_embeddings = for result <- results, do: result.embedding\n\nList.first(chunk_embeddings)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Nx.Tensor \n```","ref":"llms_rag.html#generating-embeddings"},{"type":"extras","title":"Embeddings index and retrieval - LLMs and RAG","doc":"Having all the embeddings at hand, we will now create an index using [hnswlib](https://github.com/elixir-nx/hnswlib). With the index, we will be able to quickly retrieve embeddings matching a query. The hnswlib library uses Approximate Nearest Neighbor (ANN) search underneath.\n\n```elixir\n{:ok, index} = HNSWLib.Index.new(:cosine, 384, 1_000_000)\n\nfor embedding <- chunk_embeddings do\n  HNSWLib.Index.add_items(index, embedding)\nend\n\nHNSWLib.Index.get_current_count(index)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok, 74}\n```\n\nNow, given a textual query, we first need to compute its embedding using the same embedding model. Once we have the embedding, we do a similarity lookup and get top 4 matching results.\n\n```elixir\nquery = \"What were the two main things the author worked on before college?\"\n\n%{embedding: embedding} = Nx.Serving.batched_run(GteServing, query)\n\n{:ok, labels, dist} = HNSWLib.Index.knn_query(index, embedding, k: 4)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok,\n #Nx.Tensor \n   [\n     [0, 10, 11, 54]\n   ]\n >,\n #Nx.Tensor \n   [\n     [0.11476433277130127, 0.14768105745315552, 0.15568876266479492, 0.15724539756774902]\n   ]\n >}\n```\n\nThe lookup conveniently returns indices, so we can get their corresponding chunks and join into a context text.\n\n```elixir\n# We can see some overlapping in our chunks\ncontext =\n  labels\n  |> Nx.to_flat_list()\n  |> Enum.sort()\n  |> Enum.map(fn idx -> \"[...] \" <> Enum.at(chunks, idx) <> \" [...]\" end)\n  |> Enum.join(\"\\n\\n\")\n\nIO.puts(context)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[...]\n\nWhat I Worked On\n\nFebruary 2021\n\nBefore college the two main things I worked on, outside of school, were writing and programming. I didn't write essays. I wrote what beginning writers were supposed to write then, and probably still are: short stories. My stories were awful. They had hardly any plot, just characters with strong feelings, which I imagined made them deep.\n\nThe first programs I tried writing were on the IBM 1401 that our school district used for what was then called \"data processing.\" This was in 9th grade, so I was 13 or 14. The school district's 1401 happened to be in the basement of our junior high school, and my friend Rich Draves and I got permission to use it. It was like a mini Bond villain's lair down there, with all these alien-looking machines — CPU, disk drives, printer, card reader — sitting up on a raised floor under bright fluorescent lights.\n\nThe language we used was an early version of Fortran. You had to type programs on punch cards, then stack them in the card reader and press [...]\n\n[...] g art classes at Harvard. Grad students could take classes in any department, and my advisor, Tom Cheatham, was very easy going. If he even knew about the strange classes I was taking, he never said anything.\n\nSo now I was in a PhD program in computer science, yet planning to be an artist, yet also genuinely in love with Lisp hacking and working away at On Lisp. In other words, like many a grad student, I was working energetically on multiple projects that were not my thesis.\n\nI didn't see a way out of this situation. I didn't want to drop out of grad school, but how else was I going to get out? I remember when my friend Robert Morris got kicked out of Cornell for writing the internet worm of 1988, I was envious that he'd found such a spectacular way to get out of grad school.\n\nThen one day in April 1990 a crack appeared in the wall. I ran into professor Cheatham and he asked if I was far enough along to graduate that June. I didn't have a word of my dissertation written, but in what must have been the quicke [...]\n\n[...] st bit of thinking in my life, I decided to take a shot at writing one in the 5 weeks or so that remained before the deadline, reusing parts of On Lisp where I could, and I was able to respond, with no perceptible delay \"Yes, I think so. I'll give you something to read in a few days.\"\n\nI picked applications of continuations as the topic. In retrospect I should have written about macros and embedded languages. There's a whole world there that's barely been explored. But all I wanted was to get out of grad school, and my rapidly written dissertation sufficed, just barely.\n\nMeanwhile I was applying to art schools. I applied to two: RISD in the US, and the Accademia di Belli Arti in Florence, which, because it was the oldest art school, I imagined would be good. RISD accepted me, and I never heard back from the Accademia, so off to Providence I went.\n\nI'd applied for the BFA program at RISD, which meant in effect that I had to go to college again. This was not as strange as it sounds, because I was only 25, and a [...]\n\n[...] b. I was going to do three things: hack, write essays, and work on YC. As YC grew, and I grew more excited about it, it started to take up a lot more than a third of my attention. But for the first few years I was still able to work on other things.\n\nIn the summer of 2006, Robert and I started working on a new version of Arc. This one was reasonably fast, because it was compiled into Scheme. To test this new Arc, I wrote Hacker News in it. It was originally meant to be a news aggregator for startup founders and was called Startup News, but after a few months I got tired of reading about nothing but startups. Plus it wasn't startup founders we wanted to reach. It was future startup founders. So I changed the name to Hacker News and the topic to whatever engaged one's intellectual curiosity.\n\nHN was no doubt good for YC, but it was also by far the biggest source of stress for me. If all I'd had to do was select and help founders, life would have been so easy. And that implies that HN was a mistake. Surely the b [...]\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"llms_rag.html#embeddings-index-and-retrieval"},{"type":"extras","title":"Generating an answer - LLMs and RAG","doc":"We have our context, the last thing left to do is have a LLM answer the question. In this example we will use the [Mistral](https://huggingface.co/mistralai/Mistral-7B-Instruct-v0.2) model.\n\nFor more details on running an LLM, see the [LLMs](./llms.livemd) notebook.\n\n```elixir\nrepo = {:hf, \"mistralai/Mistral-7B-Instruct-v0.2\"}\n\n{:ok, model_info} = Bumblebee.load_model(repo, type: :bf16)\n{:ok, tokenizer} = Bumblebee.load_tokenizer(repo)\n{:ok, generation_config} = Bumblebee.load_generation_config(repo)\n\ngeneration_config = Bumblebee.configure(generation_config, max_new_tokens: 100)\n\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```\n\n```elixir\nserving =\n  Bumblebee.Text.generation(model_info, tokenizer, generation_config,\n    compile: [batch_size: 1, sequence_length: 6000],\n    defn_options: [compiler: EXLA]\n  )\n\nKino.start_child({Nx.Serving, name: MistralServing, serving: serving})\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n10:46:17.065 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot_516', 4 bytes spill stores, 4 bytes spill loads\n\n\n10:46:17.681 [info] ptxas warning : Registers are spilled to local memory in function 'triton_gemm_dot_516', 104 bytes spill stores, 152 bytes spill loads\n\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok, #PID<0.347.0>}\n```\n\n```elixir\nprompt =\n  \"\"\"\n  Context information is below.\n  ---------------------\n  #{context}\n  ---------------------\n  Given the context information and not prior knowledge, answer the query.\n  Query: #{query}\n  Answer:\n  \"\"\"\n\nresults = Nx.Serving.batched_run(MistralServing, prompt)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{\n  results: [\n    %{\n      text: \"1. Writing: The author wrote short stories before college, which he describes as having hardly any plot and strong feelings.\\n2. Programming: The author started programming on an IBM 1401 computer in 9th grade, using an early version of Fortran to write programs.\",\n      token_summary: %{input: 1099, output: 61, padding: 4901}\n    }\n  ]\n}\n```\n\nAnd here we have our answer!\n\n<!-- livebook:{\"break_markdown\":true} -->\n\nFor additional context you can also visit the [Mistral docs](https://docs.mistral.ai/guides/rag/) that go through a similar example.","ref":"llms_rag.html#generating-an-answer"},{"type":"extras","title":"Fine-tuning","doc":"<!-- livebook:{\"persist_outputs\":true} -->\n\n# Fine-tuning\n\n```elixir\nMix.install([\n  {:bumblebee, \"~> 0.5.0\"},\n  {:nx, \"~> 0.7.0\"},\n  {:exla, \"~> 0.7.0\"},\n  {:axon, \"~> 0.6.1\"},\n  {:explorer, \"~> 0.7.0\"}\n])\n\nNx.default_backend(EXLA.Backend)\n```","ref":"fine_tuning.html"},{"type":"extras","title":"Introduction - Fine-tuning","doc":"Fine-tuning is the process of specializing the parameters in a pre-trained model to a specific task. Large-language models such as BERT train on a generic language-modeling task which makes them powerful at extracting features from text. Despite their power, you often still need to train them on a downstream task.\n\nThis example demonstrates how to use Bumblebee and Axon to fine-tune a pre-trained Bert model to classify Yelp reviews into classes of 1-5 stars. This example is based on [Fine-tune a pretrained model](https://huggingface.co/docs/transformers/training) from Huggingface.\n\nYou'll need to first download the Yelp Reviews dataset ([download](https://s3.amazonaws.com/fast-ai-nlp/yelp_review_full_csv.tgz)).\n\nOnce downloaded, extract it to a directory of your choosing and you're ready to go!","ref":"fine_tuning.html#introduction"},{"type":"extras","title":"Load a model - Fine-tuning","doc":"We'll start by loading a pre-trained model and tokenizer; however, we'll initialize the model to have an untrained sequence classification head.\n\nReviews in the dataset can have anywhere from 1 to 5 stars, which means we need 5 labels in our sequence classification head. We can change the default configuration by loading the model spec with `Bumblebee.load_spec/2` and making changes to spec properties with `Bumblebee.configure/2`.\n\nThe pre-trained model we'll be using is `bert-base-cased`; however, you can use any of the supported models from the HuggingFace Hub.\n\n```elixir\n{:ok, spec} =\n  Bumblebee.load_spec({:hf, \"google-bert/bert-base-cased\"},\n    architecture: :for_sequence_classification\n  )\n\nspec = Bumblebee.configure(spec, num_labels: 5)\n\n{:ok, model} = Bumblebee.load_model({:hf, \"google-bert/bert-base-cased\"}, spec: spec)\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"google-bert/bert-base-cased\"})\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n14:41:33.314 [info] TfrtCpuClient created.\n\n14:41:33.820 [debug] the following parameters were missing:\n\n  * sequence_classification_head.output.kernel\n  * sequence_classification_head.output.bias\n\n\n14:41:33.820 [debug] the following PyTorch parameters were unused:\n\n  * cls.predictions.bias\n  * cls.predictions.decoder.weight\n  * cls.predictions.transform.LayerNorm.beta\n  * cls.predictions.transform.LayerNorm.gamma\n  * cls.predictions.transform.dense.bias\n  * cls.predictions.transform.dense.weight\n  * cls.seq_relationship.bias\n  * cls.seq_relationship.weight\n\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:ok,\n %Bumblebee.Text.BertTokenizer{\n   tokenizer: #Tokenizers.Tokenizer<[\n     vocab_size: 28996,\n     continuing_subword_prefix: \"##\",\n     max_input_chars_per_word: 100,\n     model_type: \"bpe\",\n     unk_token: \"[UNK]\"\n   ]>,\n   special_tokens: %{cls: \"[CLS]\", mask: \"[MASK]\", pad: \"[PAD]\", sep: \"[SEP]\", unk: \"[UNK]\"}\n }}\n```","ref":"fine_tuning.html#load-a-model"},{"type":"extras","title":"Prepare a dataset - Fine-tuning","doc":"With the models downloaded and ready to go, you need to prepare the dataset. The downloaded dataset is a CSV. You can use the `Explorer` library to quickly load the CSV into a DataFrame.\n\nOnce the data is loaded, you need to convert raw text to tokens and the raw labels to tensors. Additionally, you need to convert the DataFrame to a Stream consisting of tuples: `{tokenized, labels}` - that is the form expected by Axon's training API.\n\n```elixir\ndefmodule Yelp do\n  def load(path, tokenizer, opts \\\\ []) do\n    path\n    |> Explorer.DataFrame.from_csv!(header: false)\n    |> Explorer.DataFrame.rename([\"label\", \"text\"])\n    |> stream()\n    |> tokenize_and_batch(tokenizer, opts[:batch_size], opts[:sequence_length])\n  end\n\n  def stream(df) do\n    xs = df[\"text\"]\n    ys = df[\"label\"]\n\n    xs\n    |> Explorer.Series.to_enum()\n    |> Stream.zip(Explorer.Series.to_enum(ys))\n  end\n\n  def tokenize_and_batch(stream, tokenizer, batch_size, sequence_length) do\n    tokenizer = Bumblebee.configure(tokenizer, length: sequence_length)\n\n    stream\n    |> Stream.chunk_every(batch_size)\n    |> Stream.map(fn batch ->\n      {text, labels} = Enum.unzip(batch)\n      tokenized = Bumblebee.apply_tokenizer(tokenizer, text)\n      {tokenized, Nx.stack(labels)}\n    end)\n  end\nend\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n{:module, Yelp, <<70, 79, 82, 49, 0, 0, 13, ...>>, {:tokenize_and_batch, 4}}\n```\n\nNow you can use the `Yelp.load/2` function to load a training set and a testing set:\n\n```elixir\nbatch_size = 32\nsequence_length = 64\n\ntrain_data =\n  Yelp.load(\"~/yelp/yelp_review_full_csv/train.csv\", tokenizer,\n    batch_size: batch_size,\n    sequence_length: sequence_length\n  )\n\ntest_data =\n  Yelp.load(\"~/yelp/yelp_review_full_csv/test.csv\", tokenizer,\n    batch_size: batch_size,\n    sequence_length: sequence_length\n  )\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Stream<[\n  enum: #Stream<[\n    enum: #Function<73.124013645/2 in Stream.zip_with/2>,\n    funs: [#Function<3.124013645/1 in Stream.chunk_while/4>]\n  ]>,\n  funs: [#Function<48.124013645/1 in Stream.map/2>]\n]>\n```\n\nYou can see what a single batch looks like by grabbing 1 from the stream:\n\n```elixir\nEnum.take(train_data, 1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n[\n  {%{\n     \"attention_mask\" => #Nx.Tensor \n       [\n         [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, ...],\n         ...\n       ]\n     >,\n     \"input_ids\" => #Nx.Tensor \n       [\n         [101, 173, 1197, 119, 2284, 2953, 3272, 1917, 178, 1440, 1111, 1107, 170, 1704, 22351, 119, 1119, 112, 188, 3505, 1105, 3123, 1106, 2037, 1106, 1443, 1217, 10063, 4404, 132, 1119, 112, 188, 1579, 1113, 1159, 1107, 3195, 1117, 4420, 132, 1119, 112, 188, 6559, 1114, ...],\n         ...\n       ]\n     >,\n     \"token_type_ids\" => #Nx.Tensor \n       [\n         [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, ...],\n         ...\n       ]\n     >\n   },\n   #Nx.Tensor \n     [5, 2, 4, 4, 1, 5, 5, 1, 2, 3, 1, 1, 4, 2, 5, 5, 5, 5, 5, 5, 4, 3, 2, 5, 1, 1, 1, 2, 2, 4, 2, 2]\n   >}\n]\n```\n\nThe dataset is rather large for CPU training, so we'll just train a small subset (250 training batches and 50 testing batches):\n\n```elixir\ntrain_data = Enum.take(train_data, 250)\ntest_data = Enum.take(test_data, 50)\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"fine_tuning.html#prepare-a-dataset"},{"type":"extras","title":"Train the model - Fine-tuning","doc":"Now we can go about training the model! First, we need to extract the Axon model and parameters from the Bumblebee model map:\n\n```elixir\n%{model: model, params: params} = model\n\nmodel\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Axon \n```\n\nThe Axon model actually outputs a map with `:logits`, `:hidden_states`, and `:attentions`. You can see this by using `Axon.get_output_shape/2` with an input. This method symbolically executes the graph and gets the resulting shapes:\n\n```elixir\n[{input, _}] = Enum.take(train_data, 1)\nAxon.get_output_shape(model, input)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{attentions: #Axon.None<...>, hidden_states: #Axon.None<...>, logits: {32, 5}}\n```\n\nFor training, we only care about the `:logits` key, so we'll extract that by attaching an `Axon.nx/2` layer to the model:\n\n```elixir\nlogits_model = Axon.nx(model, & &1.logits)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Axon \n```\n\nNow we can declare our training loop. You can construct Axon training loops using the `Axon.Loop.trainer/3` factory method with a model, loss function, and optimizer. We'll also adjust the log-settings to more frequently log metrics to standard out:\n\n```elixir\nloss =\n  &Axon.Losses.categorical_cross_entropy(&1, &2,\n    reduction: :mean,\n    from_logits: true,\n    sparse: true\n  )\n\noptimizer = Polaris.Optimizers.adam(learning_rate: 5.0e-5)\n\nloop = Axon.Loop.trainer(logits_model, loss, optimizer, log: 1)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Axon.Loop ,\n     #Function<41.3316493/2 in :erl_eval.expr/6>}\n  },\n  handlers: %{\n    completed: [],\n    epoch_completed: [\n      {#Function<27.14409478/1 in Axon.Loop.log/3>,\n       #Function<6.14409478/2 in Axon.Loop.build_filter_fn/1>}\n    ],\n    epoch_halted: [],\n    epoch_started: [],\n    halted: [],\n    iteration_completed: [\n      {#Function<27.14409478/1 in Axon.Loop.log/3>,\n       #Function<64.14409478/2 in Axon.Loop.build_filter_fn/1>}\n    ],\n    iteration_started: [],\n    started: []\n  },\n  ...\n>\n```\n\nThe call to trainer just returns a data structure. In Axon, we manipulate this data structure to control different parts of the loop. For example, you can attach metrics:\n\n```elixir\naccuracy = &Axon.Metrics.accuracy(&1, &2, from_logits: true, sparse: true)\n\nloop = Axon.Loop.metric(loop, accuracy, \"accuracy\")\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Axon.Loop ,\n     #Function<41.3316493/2 in :erl_eval.expr/6>},\n    \"loss\" => {#Function<11.3813108/3 in Axon.Metrics.running_average/1>,\n     #Function<41.3316493/2 in :erl_eval.expr/6>}\n  },\n  handlers: %{\n    completed: [],\n    epoch_completed: [\n      {#Function<27.14409478/1 in Axon.Loop.log/3>,\n       #Function<6.14409478/2 in Axon.Loop.build_filter_fn/1>}\n    ],\n    epoch_halted: [],\n    epoch_started: [],\n    halted: [],\n    iteration_completed: [\n      {#Function<27.14409478/1 in Axon.Loop.log/3>,\n       #Function<64.14409478/2 in Axon.Loop.build_filter_fn/1>}\n    ],\n    iteration_started: [],\n    started: []\n  },\n  ...\n>\n```\n\nAnd you can attach event handlers to do certain things, such as serialize the loop state at regular intervals so you don't lose your progress:\n\n```elixir\nloop = Axon.Loop.checkpoint(loop, event: :epoch_completed)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n#Axon.Loop ,\n     #Function<41.3316493/2 in :erl_eval.expr/6>},\n    \"loss\" => {#Function<11.3813108/3 in Axon.Metrics.running_average/1>,\n     #Function<41.3316493/2 in :erl_eval.expr/6>}\n  },\n  handlers: %{\n    completed: [],\n    epoch_completed: [\n      {#Function<17.14409478/1 in Axon.Loop.checkpoint/2>,\n       #Function<6.14409478/2 in Axon.Loop.build_filter_fn/1>},\n      {#Function<27.14409478/1 in Axon.Loop.log/3>,\n       #Function<6.14409478/2 in Axon.Loop.build_filter_fn/1>}\n    ],\n    epoch_halted: [],\n    epoch_started: [],\n    halted: [],\n    iteration_completed: [\n      {#Function<27.14409478/1 in Axon.Loop.log/3>,\n       #Function<64.14409478/2 in Axon.Loop.build_filter_fn/1>}\n    ],\n    iteration_started: [],\n    started: []\n  },\n  ...\n>\n```\n\nTo run the loop, you just need to call `Axon.Loop.run/4`. `Axon.Loop.run/4` takes a loop, input data, and any initial state (in this case initial parameters). You can kind of think of `Axon.Loop.run/4` as an `Enum.reduce/3`. It takes data, an accumulator, and a function - which map to `Loop.run/4` input data, initial state, and the actual loop data structure.\n\nYou'll commonly see loops written out in long chains using Elixir's `|>` operator, like this:\n\n```elixir\ntrained_model_state =\n  logits_model\n  |> Axon.Loop.trainer(loss, optimizer, log: 1)\n  |> Axon.Loop.metric(accuracy, \"accuracy\")\n  |> Axon.Loop.checkpoint(event: :epoch_completed)\n  |> Axon.Loop.run(train_data, params, epochs: 3, compiler: EXLA, strict?: false)\n\n:ok\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n\n02:46:02.170 [debug] Forwarding options: [compiler: EXLA] to JIT compiler\nEpoch: 0, Batch: 249, accuracy: 0.3462500 loss: 1.2216607\nEpoch: 1, Batch: 249, accuracy: 0.5186251 loss: 1.0558304\nEpoch: 2, Batch: 249, accuracy: 0.6236249 loss: 0.9317472\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n:ok\n```","ref":"fine_tuning.html#train-the-model"},{"type":"extras","title":"Evaluating the model - Fine-tuning","doc":"The training loop returns the final model state after training over your dataset for the given number of epochs. Axon uses the same `Axon.Loop` API to create evaluation loops as well. You can create one with the `Axon.Loop.evaluator/1` factory, instrument it with metrics, and run it on your data with your trained model state:\n\n```elixir\nlogits_model\n|> Axon.Loop.evaluator()\n|> Axon.Loop.metric(accuracy, \"accuracy\")\n|> Axon.Loop.run(test_data, trained_model_state, compiler: EXLA)\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\nBatch: 49, accuracy: 0.3675000\n```\n\n<!-- livebook:{\"output\":true} -->\n\n```\n%{\n  0 => %{\n    \"accuracy\" => #Nx.Tensor \n      0.36750003695487976\n    >\n  }\n}\n```","ref":"fine_tuning.html#evaluating-the-model"},{"type":"extras","title":"Phoenix LiveView examples","doc":"# Phoenix LiveView examples\n\nThe [examples/phoenix](https://github.com/elixir-nx/bumblebee/tree/main/examples/phoenix) directory contains minimal, single-file LiveView applications, which showcase how to integrate Bumblebee as part of the application.","ref":"readme.html"},{"type":"extras","title":"Running - Phoenix LiveView examples","doc":"Each example is fully contained in a single script (including all the dependencies), so you can run it as:\n\n```shell\nelixir example.exs\n```","ref":"readme.html#running"},{"type":"extras","title":"How - Phoenix LiveView examples","doc":"Integrating a Bumblebee task is straightforward and boils down to two steps:\n\n  1. On application startup we load the necessary models and preprocessors, build an instance of `Nx.Serving` and start it under the application supervision tree.\n\n  2. Whenever we want to make a prediction for a user input, we use `Nx.Serving.batched_run/2` with said input, usually in a `Task` to avoid blocking other interactions. It is important to use `Nx.Serving.batched_run/2` as it automatically batches requests from concurrent users and distributes the results back to them transparently.","ref":"readme.html#how"},{"type":"extras","title":"Tips - Phoenix LiveView examples","doc":"","ref":"readme.html#tips"},{"type":"extras","title":"Deployment considerations - Phoenix LiveView examples","doc":"The examples in this directory download the model data directly from HuggingFace the first time the application starts. To avoid downloading the model data during deployments, you have two options.\n\n#### 1. Explicit local versioning\n\nOne option is to version those files alongside your repository (perhaps using [Git LFS](https://git-lfs.github.com/)) or use an object storage service. Then, you'd fetch those files into a local directory as part of development/deployment.\n\nIn such scenarios, you must change the calls from `Bumblebee.load_xyz({:hf, \"microsoft/resnet\"})` to `Bumblebee.load_xyz({:local, \"/path/to/model\"})` and so on.\n\n#### 2. Cached from Hugging Face\n\nBumblebee can download and cache data from both public and private Hugging Face repositories. You can control the cache directory by setting `BUMBLEBEE_CACHE_DIR`. Therefore, one option during deployment is to set the `BUMBLEBEE_CACHE_DIR` to a directory within your application. If using Docker, you must then include said directory in your application and make sure that `BUMBLEBEE_CACHE_DIR` points to it.\n\nWhen using Docker multi-stage build, you can populate the cache in the build stage. For example, if your application has a module named `MyApp.Servings` with several Bumblebee models, you could define a function called `load_all/0` that loads all relevant information:\n\n```elixir\n  def load_all do\n    Bumblebee.load_xyz({:hf, \"microsoft/resnet\"})\n    Bumblebee.load_xyz({:hf, \"foo/bar/baz\"})\n  end\n```\n\nThen your Dockerfile calls either `RUN mix eval 'MyApp.Servings.load_all()'` in your Mix project root or `RUN bin/my_app eval 'MyApp.Servings.load_all()'` in your release root to write all relevant information to the cache directory. The last step is to copy the contents of `BUMBLEBEE_CACHE_DIR` to the final image.\n\nIt is also recommended to set `BUMBLEBEE_OFFLINE` to `true` in the final image to make sure the models are always loaded from the cache.","ref":"readme.html#deployment-considerations"},{"type":"extras","title":"Configuring Nx - Phoenix LiveView examples","doc":"We currently recommend [EXLA](https://hexdocs.pm/exla/EXLA.html) to compile the numerical computations.\n\nThere are generally two types of computations you run with Nx:\n\n  1. One-off operations, such as calling `Nx.sum(data)`, oftentimes used when preparing data for the model. Those operations are delegated to the default backend compiled one-by-one.\n\n  2. Large defn computations, such as running a neural network model. Those are usually compiled as a whole explicitly using the configured compiler.\n\nEXLA allows only a single computation per device to run at the same time, so if a GPU is available we want it to only run computations falling under the 2. type.\n\nTo achieve that, you can configure your default backend (used for 1.) to always use the CPU:\n\n```elixir\nconfig :nx, :default_backend, {EXLA.Backend, client: :host}\n```\n\nThen, for any expensive computations you can use [`Nx.Defn.compile/3`](https://hexdocs.pm/nx/Nx.Defn.html#compile/3) (or [`Axon.compile/4`](https://hexdocs.pm/axon/Axon.html#compile/4)) passing `compiler: EXLA` as an option. When you use a Bumblebee serving the compilation is handled for you, just make sure to pass `:compile` and `defn_options: [compiler: EXLA]` when creating the serving.\n\nThere's a final important detail related to parameters. With the above configuration, a model will run on the GPU, however parameters will be loaded onto the CPU (due to the default backend), so they will need to be copied onto the GPU every time. To avoid that, you can load the parameters onto the GPU directly using `Bumblebee.load_model(..., backend: EXLA.Backend)`.\n\nWhen building the Bumblebee serving, make sure to specify the compiler and `:compile` shapes, so that the computation is compiled upfront when the serving boots.\n\n```elixir\nserving =\n  Bumblebee.Text.text_embedding(model_info, tokenizer,\n    compile: [batch_size: 1, sequence_length: 512],\n    defn_options: [compiler: EXLA]\n  )\n```","ref":"readme.html#configuring-nx"},{"type":"extras","title":"User images - Phoenix LiveView examples","doc":"When working with user-given images, the most trivial approach would be to just upload an image as is, in a format like PNG or JPEG. However, this approach has two downsides:\n\n  1. In most cases full-resolution images are not necessary, because Neural Networks work on much smaller inputs. This means that a lot of data is unnecessarily uploaded over the network and that the server needs to do more work downsizing a potentially large image.\n\n  2. Decoding an image format like PNG or JPEG requires an additional package and again, puts more work on the server.\n\nBoth of these downsides can be avoided by moving all the work to the client. Specifically, when the user selects an image, we can resize it to a much smaller version and decode to pixel values right away. Both of these steps are fairly straightforward using the Canvas API.\n\nFor an example implementation of this technique see the [image classification example](https://github.com/elixir-nx/bumblebee/tree/main/examples/phoenix/image_classification.exs).","ref":"readme.html#user-images"},{"type":"extras","title":"User audio - Phoenix LiveView examples","doc":"The points made about images above are relevant to user-given audio as well. In fact, decoding audio files on the server requires ffmpeg to be installed system-wide. However, we can do all preprocessing on the client and send raw PCM data with a single channel to the server.\n\nFor an example implementation of this technique see the [speech-to-text example](https://github.com/elixir-nx/bumblebee/tree/main/examples/phoenix/speech_to_text.exs).\n\nIf you are interested in real-time streaming, look at the [Membrane Framework](https://github.com/membraneframework/membrane_core).","ref":"readme.html#user-audio"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}